package com.grendelscan.categorizers.queue;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.*;
import java.util.Set;
import java.util.concurrent.LinkedBlockingQueue;

import com.grendelscan.categorizers.*;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.Scan;
import com.grendelscan.scan.abstractQueue.AbstractScanQueue;
import com.grendelscan.tests.testJobs.TestJob;
import com.grendelscan.tests.testModules.TestModule;
import com.grendelscan.tests.testModules.collections.MasterTestModuleCollection;
import com.grendelscan.utils.Debug;

public class CategorizerQueue extends AbstractScanQueue
{
	private LinkedBlockingQueue<AbstractHttpTransaction> queue;
	
	private Scan scan;
	private List<TransactionCategorizer> transactionCategorizers;
	private boolean running = true;
	
	/** Creates a new instance of requesterQueue */
	public CategorizerQueue(Scan scan)
	{
		queue = new LinkedBlockingQueue<AbstractHttpTransaction>();
		this.scan = scan;
		transactionCategorizers = scan.getCategorizers().getTransactionCategorizers();
	}
	
	public void addTransaction(AbstractHttpTransaction transaction)
	{
		if (!queueLocked)
		{
			if (!transaction.isSuccessfullExecution())
			{
				Debug.errDebug("Attempt to categorize a transaction without a response");
			}
			else
			{
				queue.add(transaction);
			}
		}
	}
	
	public void addTransactions(Collection<AbstractHttpTransaction> transactions)
	{
		for (AbstractHttpTransaction transaction: transactions)
		{
			addTransaction(transaction);
		}
	}
	
	/**
	 * Normally, this should only be called by the queue thread. You probably want "addTransaction". 
	 * Intentionally default access level.
	 * @param transaction
	 */
	void categorizeTransaction(AbstractHttpTransaction transaction)
	{
		Map<TestModule, Set<TestJob>> jobs = new HashMap<TestModule, Set<TestJob>>();
		if (transaction.getResponseWrapper() != null)
		{
			for (TransactionCategorizer categorizer: transactionCategorizers)
			{
				if (!scan.isRunning() || !running)
				{
					return;
				}
				if (((Categorizer) categorizer).hasModules())
				{
					Map<TestModule, Set<TestJob>> tempJobs = categorizer.analyzeTransaction(transaction);
					joinJobLists(jobs, tempJobs);
				}
			}
		}
		submitJobs(jobs);
	}
	
	public AbstractHttpTransaction getNext()
	{
		AbstractHttpTransaction transaction = null;
		if (!queueLocked)
		{
			try
			{
				transaction = queue.take();
			}
			catch (InterruptedException e)
			{
				Debug.debug(e.toString());
				transaction = null;
			}
		}
		return transaction;
	}
	
	public void joinJobLists(Map<TestModule, Set<TestJob>> target, Map<TestModule, Set<TestJob>> source)
	{
		for (TestModule module: source.keySet())
		{
			if (target.containsKey(module))
			{
				Set<TestJob> tests = target.get(module);
				for (TestJob job: source.get(module))
				{
					tests.add(job);
				}
			}
			else
			{
				target.put(module, source.get(module));
			}
		}
	}
	
	public void submitJobs(Map<TestModule, Set<TestJob>> tests)
	{
		MasterTestModuleCollection moduleCollection = scan.getScanSettings().getMasterTestModuleCollection();
		// Setup dependencies
		Set<TestJob> allJobs = new HashSet<TestJob>();
		for (TestModule module: tests.keySet())
		{
			allJobs.addAll(tests.get(module));
			for (TestJob test: tests.get(module))
			{
				for (int prereq: module.getPrerequisites())
				{
					if (!scan.isRunning())
					{
						return;
					}
					TestModule prereqModule = moduleCollection.getTestModule(prereq);
					addDependencies(test, tests.get(prereqModule));
				}
				
				for (int prereq: module.getSoftPrerequisites())
				{
					if (!scan.isRunning())
					{
						return;
					}
					TestModule prereqModule = moduleCollection.getTestModule(prereq);
					if (tests.containsKey(prereqModule))
					{
						addDependencies(test, tests.get(prereqModule));
					}
				}
			}
		}
		
		scan.getTesterQueue().addTests(allJobs);
	}
	
	private void addDependencies(TestJob target, Set<TestJob> dependencies)
	{
		if (dependencies != null)
		{
			for (TestJob dep: dependencies)
			{
				target.addDependency(dep);
			}
		}
	}
	
	@Override
	protected Queue<AbstractHttpTransaction> getQueue()
	{
		return queue;
	}

	public boolean isRunning()
    {
    	return running;
    }

	public void setRunning(boolean running)
    {
    	this.running = running;
    }
	
}
