package com.grendelscan.fuzzing;

import org.apache.http.HttpRequest;
import java.util.*;

import com.grendelscan.GUI.http.HttpFormatException;
import com.grendelscan.GUI.http.interception.InterceptFilter;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.UnvalidatedHttpEntityRequest;
import com.grendelscan.requester.http.UnvalidatedHttpRequest;
import com.grendelscan.requester.http.transactions.UnvalidatedHttpTransaction;
import com.grendelscan.scan.Scan;
import com.grendelscan.tests.libraries.platformErrorMessages.PlatformErrorMessages;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HttpUtils;

public class Fuzzer
{
	private String template;
	private FuzzVector fuzzVector;
	private int maxRequests;
	private int requestCount;
	private String unusedFuzzValue;
	public static final String FUZZ_TOKEN = "%%FUZZ%%";
	private Scan scan;
	private static Thread fuzzThread;
	private boolean running;
	private List<InterceptFilter> filters;
	private boolean paused;
	public void setPaused(boolean paused)
    {
    	this.paused = paused;
    }

	public boolean isPaused()
    {
    	return paused;
    }

	private boolean matchPlatformMessages;
	
	public Fuzzer(FuzzVector fuzzVector, int maxRequests, Scan scan, String template, List<InterceptFilter> filters, boolean matchPlatformMessages)
    {
	    super();
	    this.filters = filters;
	    this.fuzzVector = fuzzVector;
	    if (maxRequests == 0)
	    {
	    	maxRequests = Integer.MAX_VALUE;
	    }
	    this.maxRequests = maxRequests;
	    this.scan = scan;
	    this.template = template;
	    this.matchPlatformMessages = matchPlatformMessages;
    }

	public boolean isRunning()
    {
    	return running;
    }

	public void stop()
	{
		running = false;
		if (fuzzThread != null)
		{
			synchronized(this)
			{
				try
	            {
		            this.wait(1500);
	            }
	            catch (InterruptedException e)
	            {
	            }
			}
			fuzzThread.interrupt();
		}
	}
	
	public boolean done()
	{
		return fuzzVector.done();
	}
	
	public void reset()
	{
		requestCount = 0;
		fuzzVector.reset();
	}
	
	public void start()
	{
		if (fuzzThread != null && fuzzThread.isAlive())
		{
			scan.getScanStatusWindow().displayMessage("Error", "Fuzzing already underway");
		}
		else
		{
			running = true;
			fuzzThread = new Thread(new fuzzRunable());
			fuzzThread.setName("Fuzzer thread");
			fuzzThread.start();
		}
	}

	private class fuzzRunable implements Runnable
	{
		public void run()
        {
			fuzzVector.reset();
	        while(!fuzzVector.done() && running && requestCount < maxRequests)
	        {
	        	pause();	
	        	UnvalidatedHttpTransaction transaction = getNextTransaction();
	        	// Could be null if the request is unparsable
	        	if (transaction != null)
	        	{
		        	transaction.execute("Fuzzer", false, true);
		        	pause();	
		        	unusedFuzzValue = "";
		        	requestCount++;
		        	if (InterceptFilter.matchesFilters(filters, transaction) || 
		        		(
		        			matchPlatformMessages
		        			&& PlatformErrorMessages.isErrorMatch(transaction, true).length() > 0)
		        		)
		        	{
		        		scan.getScanStatusWindow().getFuzzResultsTable().addTransaction(transaction);
		        	}
	        	}
	        }
	        scan.getScanStatusWindow().fuzzDone();
        }
	}
	
	private void pause()
	{
		synchronized(fuzzThread)
		{
			try
            {
				while (paused && running)
				{
					fuzzThread.wait(250);
				}
            }
            catch (InterruptedException e)
            {
            }
		}
	}
	
	private UnvalidatedHttpTransaction getNextTransaction()
	{
		UnvalidatedHttpTransaction transaction = null;
		if (!fuzzVector.done())
		{
			String value; 
			if (unusedFuzzValue == null || unusedFuzzValue.equals(""))
			{
				value = fuzzVector.getNextValue();
				unusedFuzzValue = value;
			}
			else
			{
				value = unusedFuzzValue;
			}
			if (value != null)
			{
				String rawRequest = template.replace(FUZZ_TOKEN, value);
				UnvalidatedHttpRequest request;
                try
                {
	                request = HttpUtils.parseIntoHttpRequest(rawRequest);
					byte body[] = new byte[0]; 
					if (request instanceof UnvalidatedHttpEntityRequest)
					{
						body = ((UnvalidatedHttpEntityRequest) request).getBody();
					}
					
					transaction = new UnvalidatedHttpTransaction(scan, 0, TransactionSource.FUZZER, body, request);
                }
                catch (HttpFormatException e)
                {
                	Debug.errDebug("Invalid request format in fuzzer: " + e.toString(), e);
                }
			}
		}
		return transaction;
	}
}
