package com.grendelscan.proxy;

import java.io.IOException;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;

import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.PlainSocketFactory;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.SSLSocketFactory;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpService;

import com.grendelscan.requester.http.client.*;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.Scan;
import com.grendelscan.scan.ScanSettings;
import com.grendelscan.utils.Debug;


public class Proxy
{
	private int maxThreads;
	private RequestListenerThread[] requestListenerThreads;
	protected ServerSocket serverSocket;
	protected Scan scan;
	protected CustomHttpClient proxyHttpClient;
	private boolean running;
	private ClientConnectionManager ccm;
	private final Integer runningLock = 0; 
	public Proxy (Scan scan)
	{
		this.scan = scan;
	}
	
	public void startProxy()
	{
		synchronized(runningLock)
		{
			if (running)
			{
				return;
			}
		
			HttpParams params = ClientUtilities.createHttpParams(scan, 5, 2);
			HttpContext context = ClientUtilities.createHttpContext(scan);
			
			ccm = ClientUtilities.createClientConnectionManager(params);
			proxyHttpClient = new CustomHttpClient(ccm, params, context, scan);
	
			
			this.maxThreads = getMaxThreads();
			int bindPort = getBindPort();
			String bindAddress = getIPAddress();
			
			int maxProxyQueueLength = ConfigurationManager.getInt("proxy.max_queue_length", 20);
			try
			{
				if (bindAddress.equals("0.0.0.0"))
				{
					serverSocket = new ServerSocket(bindPort, maxProxyQueueLength);
				}
				else
				{
					InetAddress bindInetAddress = InetAddress.getByName(bindAddress);
					serverSocket = new ServerSocket(bindPort, maxProxyQueueLength, bindInetAddress);
				}
				serverSocket.setSoTimeout(500);
			}
			catch (IOException e) 
			{
				Debug.errDebug("Problem opening proxy socket: " + e.toString());
				
				scan.displayMessage("Error:", "The proxy server failed to start. You may need to change\n" +
							"some parameters. The error message is: \n\n" + e.getMessage());
				running = false;
				return;
			}
			running = true;
			initThreads();
		}
	}

	protected int getMaxThreads()
	{
		return scan.getScanSettings().getMaxProxyThreads();
	}
	
	protected int getBindPort()
	{
		return scan.getScanSettings().getProxyPort();
	}
	
	protected String getIPAddress()
	{
		return scan.getScanSettings().getProxyIPAddress();
	}

	private void initThreads()
	{
		SchemeRegistry schemeRegistry = new SchemeRegistry();
		SSLSocketFactory secureSocketFactory = null;
		KeyStore trustStore;
		try
		{
			trustStore  = KeyStore.getInstance(KeyStore.getDefaultType());
			secureSocketFactory = new SSLSocketFactory(trustStore);
		}
		catch (KeyStoreException e) 
		{
			Debug.errDebug("Problem initializing KeyStore: " + e.toString(), e);
			System.exit(1);
		}
		catch (NoSuchAlgorithmException e)
		{
			Debug.errDebug("Problem initializing KeyStore: " + e.toString(), e);
			System.exit(1);
		}
		catch (KeyManagementException e)
		{
			Debug.errDebug("Problem initializing KeyStore: " + e.toString(), e);
			System.exit(1);
		}
		catch (UnrecoverableKeyException e)
		{
			Debug.errDebug("Problem initializing KeyStore: " + e.toString(), e);
			System.exit(1);
		}
		
		PlainSocketFactory plainSocketFactory = PlainSocketFactory.getSocketFactory();
		
		schemeRegistry.register(new Scheme("https", secureSocketFactory, 443));
		schemeRegistry.register(new Scheme("http", plainSocketFactory, 80));

		requestListenerThreads = new RequestListenerThread[maxThreads];
		for (int index = 0; index < maxThreads; index++)
		{
			RequestListenerThread listenerThread = new RequestListenerThread(scan, serverSocket, this);
			listenerThread.setName("ProxyThread-" + index);
			requestListenerThreads[index] = listenerThread;
			listenerThread.setDaemon(false);
			listenerThread.start();
		}
	}


	public boolean isRunning()
    {
    	return running;
    }


	public void stopProxy()
    {
		synchronized(runningLock)
		{
			if (!running)
			{
				return;
			}
			else
			{
				running = false;
			}
		}
		
		Debug.debug("Proxy disabled");
		ccm.shutdown();
		try
        {
			if (serverSocket != null)
				serverSocket.close();
        }
        catch (IOException e)
        {
        	Debug.errDebug("Problem closing proxy socket: " + e.toString(), e);
        }

        for (RequestListenerThread requestListenerThread: requestListenerThreads)
		{
			requestListenerThread.shutdown();
		}
   }
	
	public HttpService createHttpService(Socket socket, HttpParams params)
	{
		return MiscHttpFactory.createHttpServiceProxy(scan, socket, this, proxyHttpClient, false, 0, params);
	}
}
