package com.grendelscan.requester;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.regex.*;

import com.grendelscan.requester.http.factories.UriFactory;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpGetTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.*;

public class ResponseCodeOverrides
{
	private Map<Pattern, Integer> manualOverrides;
	private Map<Integer, Set<AbstractHttpTransaction>> automaticOverrideSamples;
	private Scan scan;
	private boolean useAutomaticOverrides;
	
	private int acceptableAutomaticThreshold;
	private int acceptableSamePatternThreshold;
	private HttpTransactionMatchCriteriaWeights weights;
	
	public ResponseCodeOverrides(Scan scan, Map<String, Integer> rawManualOverrides, 
						boolean useAutomaticOverrides, int acceptableAutomaticThreshold)
	{
		this.scan = scan;
		weights = new HttpTransactionMatchCriteriaWeights();
		this.useAutomaticOverrides = useAutomaticOverrides;
		this.acceptableAutomaticThreshold = acceptableAutomaticThreshold;
		acceptableSamePatternThreshold = ConfigurationManager.getInt("response_code_overrides.same_pattern_threshold", 95);
		
		manualOverrides = Collections.synchronizedMap(new HashMap<Pattern, Integer>(1));
		for (String pattern: rawManualOverrides.keySet())
		{
			manualOverrides.put(Pattern.compile(pattern, Pattern.CASE_INSENSITIVE | Pattern.DOTALL), rawManualOverrides.get(pattern));
		}
		
		if (useAutomaticOverrides)
		{
			automaticOverrideSamples = Collections.synchronizedMap(new HashMap<Integer, Set<AbstractHttpTransaction>>());
			Set<AbstractHttpTransaction> responses = Collections.synchronizedSet(new HashSet<AbstractHttpTransaction>());
			automaticOverrideSamples.put(404, responses);
		}
	}
	
	
	public synchronized void generateAllAutomaticOverrides(String baseUri)
	{
		Set <URI> uris = Collections.synchronizedSet(new HashSet<URI>());
		
		// Make the URI for a directory
		try
		{
			URI dirUri = UriFactory.makeUri(baseUri + "random" + StringUtils.generateRandomString(StringUtils.FORMAT_LOWER_CASE_ALPHA, 6) + "/", true);
			uris.add(dirUri);
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("Problem with URI syntax: " + e.toString(), e);
		}

		for (String ext: ConfigurationManager.getStringArray("file_enumeration.common_framework_extensions"))
		{
			try
			{
				URI uri = UriFactory.makeUri(baseUri + "random" + StringUtils.generateRandomString(StringUtils.FORMAT_LOWER_CASE_ALPHA, 6) + "." + ext, true);
				uris.add(uri);
			}
			catch (URISyntaxException e)
			{
				Debug.errDebug("Problem with URI syntax: " + e.toString(), e);
			}
		}
		generateAutomaticOverrides(uris, 404);
	}

	
	private void generateAutomaticOverrides(Set<URI> uris, int logicalCode)
	{			
		Set<AbstractHttpTransaction> responses = automaticOverrideSamples.get(logicalCode);

		for (URI uri: uris)
		{
			if (! scan.isRunning())
				{return;}

			Debug.debug("Testing for logical " + logicalCode + " responses: " + uri.toASCIIString());
            try
            {
				HttpGetTransaction transaction = new HttpGetTransaction(scan, uri, 0, TransactionSource.AUTOMATIC_RESPONSE_CODE_OVERRIDES);
				transaction.execute("Response code overrides ", false);
				if (transaction.isSuccessfullExecution() && transaction.getResponseStatusCode() != logicalCode)
				{
					addSignatureIfNew(transaction, responses);
				}
            }
            catch (URISyntaxException e)
            {
            	Debug.errDebug("Problem creating request due to uri: " + e.toString(), e);
            }
		}
	}
	
	
	private void addSignatureIfNew(AbstractHttpTransaction transaction, Set<AbstractHttpTransaction> responses)
	{
		synchronized(responses)
		{
			for (AbstractHttpTransaction oldTransaction: responses)
			{
//				int score = StringUtils.scoreStringDifferenceIgnoreCase(response, oldResponse, 100);
				int score = HttpTransactionUtils.scoreResponseMatch(transaction, oldTransaction, weights, 100);
				if (score >= acceptableSamePatternThreshold)
				{
					return;
				}
			}
			responses.add(transaction);
		}
	}
	
	
	public int getLogicalResponseCode(AbstractHttpTransaction transaction)
	{
		if (transaction.getResponseStatusCode() == 0)
		{
			Debug.errDebug("Transaction didn't have a response: " + transaction.getUriString());
			return 0;
		}
		
		for (Pattern override: manualOverrides.keySet())
		{
			if (override.matcher(transaction.getResponseBodyAsString()).find())
			{
				return manualOverrides.get(override);
			}
		}
		
		if (useAutomaticOverrides)
		{
			// Create a temp collection to prevent contention
			Map<Integer,Set<AbstractHttpTransaction>> tempSamples = new HashMap<Integer,Set<AbstractHttpTransaction>>(automaticOverrideSamples);
			for (int sampleResponseCode: tempSamples.keySet())
			{
				if (transaction.getResponseStatusCode() == sampleResponseCode)
				{
					continue;
				}


				Set<AbstractHttpTransaction> tempSet;
				// Minimize the lock time
				synchronized (tempSamples.get(sampleResponseCode))
				{
					tempSet = new HashSet<AbstractHttpTransaction>(tempSamples.get(sampleResponseCode));
				}
				
				for (AbstractHttpTransaction sampleTransaction: tempSet)
				{
					if (HttpTransactionUtils.scoreResponseMatch(transaction, sampleTransaction, weights, 100) >= acceptableAutomaticThreshold)
					{
						return sampleResponseCode;
					}
				}
			}
		}
		
		return transaction.getResponseStatusCode();
	}
	
}
