/*
 * 
 * ==================================================================== Licensed
 * to the Apache Software Foundation (ASF) under one or more contributor license
 * agreements. See the NOTICE file distributed with this work for additional
 * information regarding copyright ownership. The ASF licenses this file to you
 * under the Apache License, Version 2.0 (the "License"); you may not use this
 * file except in compliance with the License. You may obtain a copy of the
 * License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by many individuals on
 * behalf of the Apache Software Foundation. For more information on the Apache
 * Software Foundation, please see <http://www.apache.org/>.
 * 
 */
package com.grendelscan.requester.http.client;

import java.net.*;
import java.io.IOException;
import com.grendelscan.utils.*;
import java.util.*;

import org.apache.http.ConnectionReuseStrategy;
import org.apache.http.Header;
import org.apache.http.HttpException;
import org.apache.http.HttpHost;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.client.AuthenticationHandler;
import org.apache.http.client.ClientRequestDirector;
import org.apache.http.client.HttpRequestRetryHandler;
import org.apache.http.client.RedirectHandler;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.client.protocol.RequestDefaultHeaders;
import org.apache.http.client.protocol.RequestProxyAuthentication;
import org.apache.http.client.protocol.RequestTargetAuthentication;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.routing.HttpRoutePlanner;
import org.apache.http.impl.client.DefaultClientRequestDirector;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.params.HttpParams;
import org.apache.http.protocol.BasicHttpProcessor;
import org.apache.http.protocol.ExecutionContext;
import org.apache.http.protocol.HttpContext;
import org.apache.http.protocol.HttpProcessor;
import org.apache.http.protocol.RequestConnControl;
import org.apache.http.protocol.RequestExpectContinue;
import org.apache.http.protocol.RequestTargetHost;
import org.apache.http.protocol.RequestUserAgent;
import com.grendelscan.requester.http.serializable.*;
import org.apache.http.protocol.SyncBasicHttpContext;

import com.grendelscan.scan.Scan;

/**
 * @author David Byrne
 * 
 */
public class CustomHttpClient extends DefaultHttpClient
{
	private HttpContext contextTemplate;
	private Scan scan;
	private static int failedRequests = 0;
	private Boolean failedRequestsLock = true;
	/**
	 * 
	 * @param conman
	 * @param params
	 */
	public CustomHttpClient(ClientConnectionManager conman, HttpParams params, HttpContext contextTemplate, Scan scan)
    {
	    super(conman, params);
	    this.scan = scan;
	    this.contextTemplate = contextTemplate;
    }

	@Override
    protected HttpContext createHttpContext() 
	{
		HttpContext context = new SyncBasicHttpContext(contextTemplate);
        return context;
    }

    public HttpResponse customExecute(HttpUriRequest request, HttpContext context) throws HttpException, InterruptedException
    {
    	return customExecute(null, request, context);
    }

    @Override
    protected ClientRequestDirector createClientRequestDirector(ClientConnectionManager conman,
            ConnectionReuseStrategy reustrat, HttpRoutePlanner rouplan, HttpProcessor httpProcessor,
            HttpRequestRetryHandler retryHandler, RedirectHandler redirectHandler,
            AuthenticationHandler targetAuthHandler, AuthenticationHandler proxyAuthHandler, HttpParams params)
    {
    	return new CustomClientRequestDirector(
                conman,
                reustrat,
                rouplan,
                httpProcessor,
                retryHandler,
                redirectHandler,
                targetAuthHandler,
                proxyAuthHandler,
                params);
    }

	public final HttpResponse customExecute(HttpHost target, HttpRequest request, HttpContext context) throws HttpException, InterruptedException
    {
    	if (context == null)
    	{
    		context = createHttpContext();
    	}
    	
    	if (target == null)
    	{
    		URI uri = ((HttpUriRequest)request).getURI();
	        target = new HttpHost(
	        		uri.getHost(), 
	        		uri.getPort(), 
	        		uri.getScheme());
    	}
        
        context.setAttribute(ExecutionContext.HTTP_TARGET_HOST, target);
        
    	prepareRequest(request, context);
    	int tries = 0;
    	HttpResponse response = null;
    	String message = "";
    	while (response == null && tries++ < scan.getScanSettings().getMaxRequestRetries())
    	{
    		try
            {
	            response = super.execute(target, request, context);
            }
            catch (IOException e)
            {
            	message += e.toString() + " ";
	            Debug.errDebug("ERROR: Connection failure to server: " + e.toString(), e);
            }
    		synchronized(failedRequestsLock)
    		{
	    		if (response == null)
	    		{
	    			if(++failedRequests > scan.getScanSettings().getMaxConsecutiveFailedRequests())
	    			{
	    				scan.terminateScan("Maximum consecutive failed requests exceeded");
	    			}
	    			else
	    			{
	    				Debug.errDebug("Request failed", new Throwable());
	    			}
	    		}
	    		else
	    		{
	    			failedRequests = 0;
	    		}
    		}
    	}
    	if (response == null)
    	{
    		throw new HttpException("Request failed." + (message.equals("") ? "" : " This might be the cause: " + message));
    	}
    	return response;
    }

    public String getRequestHeadString(HttpHost target, HttpRequest request, HttpContext context) throws IOException, HttpException
    {
    	if (context == null)
    	{
    		context = createHttpContext();
    	}
    	
    	if (target == null)
    	{
    		URI uri = ((HttpUriRequest)request).getURI();
	        target = new HttpHost(
	        		uri.getHost(), 
	        		uri.getPort(), 
	        		uri.getScheme());
    	}
        context.setAttribute(ExecutionContext.HTTP_TARGET_HOST, target);
    	prepareRequest(request, context);

        getHttpProcessor().process(request, context);

    	String requestString = request.getRequestLine().toString() + "\n";
    	for (Header header: request.getAllHeaders())
    	{
    		requestString += header.toString() + "\n";
    	}
    	return requestString;
    }
    
    private void prepareRequest(HttpRequest request, HttpContext context)
    {
    	request.setParams(this.getParams());
    }

	@Override
	protected BasicHttpProcessor createHttpProcessor()
	{
		BasicHttpProcessor httpproc = new BasicHttpProcessor();
		httpproc.addInterceptor(new RequestDefaultHeaders());
		// Required protocol interceptors
		httpproc.addInterceptor(new CustomRequestContent());
		httpproc.addInterceptor(new RequestTargetHost());
		// Recommended protocol interceptors
		httpproc.addInterceptor(new RequestConnControl());
		httpproc.addInterceptor(new RequestUserAgent());
		httpproc.addInterceptor(new RequestExpectContinue());
		// HTTP state management interceptors
//		httpproc.addInterceptor(new RequestAddCookies());
//		httpproc.addInterceptor(new ResponseProcessCookies());
		// HTTP authentication interceptors
//		httpproc.addInterceptor(new RequestTargetAuthentication());
//		httpproc.addInterceptor(new RequestProxyAuthentication());
		return httpproc;
	}


	@Override
    protected HttpRoutePlanner createHttpRoutePlanner()
    {
		return new CustomHttpRoutePlanner(getConnectionManager(), scan);    
	}

	public HttpContext getContextTemplate()
    {
    	return contextTemplate;
    }
	
} 

