package com.grendelscan.requester.http.payloads;

import java.io.Serializable;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.message.BasicNameValuePair;

import com.grendelscan.requester.http.serializable.SerializableNameValuePair;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.URIStringUtils;

/**
 * This contains an HTTP request payload in a generic format. It can be
 * converted to a URL query string, or a body entity.
 * 
 * @author David Byrne
 * 
 */
public class URLEncodedRequestPayload implements RequestPayload, Serializable
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 8713657218036592003L;
	private String charset;
	private Map<String, NameValuePair> parameters;
	
	public URLEncodedRequestPayload(ArrayList<NameValuePair> parameters, String charset)
	{
		initParameterMap();
		for (NameValuePair param: parameters)
		{
			this.parameters.put(param.getName(), convertToSerializable(param));
		}
		this.charset = charset;
	}
	
	private SerializableNameValuePair convertToSerializable(NameValuePair param)
	{
		SerializableNameValuePair serializablePair; 
		if (param instanceof SerializableNameValuePair)
		{
			serializablePair = (SerializableNameValuePair) param;
		}
		else
		{
			serializablePair = new SerializableNameValuePair(param.getName(), param.getValue());
		}
		return serializablePair;
	}
	
	public URLEncodedRequestPayload(String charset)
	{
		this.charset = charset;
		initParameterMap();
	}
	
	public URLEncodedRequestPayload(String charset, String body)
	{
		initParameterMap();
		addUrlEncodedParameters(body);
		this.charset = charset;
	}
	
	/**
	 * This will recreate the parameter to properly escape the value. SetValue
	 * isn't a method unfortunately.
	 * 
	 * @param parameter
	 */
	public void addParameter(NameValuePair parameter)
	{
		// I don't think I need this because the URLEntity automatically encodes
		// the params
		// BasicNameValuePair newParameter = new
		// BasicNameValuePair(parameter.getName(),
		// URIStringUtils.escapeQueryParameterValue(parameter.getValue()));
		
		parameters.put(parameter.getName(), convertToSerializable(parameter));
	}
	
	public void addParameter(String name, String value)
	{
		value = URIStringUtils.escapeQueryParameterComponent(value);
		NameValuePair parameter = new SerializableNameValuePair(name, value);
		addParameter(parameter);
	}
	
	/**
	 * This will do its best to parse the parameterString as something encoded
	 * for URLs. If it's a single word, then it will be considered a parameter
	 * name with no value, etc.
	 * 
	 * @param parameterString
	 */
	public void addUrlEncodedParameters(String parameterString)
	{
		if (parameterString != null)
		{
			for (String parameter: parameterString.split("&"))
			{
				if (parameter.equals(""))
				{
					continue;
				}
				String components[] = parameter.split("=");
				String name = components[0];
				String value = components.length > 1 ? components[1] : "";
				SerializableNameValuePair pair = new SerializableNameValuePair(URIStringUtils.decodeUrl(name), URIStringUtils.decodeUrl(value));
				parameters.put(name, pair);
			}
		}
	}
	
	@Override
	public RequestPayload clone()
	{
		ArrayList<NameValuePair> newParams = new ArrayList<NameValuePair>(parameters.size());
		for (NameValuePair param: parameters.values())
		{
			NameValuePair np = new BasicNameValuePair(param.getName(), param.getValue());
			newParams.add(np);
		}
		URLEncodedRequestPayload n = new URLEncodedRequestPayload(newParams, charset);
		return n;
	}
	
	public NameValuePair getParameterByName(String name)
	{
		return parameters.get(name);
	}
	
	public Set<String> getParameterNames()
	{
		return parameters.keySet();
	}
	
	public Set<NameValuePair> getParameters()
	{
		return new HashSet<NameValuePair>(parameters.values());
	}
	
	public boolean isEmpty()
	{
		return parameters.equals("");
	}
	
	public void removeParametersByName(String name)
	{
		parameters.remove(name);
	}
	
	/**
	 * The difference between replaceParameter and addParameter is that
	 * replaceParameter will delete all other parameters of the same name, while
	 * addParameter can support duplicate parameter names.
	 * 
	 * @param parameter
	 */
	public void replaceParameter(NameValuePair parameter)
	{
		removeParametersByName(parameter.getName());
		addParameter(parameter);
	}
	
	
	public void replaceParameter(String name, String value)
	{
		removeParametersByName(name);
		NameValuePair parameter = new BasicNameValuePair(name, value);
		addParameter(parameter);
	}
	
	public HttpEntity toEntity()
	{
		
		List<NameValuePair> pairs = new ArrayList<NameValuePair>(parameters.size());
		int index = 0;
		for (NameValuePair pair: parameters.values())
		{
			pairs.add(pair);
		}
		
		HttpEntity entity = null;
        try
        {
	        entity = new UrlEncodedFormEntity(pairs);
        }
        catch (UnsupportedEncodingException e)
        {
        	Debug.errDebug("Very odd problem: " + e.toString(), e);
        }
		
		return entity;
	}
	
	public String toUrlQueryString()
	{
		return URIStringUtils.urlEncode(parameters.values());
	}
	
	private void initParameterMap()
	{
		parameters = new HashMap<String, NameValuePair>();
	}
	
	public byte[] getBody()
	{
		return toUrlQueryString().getBytes();
	}
	
}
