package com.grendelscan.requester.http.transactions;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Matcher;

import org.apache.http.Header;
import org.apache.http.HttpException;
import org.apache.http.HttpRequest;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.RequestLine;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.protocol.HttpContext;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.client.CustomHttpClient;
import com.grendelscan.requester.http.factories.UriFactory;
import com.grendelscan.requester.http.payloads.URLEncodedRequestPayload;
import com.grendelscan.requester.http.wrappers.StandardHttpRequestWrapper;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.FileUtils;
import com.grendelscan.utils.HttpCloner;
import com.grendelscan.utils.URIStringUtils;

public abstract class AbstractStandardHttpTransaction extends AbstractHttpTransaction
{

	protected transient boolean payloadChanged;
	protected transient boolean payloadInitialized;
	protected StandardHttpRequestWrapper requestWrapper;
	
	// protected HttpUriRequest uriRequest;
	
	protected AbstractStandardHttpTransaction(Scan scan, HttpRequest request, int referer, TransactionSource source)
	        throws URISyntaxException
	{
		super(scan, referer, source);
		if (request == null)
		{
			throw new IllegalArgumentException("Request cannot be null");
		}
		URI uri = ((HttpUriRequest) request).getURI();
		request.setParams(scan.getHttpClient().getParams());
		String scheme = uri.getScheme();
		// Make sure the scheme is supported
		if ((scheme != null) && !scheme.equals("") && !scheme.equalsIgnoreCase("http")
		        && !scheme.equalsIgnoreCase("https"))
		{
			throw new URISyntaxException(uri.toASCIIString(), "\"" + scheme + "\" is not a supported scheme");
		}
		initRequestWrapper(request);
	}
	
	protected AbstractStandardHttpTransaction(Scan scan, StandardHttpRequestWrapper requestWrapper, int referer, TransactionSource source)
	{
		super(scan, referer, source);
		if (requestWrapper == null)
		{
			throw new IllegalArgumentException("RequestWrapper cannot be null");
		}
		initRequestWrapper(requestWrapper);
	}

	/**
	 * This will add query parameters to the request. If it is a POST, they will
	 * be added to the POST body. Note that if there's already a body that isn't
	 * parameterized, it will be erased. If it's anything else, they will be
	 * added to the URI query string. If you want to add query parameters to the
	 * URI of a POST, you'll have to do it manually. In both cases, any previous
	 * value will be overwritten.
	 * 
	 * @param name
	 *            The parameter name
	 * @param value
	 *            The parameter value
	 */
	public void addQueryParameter(String name, String value)
	{
		// initEmptyURLRequestPayload();
		payloadChanged = true;
		getRequestPayload().addParameter(name, value);
	}
	
	@Override
	public void addRequestHeader(Header header)
	{
		if (getResponseWrapper() == null)
		{
			requestWrapper.addHeader(header);
		}
	}
	
	@Override
	public String getDirectory()
	{
		return requestWrapper.getURI().getPath().replaceFirst("/[^/]*$", "") + "/";
	}
	
	@Override
	public String getFilename()
	{
		return requestWrapper.getURI().getPath().replaceFirst(".*/([^/]*)$", "$1");
	}
	
	@Override
	public String getHost()
	{
		return requestWrapper.getURI().getHost();
	}
	
	@Override
	public String getMethod()
	{
		return requestWrapper.getMethod();
	}
	
	@Override
	public String getPath()
	{
		String path = "";
		String wholePath = requestWrapper.getURI().getPath();
		if (wholePath != null)
		{
			int lastSlash = wholePath.lastIndexOf("/");
			if (lastSlash >= 0)
			{
				path = wholePath.substring(0, lastSlash + 1);
			}
		}
		else
		{
			Debug.errDebug("Null path for " + requestWrapper.getURI().toASCIIString() , new Throwable(){});
		}
		return path;
	}
	
	@Override
	public String getPayloadQueryString()
	{
		if ((requestWrapper.getRequestPayload() != null)
		        && (requestWrapper.getRequestPayload() instanceof URLEncodedRequestPayload))
		{
			return getRequestPayload().toUrlQueryString();
		}
		else
		{
			return getUriQueryString();
		}
	}
	
	@Override
	public int getPort()
	{
		int port = requestWrapper.getURI().getPort();
		if (port < 0)
		{
			if (requestWrapper.getURI().getScheme().equalsIgnoreCase("HTTPS"))
			{
				port = 443;
			}
			else
			{
				port = 80;
			}
		}
		return port;
	}
	
	@Override
	public Set<String> getQueryParameterNames()
	{
		Set<String> names;
		if (requestWrapper.getRequestPayload() == null)
		{
			names = new HashSet<String>();
			String query = getPayloadQueryString();
			Matcher m = queryParameterPattern.matcher(query);
			while (m.find())
			{
				names.add(m.group(1));
			}
		}
		else
		{
			names = getRequestPayload().getParameterNames();
		}
		
		return names;
		
	}
	
	/**
	 * This is really the request payload. I.e. the query string for a GET and
	 * the body for a post. DON'T USE THIS TO MODIFY PARAMETERS. THEY MAY NOT BE
	 * USED.
	 * 
	 * @return
	 */
	@Override
	public Set<NameValuePair> getQueryParameters()
	{
		return getRequestPayload().getParameters();
	}
	
	@Override
	public String getQueryParameterValue(String name)
	{
		return getRequestPayload().getParameterByName(name).getValue();
	}
	
	@Override
	public HttpRequest getRequest()
	{
		return requestWrapper.getRequest();
	}
	
	public URLEncodedRequestPayload getRequestPayload()
	{
		return (URLEncodedRequestPayload) requestWrapper.getRequestPayload();
	}
	
	public StandardHttpRequestWrapper getRequestWrapper()
	{
		return requestWrapper;
	}
	
	@Override
	public String getScheme()
	{
		return requestWrapper.getURI().getScheme();
	}
	
	public URI getURI()
	{
		updatePayload();
		return requestWrapper.getURI();
	}
	
	@Override
	public String getUriQueryString()
	{
		updatePayload();
		String query = requestWrapper.getURI().getQuery();
		if (query == null)
		{
			query = "";
		}
		return query;
	}
	
	@Override
	public String getUriString()
	{
		updatePayload();
		return requestWrapper.getURI().toASCIIString();
	}
	
	public boolean isPayloadChanged()
	{
		return payloadChanged;
	}
	
	public boolean isPayloadInitialized()
	{
		return payloadInitialized;
	}
	
	@Override
	public boolean isSecure()
	{
		if (requestWrapper.getURI().getScheme().equalsIgnoreCase("https"))
		{
			return true;
		}
		return false;
	}
	
	@Override
	public void removeQueryParameters(String name)
	{
		payloadChanged = true;
		getRequestPayload().removeParametersByName(name);
	}
	
	@Override
	public void replaceQueryParameter(NameValuePair parameter)
	{
		payloadChanged = true;
		getRequestPayload().replaceParameter(parameter);
	}
	
	@Override
	public void replaceQueryParameter(String name, String value)
	{
		payloadChanged = true;
		getRequestPayload().replaceParameter(name, value);
	}
	
	public void setPayloadChanged(boolean payloadChanged)
	{
		this.payloadChanged = payloadChanged;
	}
	
	public void setPayloadInitialized(boolean payloadInitialized)
	{
		this.payloadInitialized = payloadInitialized;
	}
	
	public void setPayloadQueryString(String query)
	{
		if (getResponseWrapper() == null)
		{
			payloadChanged = true;
			requestWrapper.setRequestPayload(new URLEncodedRequestPayload(getRequestCharacterSet(), query));
		}
	}
	
	public void setRequestPayload(URLEncodedRequestPayload requestPayload)
	{
		payloadChanged = true;
		requestWrapper.setRequestPayload(requestPayload);
	}
	
	public void setRequestWrapper(StandardHttpRequestWrapper requestWrapper)
	{
		this.requestWrapper = requestWrapper;
	}
	
	/**
	 * If the string doesn't parse as a valid absolute URI, nothing will change,
	 * so be careful.
	 * 
	 * @param uri
	 */
	@Override
	public void setURI(String uri)
	{
		try
		{
			URI u = UriFactory.makeUri(uri, true);
			setURI(u);
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("Invalid URL format (" + uri + "): " , e);
		}
	}
	
	public void setURI(URI uri)
	{
		requestWrapper.setURI(uri);
	}
	
	public abstract void updatePayload();
	
	/**
	 * Doesn't copy the response, the request, or the payload.
	 * 
	 * Is there a better way of doing this?
	 * 
	 */
	protected void copyFields(AbstractStandardHttpTransaction target)
	{
		super.copyFields(target);
//		target.setRequestWrapper(requestWrapper.clone());
	}
	
//	abstract protected String extractPayload();
	
	protected abstract void initRequestWrapper(HttpRequest request);
	
	protected void initRequestWrapper(StandardHttpRequestWrapper requestWrapper)
	{
		payloadInitialized = true;
		payloadChanged = false;
		
		this.requestWrapper = requestWrapper;
	}
	
	private URI getLocationHeaderUri(HttpResponse response)
	{
		URI locationUri = null;
		String newLocation = response.getFirstHeader("Location").getValue();
		newLocation = URIStringUtils.escapeUri(newLocation);
		try
		{
			locationUri = UriFactory.makeAbsoluteUri(newLocation, getURI());
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("Problem with location header format (" + newLocation + "): " + e.toString(), e);
		}
		return locationUri;
	}
	
	private void handleRedirect(HttpResponse initialResponse, CustomHttpClient client, HttpContext context, boolean testTransaction, String executionSource)
	{

		/*
		 * If it's a redirect, request the redirect location header,
		 * then fork into three total transactions: the original request
		 * with the redirect response, the request of the redirect
		 * location with that response, and the original transaction:
		 * the original request with the eventual response after
		 * redirect. All three should be processed for testing.
		 * 
		 */

		URI locationURI = getLocationHeaderUri(initialResponse);
		
		if (locationURI == null)
		{
			// nothing else we can do here
			return;
		}
		
		HttpResponse redirectRequestResponse = null;
		AbstractStandardHttpTransaction transactionWithOriginalRequestAndRedirectResponse = null;
		
		/*
		 * Confirm that this isn't a redirect to the same URL and that it points 
		 * somewhere we can go
		 */
		if (!locationURI.toASCIIString().equals(getAbsoluteUriString()) 
			&& scan.getScanSettings().getUrlFilters().isUriAllowed(locationURI.toASCIIString()))
		{
			// Setup a transaction for the original request and
			// the redirect (3xx) response
			transactionWithOriginalRequestAndRedirectResponse = (AbstractStandardHttpTransaction) clone(this.source);
			transactionWithOriginalRequestAndRedirectResponse.setResponse(initialResponse);
			transactionWithOriginalRequestAndRedirectResponse.save();
			originalResponseTransactionID = transactionWithOriginalRequestAndRedirectResponse.getId();
			if (testTransaction)
			{
				scan.getCategorizerQueue().addTransaction(transactionWithOriginalRequestAndRedirectResponse);
			}
			
			// follow the redirect with a temporary transaction,
			// then set the response
			// in this (the original) transaction
			try
			{
				HttpGetTransaction redirectLocationRequestTransaction = new HttpGetTransaction(scan, locationURI, transactionWithOriginalRequestAndRedirectResponse.getId(), source);
				redirectLocationRequestTransaction.setRedirectCount(redirectCount + 1);
				redirectLocationRequestTransaction.execute(client, context, testTransaction, "Redirect for " + executionSource);
				redirectRequestTransactionID = redirectLocationRequestTransaction.getId();
				if (redirectLocationRequestTransaction.isSuccessfullExecution())
				{
					if (testTransaction)
					{
						scan.getCategorizerQueue().addTransaction(redirectLocationRequestTransaction);
					}
					// Need to clone the response to deal with consumed entities
					redirectRequestResponse = HttpCloner.clone(redirectLocationRequestTransaction.getResponse(), 
							redirectLocationRequestTransaction.getResponseBody());
				}
			}
			catch (URISyntaxException e)
			{
				Debug.errDebug("Problem requesting redirect due to the URL: " + e.toString(), e);
			}
			
			if (redirectRequestResponse == null) // If the redirect request failed
			{
				redirectRequestResponse = HttpCloner.clone(transactionWithOriginalRequestAndRedirectResponse.getResponse(), 
						transactionWithOriginalRequestAndRedirectResponse.getResponseBody());
			}
		}
		else
		{
			redirectRequestResponse = initialResponse;
		}
		
		setResponse(redirectRequestResponse);
	}
	
	@Override
	protected void privateExecute(CustomHttpClient client, HttpContext context, boolean testTransaction, String executionSource)
	{
		updatePayload();
		// sessionState.insertSesssionID(this);
		updateCookies();
		HttpResponse localResponse;
		try
		{
			HttpRequest request = getRequest();
			sentRequestHead = client.getRequestHeadString(null, request, context);
			localResponse = client.customExecute((HttpUriRequest) request, context);
			executionTime = (new java.util.Date()).getTime();

			int statusCode = localResponse.getStatusLine().getStatusCode();
			if (followRedirects 
					&& statusCode >= 300 && statusCode < 400 && statusCode != 304 
					&& redirectCount <= scan.getScanSettings().getMaxRedirects())
			{
				handleRedirect(localResponse, client, context, testTransaction, executionSource);
			}
			else if (localResponse != null) // if it wasn't a redirect
			{
				setResponse(localResponse);
			}
			
			if (getResponseWrapper() == null)
			{
				Debug.errDebug("Null response");
			}
			else
			{
				successfullExecution = true;
			}
		}
		catch (IOException e)
		{
			Debug.errDebug("Problem executing request for " + getUriString() + ": " + e.toString(), e);
		}
		catch (InterruptedException e)
		{
			// Not sure what I should be doing here
		}
		catch (HttpException e)
		{
			Debug.errDebug("Not sure why this happened at HttpTransaction.execute: " + e.toString(), e);
		}
	}

	@Override
    public Header getFirstRequestHeader(String headerName)
    {
	    return requestWrapper.getFirstHeader(headerName);
    }

	@Override
    public Header getLastRequestHeader(String headerName)
    {
	    return requestWrapper.getLastHeader(headerName);
    }

	@Override
    public Header[] getRequestHeaders(String headerName)
    {
	    return requestWrapper.getHeaders(headerName);
    }

	@Override
    public void addRequestHeader(String name, String value)
    {
		requestWrapper.addHeader(name, value);
    }

	@Override
    public Header[] getAllRequestHeaders()
    {
	    return requestWrapper.getCopyOfAllHeaders();
    }

	@Override
    public void removeRequestHeaders(String name)
    {
		requestWrapper.removeHeaders(name);
    }

    public RequestLine getRequestLine()
    {
	    return requestWrapper.getRequest().getRequestLine();
    }


	@Override
    public synchronized void writeToDisk()
    {
		if (originalResponseTransactionID == 0)
		{
			super.writeToDisk();
		}
		else
		{
	    	AbstractHttpTransaction originalResponseTransaction = scan.getTransactionRecord().getTransaction(originalResponseTransactionID);
	    	AbstractHttpTransaction redirectRequestTransaction = scan.getTransactionRecord().getTransaction(redirectRequestTransactionID);

	    	String transactionTest = "NOTE: This contains two transactions that handled a redirect\n\n\n"  
	    		+ originalResponseTransaction.toString() + "\n\n==========================================================\n\n"
	    		+ redirectRequestTransaction.toString();
	    	writtenToDisk = true;
			FileUtils.writeToFile(scan.getScanSettings().getOutputDirectory() + File.separator + scan.getScanSettings().getSavedTextTransactionsDirectory() + File.separator + getId() + ".txt", transactionTest);
		}
    }
}
