package com.grendelscan.requester.http.transactions;

import org.apache.http.conn.HttpHostConnectException;
import java.io.IOException;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.*;
import org.apache.http.message.*;
import org.apache.http.protocol.*;
import org.apache.http.client.methods.*;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.*;
import com.grendelscan.requester.http.client.CustomHttpClient;
import com.grendelscan.requester.http.serializable.SerializableRequestLine;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.URIStringUtils;

/**
 * This is for requests that may violate HTTP standards so typical objects (e.g.
 * java.net.uri) cannot be used.
 * 
 * @author David Byrne
 * 
 */
public class UnvalidatedHttpTransaction extends AbstractHttpTransaction
{
	/**
	 * 
	 */
	private static final long serialVersionUID = 256619227298731217L;
	private byte requestBody[];
	protected UnvalidatedHttpRequest unvalidatedRequest;
	
	public UnvalidatedHttpTransaction(Scan scan, int referer, String method, String uri, String host,
	        int port, boolean ssl, TransactionSource source, int majorProtocolVersion, int minorProtocolVersion, byte body[])
	{
		super(scan, referer, source);
		this.requestBody = body;
		if (body == null)
		{
			unvalidatedRequest =
			        new UnvalidatedHttpRequest(method, uri, "HTTP", majorProtocolVersion, minorProtocolVersion, host, port, ssl);
		}
		else
		{
			unvalidatedRequest =
		        new UnvalidatedHttpEntityRequest(method, uri, "HTTP", majorProtocolVersion, minorProtocolVersion, host, port, ssl, body);
		}
		unvalidatedRequest.setParams(scan.getHttpClient().getParams());
		this.addRequestHeader("Host", getHost());
		generalObjectInit();
		save();
	}
	
	public UnvalidatedHttpTransaction(Scan scan, int referer, String method, String uri, String host, int port,
	        boolean ssl, TransactionSource source, byte body[])
	{
		this(scan, referer, method, uri, host, port, ssl, source, 1, 1, body);
	}
	
	
	
	public UnvalidatedHttpTransaction(Scan scan, int referer, TransactionSource source, byte[] body,
            UnvalidatedHttpRequest unvalidatedRequest)
    {
	    super(scan, referer, source);
	    this.requestBody = body;
	    this.unvalidatedRequest = unvalidatedRequest;
		if(!
			(
				(source == TransactionSource.PROXY && !scan.getScanSettings().isTestProxyRequests())
				|| 
				(source == TransactionSource.MANUAL_REQUEST && !scan.getScanSettings().isTestManualRequests())
			)
			)
		{
		   generalObjectInit();
		}
		save();
   }

	@Override
	public void addRequestHeader(Header header)
	{
		if (getResponseWrapper() == null)
		{
			unvalidatedRequest.addHeader(header);
		}
	}
	
	@Override
	public void addRequestHeader(String name, String value)
	{
		unvalidatedRequest.addHeader(name, value);
	}
	
	@Override
	public AbstractHttpTransaction clone(TransactionSource source)
	{
		this.source = source;
		UnvalidatedHttpTransaction newTransaction =
		        new UnvalidatedHttpTransaction(scan, referer, unvalidatedRequest.getRequestLine().getMethod(),
		                unvalidatedRequest.getRequestLine().getUri(), unvalidatedRequest.getTargetedHost(),
		                unvalidatedRequest.getTargetedPort(), unvalidatedRequest.isSsl(),
		                source, unvalidatedRequest.getProtocolVersion().getMajor(), unvalidatedRequest.getProtocolVersion().getMinor(), requestBody);
		return newTransaction;
	}
	
	@Override
	public Header[] getAllRequestHeaders()
	{
		return unvalidatedRequest.getAllHeaders();
	}
	
	@Override
	public String getDirectory()
	{
		
		return URIStringUtils.getDirectory(unvalidatedRequest.getRequestLine().getUri());
	}
	
	@Override
	public String getFilename()
	{
		
		return URIStringUtils.getFilename(unvalidatedRequest.getRequestLine().getUri());
	}
	
	@Override
	public Header getFirstRequestHeader(String headerName)
	{
		return unvalidatedRequest.getFirstHeader(headerName);
	}
	
	/**
	 * Note that this will return the host in the URI, NOT the targeted host. To
	 * get the targeted host, try "getTargetedHost".
	 */
	@Override
	public String getHost()
	{
		
		String tempHost = URIStringUtils.getHost(unvalidatedRequest.getRequestLine().getUri());
		if ((tempHost == null) || tempHost.equals(""))
		{
			tempHost = getTargetedHost();
		}
		return tempHost;
	}
	
	@Override
	public Header getLastRequestHeader(String headerName)
	{
		return unvalidatedRequest.getLastHeader(headerName);
	}
	
	@Override
	public String getMethod()
	{
		return unvalidatedRequest.getRequestLine().getMethod();
	}
	
	@Override
	public String getPath()
	{
		
		return URIStringUtils.getPath(unvalidatedRequest.getRequestLine().getUri());
	}
	
	/**
	 * This will only interact with the URI query string
	 */
	@Override
	public String getPayloadQueryString()
	{
		
		String query;
		String uri = unvalidatedRequest.getRequestLine().getUri();
		int pos = uri.indexOf("?");
		if (pos > 0)
		{
			query = uri.substring(pos + 1);
		}
		else
		{
			query = "";
		}
		return query;
	}
	
	/**
	 * Note that this will return the port in the URI, NOT the targeted port.
	 * Tho get the targeted port, try "getTargetedPort".
	 */
	@Override
	public int getPort()
	{
		
		int port = URIStringUtils.getPort(unvalidatedRequest.getRequestLine().getUri());
		if (port <= 0)
		{
			port = getTargetedPort();
		}
		return port;
	}
	
	/**
	 * This will only interact with the URI query string
	 */
	@Override
	public Set<String> getQueryParameterNames()
	{
		Set<String> names = new HashSet<String>();
		String query = getPayloadQueryString();
		Matcher m = queryParameterPattern.matcher(query);
		while (m.find())
		{
			names.add(m.group(1));
		}
		
		return names;
	}
	
	/**
	 * This will only interact with the URI query string
	 */
	@Override
	public Set<NameValuePair> getQueryParameters()
	{
		Set<NameValuePair> parameters = new HashSet<NameValuePair>();
		String query = getPayloadQueryString();
		Matcher m = queryParameterPattern.matcher(query);
		while (m.find())
		{
			NameValuePair param = new BasicNameValuePair(m.group(1), m.group(2));
			parameters.add(param);
		}
		return parameters;
	}
	
	@Override
	public String getQueryParameterValue(String name)
	{
		String query = getPayloadQueryString();
		Matcher m = queryParameterPattern.matcher(query);
		while (m.find())
		{
			if (m.group(1).equalsIgnoreCase(name))
			{
				return m.group(2);
			}
		}
		
		return null;
	}
	
	@Override
	public UnvalidatedHttpRequest getRequest()
	{
		return unvalidatedRequest;
	}
	
	@Override
	public Header[] getRequestHeaders(String headerName)
	{
		return unvalidatedRequest.getHeaders(headerName);
	}
	
	@Override
	public String getScheme()
	{
		
		String scheme = URIStringUtils.getScheme(unvalidatedRequest.getRequestLine().getUri());
		if ((scheme == null) || scheme.equals(""))
		{
			if (isSecure())
			{
				scheme = "https";
			}
			else
			{
				scheme = "http";
			}
		}
		return scheme;
	}
	
	public String getTargetedHost()
	{
		
		return unvalidatedRequest.getTargetedHost();
	}
	
	public int getTargetedPort()
	{
		
		return unvalidatedRequest.getTargetedPort();
	}
	
	public UnvalidatedHttpRequest getUnvalidatedRequest()
	{
		return unvalidatedRequest;
	}
	
	@Override
	public String getUriQueryString()
	{
		
		return URIStringUtils.getQuery(unvalidatedRequest.getRequestLine().getUri());
	}
	
	@Override
	public String getUriString()
	{
		
		return unvalidatedRequest.getRequestLine().getUri();
	}
	
	@Override
	public boolean isSecure()
	{
		
		return unvalidatedRequest.isSsl();
	}
	
	@Override
	public void removeQueryParameters(String name)
	{
		setURI(URIStringUtils.removeQueryParameter(getUriString(), name));
	}
	
	@Override
	public void removeRequestHeaders(String name)
	{
		unvalidatedRequest.removeHeaders(name);
	}
	
	/**
	 * This will only interact with the URI query string
	 */
	@Override
	public void replaceQueryParameter(NameValuePair parameter)
	{
		
		RequestLine oldLine = unvalidatedRequest.getRequestLine();
		String uri = oldLine.getUri();
		uri = uri.replaceFirst("^([^?]*\\?.*?" + Pattern.quote(parameter.getName()) + "=)[^&]++", "$1" + parameter.getValue());
		SerializableRequestLine line = new SerializableRequestLine(oldLine.getMethod(), uri, oldLine.getProtocolVersion());
		unvalidatedRequest.setRequestLine(line);
	}
	
	/**
	 * This will only interact with the URI query string
	 */
	@Override
	public void replaceQueryParameter(String name, String value)
	{
		NameValuePair param = new BasicNameValuePair(name, value);
		replaceQueryParameter(param);
	}
	
	public void setRequest(UnvalidatedHttpRequest request)
	{
		unvalidatedRequest = request;
	}
	
	public void setUnvalidatedRequest(UnvalidatedHttpRequest unvalidatedRequest)
	{
		this.unvalidatedRequest = unvalidatedRequest;
	}
	
	@Override
	public void setURI(String uri)
	{
		SerializableRequestLine oldLine = unvalidatedRequest.getRequestLine();
		SerializableRequestLine newLine = new SerializableRequestLine(oldLine.getMethod(), uri, oldLine.getProtocolVersion());
		unvalidatedRequest.setRequestLine(newLine);
	}
	
	// Finish
	@Override
	protected void privateExecute(CustomHttpClient client, HttpContext context, boolean testTransaction,
	        String executionSource)
	{
		
		try
		{
			sentRequestHead = scan.getRawHttpClient().getRequestHeadString(unvalidatedRequest.getTarget(), unvalidatedRequest, context);
			HttpResponse response = scan.getRawHttpClient().customExecute(unvalidatedRequest.getTarget(), unvalidatedRequest, context);
			executionTime = (new java.util.Date()).getTime();
			if (response != null)
			{
				setResponse(response);
				successfullExecution = true;
			}
		}
		catch (HttpException e)
		{
			Debug.errDebug("HttpException: " + e.toString(), e);
		}
		catch (IOException e)
		{
			Debug.errDebug("IOException: " + e.toString(), e);
		}
		catch (InterruptedException e)
		{
			Debug.errDebug("InterruptedException: " + e.toString(), e);
		}
	}

	@Override
    public RequestLine getRequestLine()
    {
	    return unvalidatedRequest.getRequestLine();
    }

	@Override
    public byte[] getRequestBody()
    {
		if (requestBody == null)
		{
			return new byte[0];
		}
	    return requestBody;
    }

	@Override
    public String requestToString()
    {
	    String request =  super.requestToString();
	    request += new String(getRequestBody());
	    return request;
    }

}
