package com.grendelscan.requester.queue;


import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Queue;
import java.util.concurrent.LinkedBlockingQueue;

import com.grendelscan.requester.TransactionRecord;
import com.grendelscan.requester.authentication.AuthenticationPackage;
import com.grendelscan.requester.http.client.CustomHttpClient;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.sessionState.SessionState;
import com.grendelscan.requester.sessionState.SessionStateUtils;
import com.grendelscan.scan.Scan;
import com.grendelscan.scan.abstractQueue.AbstractScanQueue;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.StringUtils;
/**
 * 
 * @author David Byrne
 */
public class RequesterQueue extends AbstractScanQueue
{
	private int loggedOutCompareThreshold = 90;
	private LinkedBlockingQueue<Integer> queue;
	private Scan scan;
	private TransactionRecord transactionRecord;
	private int maxRequestDepth;
	private Map<String, Integer> failureCount;
	private Map<SessionState, SessionState> newSessions;
    private Map<String, Integer> sourceCounts;
    private boolean running = true;
	public boolean isRunning()
    {
    	return running;
    }

	public void setRunning(boolean running)
    {
    	this.running = running;
    }

	/** Creates a new instance of requesterQueue */
	public RequesterQueue(Scan scan)
	{
		queue = new LinkedBlockingQueue<Integer>();
		this.scan = scan;
		failureCount = new HashMap<String, Integer>(1);
		transactionRecord = scan.getTransactionRecord();
		maxRequestDepth = scan.getScanSettings().getMaxRequestDepth();
		newSessions = new HashMap<SessionState, SessionState>();
		sourceCounts = new HashMap<String, Integer>(1);

	}

	public void addRequests(Collection<AbstractHttpTransaction> requests, String source)
	{
		for (AbstractHttpTransaction transaction: requests)
		{
			addRequest(transaction, source);
		}
	}
	

	public boolean addRequest(AbstractHttpTransaction transaction, String source)
	{
//if (!source.equals("Module 2 - Directory only"))
//{
//	System.err.println("Debug");
//	
//}
		Boolean result = false;
		if (!queueLocked)
		{
			if (transaction != null)
			{
				if (isScopeAndDepthGood(transaction))
				{
					try
					{
						Debug.debug(((source != null && !source.equals(""))? source + " a": "A") + "dded to request queue: " + transaction.getUriString());
						transaction.save();
						queue.put(transaction.getId());
						result = true;
					}
					catch (InterruptedException e)
					{
						Debug.debug("Threading problem in RequesterQueue.addRequest(HttpTransaction): " + e.toString());
					}
				}
			}
		}
		
		return result;
	}


	public boolean isScopeAndDepthGood(AbstractHttpTransaction transaction)
	{
		boolean good = false;
		if (
				transaction.getRequestDepth() <= maxRequestDepth 
				&&
				scan.getScanSettings().getUrlFilters().isUriAllowed(transaction.getUriString())
//				&&
//				scan.getScanSettings().getUrlFilters().isExtensionOkay(transaction.getFilename())
			)
		{
			good = true;
		}
		else
		{
			Debug.debug("out of scope or too deep URI: " + transaction.getUriString());
		}
		
		return good;
	}

	
	/**
	 * 
	 * @param request
	 * @param ignoreUser If true, the request will not be sent if it has already 
	 * been requested with another user account.
	 * @return
	 */
	public boolean addSpiderRequest(AbstractHttpTransaction request, boolean ignoreUser, String source)
	{
//if (!source.equals("Module 2 - Directory only"))
//{
//	System.err.println("Debug");
//	
//}
		Boolean result = false;
		if (request != null)
		{
			String uri = request.getAbsoluteUriString();
			if (request.getMethod().equalsIgnoreCase("POST"))
			{
				uri += "&" + request.getPayloadQueryString();
			}
			
			if (scan.getTransactionRecord().hasUriBeenRequested(uri, request.getUsername(), ignoreUser))
			{
//				scan.debug("already sent: " + request.getUriString());
			}
			else
			{
				result = addRequest(request, source);
			}
		}
		return result;
	}


	public AbstractHttpTransaction getNext()
	{
		AbstractHttpTransaction transaction = null;
		if (!queueLocked)
		{
			try
			{
				transaction = scan.getTransactionRecord().getTransaction(queue.take());
			}
			catch (InterruptedException e)
			{
			}
		}
		return transaction;
	}

	
	public boolean processRequestNow(AbstractHttpTransaction request, String source, boolean testTransaction, boolean onlyIfUriNew, boolean ignoreRestrictions, boolean ignoreUsername)
	{
		boolean wasSent = false;
		if (! onlyIfUriNew || ! transactionRecord.hasUriBeenRequested(request.getAbsoluteUriString(), request.getUsername(), ignoreUsername))
		{
			wasSent = sendRequest(request, source, scan.getHttpClient(), testTransaction, ignoreRestrictions);
			if (testTransaction && wasSent)
			{
				scan.getCategorizerQueue().addTransaction(request);
			}
		}

		return wasSent;
	}


	public boolean sendRequest(AbstractHttpTransaction request, String source, CustomHttpClient client, boolean testTransaction)
	{
		return sendRequest(request, source, client, testTransaction, false);
	}

	private void incSource(String source)
	{
		Integer i = sourceCounts.get(source);
		if (i == null)
		{
			i = 0;
		}
		i++;
		sourceCounts.put(source, i);
	}
	
	public boolean sendRequest(AbstractHttpTransaction transaction, String source, CustomHttpClient client, boolean testTransaction, boolean ignoreRestrictions)
	{
		boolean wasSent = false;
		if (ignoreRestrictions || isScopeAndDepthGood(transaction))
		{
			Debug.debug(((source != null && !source.equals(""))? source + " is r": "R") + "equesting " + transaction.getMethod() + " " + transaction.getUriString());
			incSource(source);
			if (transaction.getAuthenticationPackage() != null)
			{
				updateSessionStates(transaction);
			}
			transaction.execute(client, null, testTransaction, source);
			if (!transaction.isSuccessfullExecution())
			{
				int oldCount = 0;
				if (failureCount.containsKey(transaction.getHostUri()))
				{
					oldCount = failureCount.get(transaction.getHostUri());
				}
				if (++oldCount >= scan.getScanSettings().getMaxFailedRequestsPerHost())
				{
					scan.fatalError("Too many failed requests to " + transaction.getHostUri());
					return false;
				}
				failureCount.put(transaction.getHostUri(), oldCount);
			}
			else if (transaction.getResponseStatusCode() == 407)
			{
				scan.fatalError("Upstream proxy authentication required");
				return false;
			}
			else
			{
				wasSent = true;
			}
		}
//		if (wasSent)
//		{
//			if (scan.getScanSettings().isUseLoggedOutDetection() && transaction.getAuthenticationPackage() != null)
//			{ 
//				if (StringUtils.scoreStringDifferenceIgnoreCase(transaction.getResponseTextNodes(), transaction.getAuthenticationPackage().getLoggedOutPageText(), 100) > loggedOutCompareThreshold)
//				{
//					Debug.debug("A session was logged out. Getting a new session ID and trying again.");
//					AuthenticationPackage authPackage = transaction.getAuthenticationPackage();
//					AbstractHttpTransaction loginTransaction = authPackage.createLoginTransaction(transaction.getUsername());
//					loginTransaction.execute(client, null, false, "Re-login for " + source);
//					for(String sessionIDName: authPackage.getSessionIDNames())
//					{
//						SessionState newSession = SessionStateUtils.findNewSessionIDInResponse(sessionIDName, loginTransaction);
//						newSessions.put(transaction.getSessionStates().get(sessionIDName), newSession);
//					}
//					
//					updateSessionStates(transaction);
//				}
//			}
//		}

		return wasSent;
	}
	
	private void updateSessionStates(AbstractHttpTransaction transaction)
	{
		for (SessionState oldSession: transaction.getSessionStates().values())
		{
			SessionState sessionUpdates = newSessions.get(oldSession);
			
			if (sessionUpdates != null)
			{
				/*
				 * This is probably a dumb way to solve the problem, but I'm 
				 * on a midnight flight from Chicago to Denver, so too bad.
				 * The idea is to follow the chain of new sessions until the
				 * end is reached
				 */
				while(newSessions.containsKey(sessionUpdates))
				{
					sessionUpdates = newSessions.get(sessionUpdates);
				}
				transaction.replaceSessionState(newSessions.get(oldSession));
			}
		}
	}

	@Override
    protected Queue<Integer> getQueue()
    {
	    return queue;
    }
	

}
