package com.grendelscan.tests.libraries.SessionIDTesting;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.cookie.Cookie;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.CollectionUtils;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HttpUtils;

public class SessionID
{
	private static int maxCookieRequests;
	private static Map<String, int[]> cookieSamples;
	private static int maxSessionIDRequests;
	private static float minAcceptableBitChangePercent;
	private static int minAcceptableBitLength;
	private static float minAcceptableNetBitChangePercent;
	private static List<String> notSessionIDs;
	private static List<String> fixationTestPatterns;
	private static Scan scan;
	private static Set<String> testedSessionIDs;
	private static List<Pattern> knownSessionIDRegexs;
	
	public static void initializeLibrary(Scan scan)
	{
		SessionID.scan = scan;
		maxSessionIDRequests = ConfigurationManager.getInt("session_id_analysis.max_session_id_requests");
		minAcceptableBitLength = ConfigurationManager.getInt("session_id_analysis.min_acceptable_bit_length");
		minAcceptableBitChangePercent = ConfigurationManager.getFloat("session_id_analysis.min_bit_change_percentage");
		minAcceptableNetBitChangePercent = ConfigurationManager.getFloat("session_id_analysis.min_net_bit_change_percentage");
		notSessionIDs = ConfigurationManager.getList("session_id_analysis.not_session_ids");
		fixationTestPatterns = ConfigurationManager.getList("session_id_analysis.session_fixation_patterns");
		maxCookieRequests = SessionID.getMaxSessionIDRequests();
		cookieSamples = new HashMap<String, int[]>(1);
		knownSessionIDRegexs = scan.getScanSettings().getKnownSessionIDRegexs();
		testedSessionIDs = Collections.synchronizedSet(new HashSet<String>());
	}
	
	
	public static void addKnownSessionID(String sessionID)
	{
		Pattern p = Pattern.compile(Pattern.quote(sessionID), Pattern.CASE_INSENSITIVE);
		knownSessionIDRegexs.add(p);
	}
	
	public static boolean isKnownSessionID(String candidate)
	{
		boolean known = false;
		
		for (Pattern p: knownSessionIDRegexs)
		{
			if (p.matcher(candidate).find())
			{
				known = true;
				break;
			}
		}
		
		return known;
	}
	

	/**
	 * 
	 * @param testSessionIDValues
	 * @param bitChanges
	 * @return -1 if the bits are okay, otherwise the could of bits
	 */
	public static int getSessionIDBitEntropy(List<String> testSessionIDValues)
	{
		return getSessionIDBitEntropy(CollectionUtils.toStringArray(testSessionIDValues));
	}
	
	/**
	 * 
	 * @param testSessionIDValues
	 * @param bitChanges
	 * @return -1 if the bits are okay, otherwise the could of bits
	 */
	public static int getSessionIDBitEntropy(String[] testSessionIDValues)
	{
		int bitChanges = -1;
		boolean bad = false;
		for (int index = 0; index < testSessionIDValues.length - 1; index++)
		{
			if (testSessionIDValues[index] == null || testSessionIDValues[index + 1] == null)
			{
//				Debug.errDebug("Null session ID.", new Throwable());
				continue;
			}
			bitChanges = getBitChangeCount(testSessionIDValues[index], testSessionIDValues[index + 1]);
			if (bitChanges < minAcceptableBitChangePercent * minAcceptableBitLength)
			{
				bad = true;
				break;
			}
		}

		if (!bad)
		{
			bitChanges = getNetBitChangeCount(testSessionIDValues);
			if (bitChanges < minAcceptableNetBitChangePercent * minAcceptableBitLength)
			{
				bad = true;
			}
		}
		
		if (!bad) 
		{
			bitChanges = -1;
		}
		
		return bitChanges;
	}

	public static boolean findDuplicateSessionIDs(Iterable<String> testSessionIDValues)
	{
		return findDuplicateSessionIDs(CollectionUtils.toStringArray(testSessionIDValues));
	}

	public static boolean findDuplicateSessionIDs(String[] testSessionIDValues)
	{
		boolean duplicate = false;
		HashSet<String> cookies = new HashSet<String>(maxSessionIDRequests + 1);

		for (int index = 0; index <= maxSessionIDRequests; index++)
		{
			if (cookies.contains(testSessionIDValues[index]))
			{
				duplicate = true;
				break;
			}
			cookies.add(testSessionIDValues[index]);
		}
		
		return duplicate;
	}

	public static int getBitChangeCount(String stringA, String stringB)
	{
		String[] testStrings = { stringA, stringB };
		return getNetBitChangeCount(testStrings);
	}

	public static int getBitSetCount(int target)
	{
		int bits = 0;
		/*
		 * Interesting trivia from
		 * http://graphics.stanford.edu/~seander/bithacks.html#CountBitsSetKernighan
		 * 
		 * Published in 1988, the C Programming Language 2nd Ed. (by Brian W.
		 * Kernighan and Dennis M. Ritchie) mentions this in exercise 2-9. On
		 * April 19, 2006 Don Knuth pointed out to me that this method "was
		 * first published by Peter Wegner in CACM 3 (1960), 322. (Also
		 * discovered independently by Derrick Lehmer and published in 1964 in a
		 * book edited by Beckenbach.)"
		 */
		for (bits = 0; target > 0; bits++)
		{
			target &= target - 1; // clear the least
			// significant bit set
		}

		return bits;
	}

	public static int getNetBitChangeCount(String[] testSessionIDValues)
	{
		int minLength;
		int bitsSet = 0;
		byte[] currentBytes, originalBytes;
		int[] netBytes;

		minLength = testSessionIDValues[0].length();
		for (int index = 1; index < testSessionIDValues.length; index++)
		{
			if (testSessionIDValues[index] != null && testSessionIDValues[index].length() < minLength)
			{
				minLength = testSessionIDValues[index].length();
			}
		}

		originalBytes = testSessionIDValues[0].getBytes();
		netBytes = new int[minLength];
		// Copy originalBytes into netBytes
		for (int byteIndex = 0; byteIndex < minLength; byteIndex++)
		{
//			netBytes[byteIndex] = originalBytes[byteIndex];
			netBytes[byteIndex] = 0;
		}

		// This tracks which bits have changed at some point across the requests
		// compared to the first request
		for (int index = 1; index < testSessionIDValues.length; index++)
		{
			currentBytes = testSessionIDValues[index].getBytes();
			for (int byteIndex = 0; byteIndex < minLength; byteIndex++)
			{
				netBytes[byteIndex] = netBytes[byteIndex] | (originalBytes[byteIndex] ^ currentBytes[byteIndex]);
			}
		}

		for (int byteIndex = 0; byteIndex < minLength; byteIndex++)
		{
			bitsSet += getBitSetCount(netBytes[byteIndex]);
		}

		return bitsSet;
	}
	

	/**
	 * Looks for SessionIDs in Set-Cookie parameters and the response body. 
	 * If there is a cookie and a URL parameter sessionID, only the cookie location will be returned
	 * @param transaction
	 * @return
	 */
	public static SessionIDLocation findSessionIDLocation(AbstractHttpTransaction transaction)
	{
		SessionIDLocation containsID = SessionIDLocation.NO_SESSION_ID;
		
		if (transaction.getResponse() != null)
		{
			for (Cookie cookie: HttpUtils.getSetCookies(transaction))
			{
				containsID = SessionIDLocation.COOKIE_SESSION_ID;
				for (String knownID: notSessionIDs)
				{
					if (cookie.getName().equalsIgnoreCase(knownID))
					{
						containsID = SessionIDLocation.NO_SESSION_ID;
						break;
					}
				}
				if (containsID == SessionIDLocation.COOKIE_SESSION_ID)
				{
					break;
				}
			}
			
			if (containsID == SessionIDLocation.NO_SESSION_ID)
			{
				String body = transaction.getResponseBodyAsString();
				
				if (stringContainsURLSessionID(body))
				{
					containsID = SessionIDLocation.URL_SESSION_ID;
				}
			}
		}
		return containsID; 
	}
	
	public static boolean transactionRequestContainsCookie(AbstractHttpTransaction transaction, String cookieName)
	{
		boolean contains = false;
		
		for (Cookie cookie: transaction.getUsedCookies())
		{
			if (cookie.getName().equalsIgnoreCase(cookieName))
			{
				contains = true;
				break;
			}
		}
		return contains;
	}

	public static Set<String> findUrlSessionIDs(String htmlBody)
	{
		Set<String> ids = new HashSet<String>();
		
		for(Pattern sessionIDPattern: knownSessionIDRegexs)
		{
			Pattern p = Pattern.compile("\\b(" + sessionIDPattern.pattern() + ")['\"\\s]*=", Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(htmlBody);
			while (m.find())
			{
				ids.add(m.group(1));
			}
		}
		
		return ids;
	}
	
	public static void findUrlSessionIDsAndValues(String htmlBody, Map<String, List<String>> sessionIDs)
	{
		
		for(Pattern sessionIDPattern: knownSessionIDRegexs)
		{
			Pattern p = Pattern.compile("\\b(" + sessionIDPattern.pattern() + ")['\"\\s]*=['\"\\s]*([\\w\\-_]+)", Pattern.CASE_INSENSITIVE);
			Matcher m = p.matcher(htmlBody);
			while (m.find())
			{
				String id = m.group(1);
				List<String> values;
				if (! sessionIDs.containsKey(id))
				{
					values = new ArrayList<String>();
					sessionIDs.put(id, values);
				}
				else
				{
					values = sessionIDs.get(id);
				}
				
				if (!values.contains(m.group(2)))
				{
					values.add(m.group(2));
				}
			}
		}
	}
	
	public static boolean stringContainsURLSessionID(String htmlBody)
	{
		boolean contains = false;
		
		for(Pattern sessionIDPattern: knownSessionIDRegexs)
		{
			Pattern p = Pattern.compile("\\b" + sessionIDPattern.pattern() + "['\"\\s]*=", Pattern.CASE_INSENSITIVE);
			if (p.matcher(htmlBody).find())
			{
				contains = true;
				break;
			}
		}
		
		return contains;
	}

	public static int getMaxSessionIDRequests()
    {
    	return maxSessionIDRequests;
    }


	public static float getMinAcceptableBitChangePercent()
    {
    	return minAcceptableBitChangePercent;
    }


	public static int getMinAcceptableBitLength()
    {
    	return minAcceptableBitLength;
    }


	public static float getMinAcceptableNetBitChangePercent()
    {
    	return minAcceptableNetBitChangePercent;
    }


	public static List<String> getNotSessionIDs()
    {
    	return notSessionIDs;
    }


	public static List<String> getFixationTestPatterns()
    {
    	return fixationTestPatterns;
    }
/*	
	public static Set<String> getCookieNames(AbstractHttpTransaction transaction)
	{
		HashSet<String> cookies = new HashSet<String>();
	
		Pattern cookiePattern =
	        Pattern
	                .compile("((?:\"[^\"]+\")|(?:[a-zA-Z0-9\\-_]+))\\s*=\\s*((?:\"[^\"]+\")|(?:[^\\S;]+))\\s*;?+\\s*");
		Matcher matcher = cookiePattern.matcher(transaction.getFirstRequestHeader("Cookie").getValue());
		while (matcher.find())
		{
			cookies.add(matcher.group(1));
		}
		return cookies;
	}
	*/


	public static Set<String> getTestedSessionIDs()
    {
    	return testedSessionIDs;
    }
	
	/**
	 * Takes a transaction, looks for set cookies, tests to see if the cookie values are 
	 * changing, then adds them the list of known session IDs if they are 
	 * @param transaction
	 * @param source Execution source
	 */
	public static void seedSessionIDDatabase(AbstractHttpTransaction transaction, String source)
	{
		int[] testTransactions = getSampleTransactions(transaction, source);
		for (Cookie cookie: HttpUtils.getSetCookies(transaction))
		{
			if (isKnownSessionID(cookie.getName()))
			{
				cookieSamples.put(getCookieFingerprint(cookie), testTransactions);
			}
		}
	}
	
	/**
	 * Will return an array of sample cookie transactions. It will also add any 
	 * changing cookie values to the list of known session IDs
	 * 
	 * @param transaction The transaction to duplicate if no cookie samples are available
	 * @param exampleCookie The type of cookie being sought. The name, domain and path are considered.
	 * @param source The source to use when executing new requests
	 * @return
	 */
	public static AbstractHttpTransaction[] getSampleCookieTransactions(AbstractHttpTransaction transaction, Cookie exampleCookie, String source)
	{
		String exampleCookieFingerprint = getCookieFingerprint(exampleCookie);
		if (!cookieSamples.containsKey(exampleCookieFingerprint))
		{
			if (transaction.getResponseHeaders("Set-Cookie").length > 0)
			{
				seedSessionIDDatabase(transaction, source);
			}
			else
			{
				throw new IllegalArgumentException("No cookies were found, and the transaction didn't have any set-cookie response headers.");
			}
		}
		if (!cookieSamples.containsKey(exampleCookieFingerprint))
		{
			throw new IllegalArgumentException("No cookies retrieved.");
		}
		AbstractHttpTransaction cookieTransactions[] = new AbstractHttpTransaction[maxCookieRequests + 1];
		int index = 0;
		for (int transactionID: cookieSamples.get(exampleCookieFingerprint))
		{
			AbstractHttpTransaction cookieTransaction = scan.getTransactionRecord().getTransaction(transactionID);
			cookieTransactions[index++] = cookieTransaction;
		}
		scan.getCategorizerQueue();
		return cookieTransactions;
	}
	
	private static String getCookieFingerprint(Cookie cookie)
	{
		return cookie.getName() + cookie.getDomain() + cookie.getPath();
	}
	

	private static int[] getSampleTransactions(AbstractHttpTransaction transaction, String source)
	{
		AbstractHttpTransaction testTransactions[] = new AbstractHttpTransaction[maxCookieRequests + 1];
		int[] testTransactionIDs = new int[maxCookieRequests + 1];
		testTransactionIDs[0] = transaction.getId();
		testTransactions[0] = transaction;
		
		for (int index = 1; index <= maxCookieRequests; index++)
		{
			AbstractHttpTransaction testTransaction = transaction.clone(TransactionSource.TEST);
			testTransaction.setSource(TransactionSource.TEST);
			/*
			 * The craw client is used because it will not use connection handling, 
			 * causing the connection to be closed after each request. This is 
			 * required because some web apps will attach session state with a TCP
			 * session. Very stupid, but it still happens in the real world. 
			 */
			// This header stuff is because it has to be sent with the raw client
			if (testTransaction.getMethod().equalsIgnoreCase("POST"))
			{
				int length = transaction.getRequestBody().length;
				testTransaction.addRequestHeader("Content-Length", String.valueOf(length));
			}
			testTransaction.addRequestHeader("Host", transaction.getHost());
			testTransaction.setFollowRedirects(false);
			
			scan.getRequesterQueue().sendRequest(testTransaction, "Sample cookie fetcher on behalf of " + source, scan.getRawHttpClient(), false, false);
//			testTransaction.execute("Sample cookie fetcher on behalf of " + source, false, false);
			testTransactionIDs[index] = testTransaction.getId();
			testTransactions[index] = testTransaction;
			
			if (index == 1)
			{
				try
				{
					/*
					 * Pauses to let the second counter increment on the web
					 * server. This is because a poorly written app may generate
					 * identical session ids in a short time period. We'll pause
					 * for the first request, but subsequent requests will be
					 * executed quickly to try and catch a duplicate value.
					 */
					Thread.sleep(1500);
				}
				catch (InterruptedException e)
				{
				}
			}
		}
		
		for (Cookie cookie: HttpUtils.getSetCookies(transaction))
		{
			/*
			 * If it isn't already known as a session ID, and if it isn't a 
			 * a known non-session ID, and if the value is changing
			 */
			if (!SessionID.isKnownSessionID(cookie.getName())
					&& !notSessionIDs.contains(cookie.getName()) 
					&& isValueChanging(testTransactions, cookie.getName()))
			{
				SessionID.addKnownSessionID(cookie.getName());
			}
		}
		
		return testTransactionIDs;
	}
	
	private static boolean isValueChanging(AbstractHttpTransaction[] testTransactions, String cookieName)
	{
		boolean valueChanged = true;
		
		Cookie testCookies[] = getSetCookiesByName(testTransactions, cookieName);
		
		for (int index = 1; index <= maxCookieRequests; index++)
		{
			// If the value is changing with different
			// requests, then it should be tested
			
			if (! scan.isRunning())
			{
				break;
			}
			if (testCookies[index] == null)
			{
//				Debug.errDebug("No cookie where one expected in index " + index, new Throwable());
				continue;
			}
			if (testCookies[index].getValue().equals(testCookies[index - 1].getValue()))
			{
				valueChanged = false;
				break;
			}
		}
		return valueChanged;
	}

	public static Cookie[] getSetCookiesByName(AbstractHttpTransaction[] testTransactions, String cookieName)
	{
		Cookie cookies[] = new Cookie[maxCookieRequests + 1];
		
		for (int index = 0; index <= maxCookieRequests; index++)
		{
			boolean foundCookie = false;
			for (Cookie setCookie: HttpUtils.getSetCookies(testTransactions[index]))
			{
				if (!scan.isRunning())
				{
					return cookies;
				}
				if (setCookie.getName().equals(cookieName))
				{
					cookies[index] = setCookie;
					foundCookie = true;
					break;
				}
			}
			if (!foundCookie)
			{
//				Debug.errDebug("Cookie not found where one was expected.", new Throwable());
				continue;
			}
		}
		
		return cookies;
	}
	

	
}
