package com.grendelscan.tests.libraries.TokenTesting;


import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.Header;
import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.grendelscan.html.HtmlNodeWriter;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.Scan;
import com.grendelscan.utils.CollectionUtils;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.MimeUtils;
import com.grendelscan.utils.StringUtils;

/**
 * This is used to track when tokens are submited in a query to probe where they
 * end up. It is mostly useful for XSS and HTTP response splitting testing.
 * 
 * @author David Byrne
 * 
 */
public class TokenTesting
{
	private static int tokenPrefixLength = ConfigurationManager.getInt("token_testing.token_prefix_length");
	private static int tokenSuffixLength = ConfigurationManager.getInt("token_testing.token_suffix_length");
	private static String tokenPrefix =
        StringUtils.generateRandomString(StringUtils.FORMAT_UPPER_CASE_ALPHA, tokenSuffixLength);
	private static Map<String, Integer> tokenTests;
	private static Scan scan;
	


	public static void initializeLibrary(Scan scan)
    {
		tokenTests = Collections.synchronizedMap(new HashMap<String, Integer>());
    	TokenTesting.scan = scan;
    }

	/**
	 * Returns a set of the TokenContexts found in the transaction. All recorded
	 * tokens are searched for.
	 * 
	 * @param transaction
	 * @return
	 */
	public static Set<TokenContext> findTokenContexts(AbstractHttpTransaction transaction)
	{
		synchronized (tokenTests)
		{
			return findTokenContexts(transaction, tokenTests.keySet());
		}
	}

	/**
	 * This takes a discovered context, and repeats the input transaction (where the token is submitted) 
	 * and output transaction (where the token is observed in the result) with a new token. The new token could
	 * also be an attack string. The new output transaction is returned. The original input and output 
	 * transaction can be the same object. The new transactions are not submitted for testing.
	 * @param context
	 * @param newToken
	 * @return Element 0 is the new input transaction, element 1 is the new output transaction. They may
	 * be the same object, but both elements will be populated
	 */
	public static AbstractHttpTransaction[] duplicateTokenTest(TokenContext context, String newToken, 
			String stringSource, TransactionSource source)
	{
		AbstractHttpTransaction originalOutputTransaction = scan.getTransactionRecord().getTransaction(context.getOutputTransactionID());
		AbstractHttpTransaction originalInputTransaction = TokenTesting.getTokenTest(context.getToken()); 
		AbstractHttpTransaction trans[] = new AbstractHttpTransaction[2];
		if (originalInputTransaction.getId() == originalOutputTransaction.getId())
		{
			AbstractHttpTransaction newTestTransaction = originalOutputTransaction.clone(source);
			newTestTransaction.replaceQueryParameterValue(context.getToken(), newToken);
			String uri = newTestTransaction.getAbsoluteUriString();
			uri = uri.replace(context.getToken(), newToken);
			newTestTransaction.setURI(uri);
			
			newTestTransaction.execute("TokenTesting.duplicateTokenTest on behalf of " + stringSource, false);
			trans[0] = newTestTransaction;
			trans[1] = newTestTransaction;
		}
		else
		{
			AbstractHttpTransaction newInputTransaction = originalInputTransaction.clone(source);
			newInputTransaction.replaceQueryParameterValue(context.getToken(), newToken);
			String uri = newInputTransaction.getAbsoluteUriString();
			uri = uri.replace(context.getToken(), newToken);
			newInputTransaction.setURI(uri);
			newInputTransaction.execute("TokenTesting.duplicateTokenTest on behalf of " + stringSource, false);
			
			AbstractHttpTransaction newOutputTransaction = originalOutputTransaction.clone(source);
			newOutputTransaction.execute("TokenTesting.duplicateTokenTest on behalf of " + stringSource, false);
			trans[0] = newInputTransaction;
			trans[1] = newOutputTransaction;
		}
		
		return trans;
	}
	
	
	/**
	 * Returns a set of the TokenContexts found in the transaction. Only the
	 * specified tokens are searched for.
	 * 
	 * @param transaction
	 * @param tokens
	 * @return
	 */
	public static Set<TokenContext> findTokenContexts(AbstractHttpTransaction transaction, Set<String> tokens)
	{
//		Set<TokenContext> contexts = new HashSet<TokenContext>();
		Map<String, TokenContext> contexts = new HashMap<String, TokenContext>();
		if (transaction.isResponsePresent())
		{
			parseHeaders(transaction, contexts, tokens);
			if (scan.isRunning())
			{
				parseByMimeType(transaction, contexts, CollectionUtils.toStringArray(tokens));
			}
		}
//		if (contexts.size() > 1)
//		{
//			System.err.println("Debug");
//		}
		return new HashSet<TokenContext>(contexts.values());
//		return contexts;
	}
	
	/**
	 * Returns a set of the TokenContexts found in the transaction. Only the
	 * specified token is searched for.
	 * 
	 * @param transaction
	 * @param tokens
	 * @return
	 */
	public static Set<TokenContext> findTokenContexts(AbstractHttpTransaction transaction, String token)
	{
		Set<String> tokens = new HashSet<String>(1);
		tokens.add(token);
		return findTokenContexts(transaction, tokens);
	}
	
	
	/**
	 * Confirms that repeating the same requests with a different token will result in the same
	 * token context.
	 * @param transaction
	 * @param originalContext
	 * @return
	 */
	public static boolean verifyTokenRepeatability(TokenContext originalContext, String stringSource, TransactionSource source)
	{
		boolean repeatable = false;
		
		
		String newToken = generateToken();
		Set<String> newTokenSet = new HashSet<String>();
		newTokenSet.add(newToken);
		AbstractHttpTransaction testTransactions[] = duplicateTokenTest(originalContext, newToken, stringSource, source);
		AbstractHttpTransaction newInputTransaction = testTransactions[0];
		AbstractHttpTransaction newOutputTransaction = testTransactions[1];
		recordTokenTest(newToken, newInputTransaction);
		
		Set<TokenContext> newContexts = findTokenContexts(newOutputTransaction, newTokenSet);
		for (TokenContext newContext: newContexts)
		{
			if (newContext.getContextType().equals(originalContext.getContextType()))
			{
				repeatable = true;
				break;
			}
		}
		
		return repeatable;
	}
	

	public static String generateToken()
	{
		return tokenPrefix
		        + StringUtils.generateRandomString(StringUtils.FORMAT_UPPER_CASE_ALPHA, tokenPrefixLength);
	}

	public static String getTokenPrefix()
	{
		return tokenPrefix;
	}

	public static int getTokenPrefixLength()
	{
		return tokenPrefixLength;
	}

	public static int getTokenSuffixLength()
	{
		return tokenSuffixLength;
	}

	public static AbstractHttpTransaction getTokenTest(String token)
	{
		int ID;
		synchronized (tokenTests)
		{
			ID = tokenTests.get(token);
		}
		return scan.getTransactionRecord().getTransaction(ID);
	}

	public static void recordTokenTest(String token, AbstractHttpTransaction transaction)
	{
		synchronized (tokenTests)
		{
			tokenTests.put(token, transaction.getId());
		}
	}

	private static void addNodeResult(String token, Node node, int outputTransactionID, Map<String, TokenContext> results)
//	private static void addNodeResult(String token, Node node, int outputTransactionID, Set<TokenContext> results)
	{
		for (TokenContextType contextType: findNodeTokenContextTypes(token, node))
		{
			if (results.containsKey(token + contextType))
			{
				break;
			}
			Set<String> quotes = new HashSet<String>(1); 
			AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(outputTransactionID);
			String body = transaction.getResponseBodyAsString();
			Pattern p = Pattern.compile(token + "(['\"])");
			Matcher m = p.matcher(body);
			while (m.find())
			{
				quotes.add(m.group(1));
			}
			String quoteChars[] = new String[quotes.size()];
			int index = 0;
			for (String quote: quotes)
			{
				quoteChars[index++] = quote;
			}
			HtmlContext context = new HtmlContext(token, contextType, outputTransactionID, quoteChars);
			results.put(token + contextType, context);
//			results.add(context);
		}
	}

	private static Set<TokenContextType> findNodeTokenContextTypes(String token, Node node)
	{
		HashSet<TokenContextType> contexts = new HashSet<TokenContextType>();
		String nodeString = HtmlNodeWriter.write(node);
		String nodeName = node.getNodeName();
		String nodeValue = node.getNodeValue();
		int nodeType = node.getNodeType();

		if (nodeString.toUpperCase().contains(token.toUpperCase()))
		{
			if (nodeName.toUpperCase().contains(token.toUpperCase()))
			{
				if (node instanceof Attr)
				{
					contexts.add(TokenContextType.HTML_TAG_ATTRIBUTE_NAME);
				}
				else
				{
					contexts.add(TokenContextType.HTML_TAG_NAME);
				}
			}
			if (nodeValue.toUpperCase().contains(token.toUpperCase()))
			{
				if (node instanceof Attr)
				{

					if (HtmlUtils.isDOMEvent(nodeName))
					{
						contexts.add(TokenContextType.HTML_EVENT_HANDLER);
					}
					else
					{
						contexts.add(TokenContextType.HTML_TAG_ATTRIBUTE_VALUE);
					}
				}
				else
				{
					if (nodeType == Node.TEXT_NODE)
					{
						if (node.getParentNode().getNodeName().equalsIgnoreCase("SCRIPT"))
						{
							contexts.add(TokenContextType.HTML_SCRIPT);
						}
						else if (node.getParentNode().getNodeName().equalsIgnoreCase("STYLE"))
						{
							contexts.add(TokenContextType.HTML_STYLE);
						}
						else if (node.getParentNode().getNodeName().equalsIgnoreCase("TEXTAREA"))
						{
							contexts.add(TokenContextType.HTML_TEXTAREA);
						}
						else if (node.getParentNode().getNodeName().equalsIgnoreCase("PRE"))
						{
							contexts.add(TokenContextType.HTML_PRE);
						}
						else if (node.getParentNode().getNodeName().equalsIgnoreCase("TITLE"))
						{
							contexts.add(TokenContextType.HTML_TITLE);
						}
						else
						{
							contexts.add(TokenContextType.HTML_TEXT);
						}

					}
					else if (nodeType == Node.COMMENT_NODE)
					{
						contexts.add(TokenContextType.HTML_COMMENT);
					}
				}
			}
		}

		return contexts;
	}


	private static void parseByMimeType(AbstractHttpTransaction transaction, Map<String, TokenContext> contexts,
//	private static void parseByMimeType(AbstractHttpTransaction transaction, Set<TokenContext> contexts,
	        String tokens[])
	{
		
		if (! transaction.getResponseBodyAsString().toLowerCase().contains(tokenPrefix.toLowerCase()))
		{
			return;
		}
		String mimeType = transaction.getResponseMimeType().toLowerCase();
		
		if (MimeUtils.isHtmlMimeType(mimeType))
		{
			recursiveNodeSearch(tokens, transaction.getResponseDOM(), contexts, transaction.getId());
		}
		else
		{
			String responseBody = transaction.getResponseBodyAsString().toUpperCase();
			for (String token: tokens)
			{
				if (responseBody.contains(token))
				{
					TokenContextType contextType;
					// Won't be reached because text/xml can be an XHTML mime type
					if (mimeType.equals("text/xml") || mimeType.equals("appplication/xml") )
					{
						contextType = TokenContextType.XML;
					}
					else if (mimeType.equals("text/javascript") || mimeType.equals("text/ecmascript") || mimeType.equals("application/x-javascript"))
					{
						contextType = TokenContextType.JAVASCRIPT_NON_HTML;
					}
					else if (mimeType.equals("text/css"))
					{
						contextType = TokenContextType.CSS;
					}
					else if (mimeType.equals("text/plain"))
					{
						contextType = TokenContextType.TEXT;
					}
					else
					{
						contextType = TokenContextType.OTHER_NON_HTML;
					}
					NonHtmlBodyContext context = new NonHtmlBodyContext(token, contextType, transaction.getId(), mimeType);
					if (!contexts.containsKey(token + contextType))
					{
						contexts.put(token + contextType, context);
					}
//					contexts.add(context);
				}
			}
		}
	}

	private static void parseHeaders(AbstractHttpTransaction transaction, Map<String, TokenContext> contexts, Set<String> tokens)
//	private static void parseHeaders(AbstractHttpTransaction transaction, Set<TokenContext> contexts, Set<String> tokens)
	{
		for (Header header: transaction.getAllResponseHeaders())
		{
			String name = header.getName().toUpperCase();
			String value = header.getValue().toUpperCase();
			Iterator<String> iterator = tokens.iterator();
			while(iterator.hasNext())
			{
				String token = iterator.next();
				if (name.contains(token) && !contexts.containsKey(token + TokenContextType.HTTP_HEADER_NAME))
//				if (name.contains(token))
				{
					HttpHeaderContext context = new HttpHeaderContext(token, TokenContextType.HTTP_HEADER_NAME, transaction.getId(), header);
					contexts.put(token + TokenContextType.HTTP_HEADER_NAME, context);
//					contexts.add(context);
				}
				if (value.contains(token) && !contexts.containsKey(token + TokenContextType.HTTP_HEADER_VALUE))
//				if (value.contains(token))
				{
					HttpHeaderContext context = new HttpHeaderContext(token, TokenContextType.HTTP_HEADER_VALUE, transaction.getId(), header);
//					contexts.add(context);
					contexts.put(token + TokenContextType.HTTP_HEADER_VALUE, context);
				}
			}
		}
	}

	private static void recursiveNodeSearch(String searchStrings[], Node node, Map<String, TokenContext> results, int outputTransactionID)
//	private static void recursiveNodeSearch(String searchStrings[], Node node, Set<TokenContext> results, int outputTransactionID)
	{
		/*
		 * Add the node as a result if the node value contains the token. This
		 * should usually be for text nodes and attribute nodes
		 */
		if (node == null)
		{
			return;
		}
		String nodeValue = node.getNodeValue();
		if (nodeValue != null)
		{
			for (String token: searchString(nodeValue, searchStrings))
			{
				addNodeResult(token, node, outputTransactionID, results);
			}
		}

		/*
		 * Add the node as a result if the node name contains the token.
		 */
		for (String token: searchString(node.getNodeName(), searchStrings))
		{
			addNodeResult(token, node, outputTransactionID, results);
		}

		short type = node.getNodeType();
		switch (type)
		{
			case Node.DOCUMENT_NODE:
			{
				Document document = (Document) node;
				recursiveNodeSearch(searchStrings, document.getDocumentElement(), results, outputTransactionID);
				break;
			}
				/*
				 * I don't think the doc type node needs to be searched.
				 */

			case Node.ELEMENT_NODE:
			{
				NamedNodeMap attrs = node.getAttributes();
				for (int index = 0; index < attrs.getLength(); index++)
				{
					Attr attr = (Attr) attrs.item(index);
					String name = attr.getNodeName();
					String value = attr.getNodeValue();
					if (name != null)
					{
						for (String token: searchString(name, searchStrings))
						{
							addNodeResult(token, attr, outputTransactionID, results);
						}
					}
					if (value != null)
					{
						for (String token: searchString(value, searchStrings))
						{
							addNodeResult(token, attr, outputTransactionID, results);
						}
					}
				}

				Node child = node.getFirstChild();
				while (child != null)
				{
					recursiveNodeSearch(searchStrings, child, results, outputTransactionID);
					child = child.getNextSibling();
				}
				break;
			}

			case Node.ENTITY_REFERENCE_NODE:
			{
				Node child = node.getFirstChild();
				while (child != null)
				{
					recursiveNodeSearch(searchStrings, child, results, outputTransactionID);
					child = child.getNextSibling();
				}
				break;
			}
		}
	}

	private static Set<String> searchString(String targetString, String[] searchStrings)
	{
		Set<String> results = new HashSet<String>();
		String target = targetString.toUpperCase();
		
		for (String token: searchStrings)
		{
			if (target.contains(token))
			{
				results.add(token);
			}
		}
		return results;
	}

}
