package com.grendelscan.tests.testModules;


import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpGetTransaction;
import com.grendelscan.requester.queue.RequesterQueue;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.SelectableOption;
import com.grendelscan.tests.testModules.settings.SingleSelectOptionGroup;
import com.grendelscan.tests.testTypes.ByDirectoryTest;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.HttpUtils;

public class Module0006 extends TestModule implements ByDirectoryTest
{
	private List<String> discoveredNames;
	private List<String> fileNames;
	private List<String> fileSuffixes;
	private RequesterQueue requesterQueue;
	private List<ConfigurationOption> configOptions;
	private SelectableOption smallNameOption, mediumNameOption, largeNameOption,
							smallExtensionOption, mediumExtensionOption, largeExtensionOption;

	public Module0006()
    {
		SingleSelectOptionGroup nameOptionsGroup = new SingleSelectOptionGroup("Name list size", "Select the name list size to use.");
		configOptions = new ArrayList<ConfigurationOption>();
		configOptions.add(nameOptionsGroup);

		smallNameOption = new SelectableOption("Small (20 filenames)", false, "");
		nameOptionsGroup.addOption(smallNameOption);
		mediumNameOption = new SelectableOption("Medium (40 filenames)", true, "");
		nameOptionsGroup.addOption(mediumNameOption);
		largeNameOption = new SelectableOption("Large (64 filenames)", false, "");
		nameOptionsGroup.addOption(largeNameOption);

		SingleSelectOptionGroup extensionOptionsGroup = new SingleSelectOptionGroup("Extension list size", "Select the extesion list size to use.");
		configOptions.add(extensionOptionsGroup);

		smallExtensionOption = new SelectableOption("Small (10 extensions)", true, "");
		extensionOptionsGroup.addOption(smallExtensionOption);
		mediumExtensionOption = new SelectableOption("Medium (25 extensions)", false, "");
		extensionOptionsGroup.addOption(mediumExtensionOption);
		largeExtensionOption = new SelectableOption("Large (63 extensions)", false, "");
		extensionOptionsGroup.addOption(largeExtensionOption);
    }

	@Override
	public String getDescription()
	{
		return "This module will look for common file names in all directories " +
				"and sub-directories observed during the scan. If you pick long " +
				"name and extension lists, this can take a very long time, so " +
				"only enable this module if you're willing to wait. \n" +
				"\n" +
				"The word lists are defined in conf/file_enumeration.conf.";
	}

	@Override
    public int[] getPrerequisites()
    {
		int i[] = {19};
	    return i;
    }

	
	@Override
	public int getModuleNumber()
	{
		return 6;
	}

	@Override
	public String getName()
	{
		return "File enumerator";
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.FILE_ENUMERATION;
	}

	@SuppressWarnings("unchecked")
    @Override
	public void initialize()
	{
		discoveredNames = new ArrayList(1);
		fileSuffixes = ConfigurationManager.getList("file_enumeration.small_extension_list"); 
		if (mediumExtensionOption.isSelected() || largeExtensionOption.isSelected())
		{
			fileSuffixes.addAll(ConfigurationManager.getList("file_enumeration.medium_extension_list")); 
		}
		if (largeExtensionOption.isSelected())
		{
			fileSuffixes.addAll(ConfigurationManager.getList("file_enumeration.large_extension_list")); 
		}
		
		fileNames = ConfigurationManager.getList("file_enumeration.small_name_list"); 
		if (mediumNameOption.isSelected() || largeNameOption.isSelected())
		{
			fileNames.addAll(ConfigurationManager.getList("file_enumeration.medium_name_list")); 
		}
		if (largeNameOption.isSelected())
		{
			fileNames.addAll(ConfigurationManager.getList("file_enumeration.large_name_list")); 
		}
		requesterQueue = scan.getRequesterQueue(); 
	}

	public void testByDirectory(int transactionID, String directory)
	{
    	AbstractHttpTransaction originalTransaction = scan.getTransactionRecord().getTransaction(transactionID);
		for(String name: fileNames)
		{
			for (String suffix: fileSuffixes)
			{
				if (!scan.isRunning())
				{
					break;
				}
				String uri = originalTransaction.getScheme() + "://" + originalTransaction.getHost() + directory + name + "." + suffix;
				try
				{
					HttpGetTransaction testTransaction = new HttpGetTransaction(scan, uri, originalTransaction.getId(), TransactionSource.ENUMERATION);
					requesterQueue.processRequestNow(testTransaction, "Module 6", false, true, false, true);
					if (HttpUtils.fileExists(testTransaction.getLogicalResponseCode())) 
					{
						scan.getCategorizerQueue().addTransaction(testTransaction);
						discoveredNames.add(uri);
					}
				}
				catch (URISyntaxException e) 
				{
					Debug.errDebug("Problem with URI syntax in Module0006 (" + uri + "): " + e.toString());
				}
			}
		}
	}

	private void report()
	{
		String list = "";
		for (String uri: discoveredNames)
		{
			list += HtmlUtils.makeLink(uri) + "<br>";
		}
		String uri = discoveredNames.get(0) + (discoveredNames.size() > 1 ? " and others" : "");
		ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.INFO, uri, "Files discovered", 
				"Files were discovered by guessing the name", 
				"The following files were discovered by guessing the name: <br>" + list, 
				"If these files were not linked to by the website, they could lead an attacker to unintended areas.", 
				"Review the contents of the files and remove them if they aren't required.", "");
		scan.getReportCollector().addEvent(event);
	}

	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	@Override
    public void scanIsComplete()
    {
		if (discoveredNames.size() > 0)
		{	
			report();
		}
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }

}
