package com.grendelscan.tests.testModules;


import java.net.URISyntaxException;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpGetTransaction;
import com.grendelscan.requester.queue.RequesterQueue;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testTypes.ByFileTest;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.HttpUtils;
import com.grendelscan.utils.MimeUtils;
import com.grendelscan.utils.URIStringUtils;

public class Module0007 extends TestModule implements ByFileTest
{

	private String[] filePrefixes;
	private String[] fileSuffixes;
	private RequesterQueue requesterQueue;
	
	@Override
	public String getDescription()
	{
		return "Checks for backups of files observed during the scan. " +
				"The tested extensions are defined in conf/file_enumeration.conf. Unlike " +
				"the file and directory enumerators, this doesn't take too long.";
	}

	@Override
	public int getModuleNumber()
	{
		return 7;
	}

	@Override
	public String getName()
	{
		return "Backup file enumerator";
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.FILE_ENUMERATION;
	}

	@Override
	public void initialize()
	{
		filePrefixes = ConfigurationManager.getStringArray("file_enumeration.backup_prefixes");
		fileSuffixes = ConfigurationManager.getStringArray("file_enumeration.backup_suffixes");
		requesterQueue = scan.getRequesterQueue(); 
	}

	public void testByFile(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	// We only are interested in pages with web content 
    	if (!MimeUtils.isWebMimeType(transaction.getResponseMimeType()))
    	{
    		return;
    	}
		String name = transaction.getFilename();
		String baseUri = transaction.getBaseUri();
		String dir = URIStringUtils.getDirectory(baseUri);
		if (name.equals("") && (dir.equals("") || dir.equals("/")))
		{
			return;
		}
		if (name.equals(""))
		{
			baseUri = baseUri.substring(0, baseUri.length() - 1);
		}
		
		String uri;
		for (String suffix: fileSuffixes)
		{
			if (!scan.isRunning())
			{
				return;
			}
			uri = baseUri + "." + suffix;
			testFile(uri, name + "." + suffix, transaction);
		}
		for (String prefix: filePrefixes)
		{
			if (!scan.isRunning())
			{
				return;
			}
			uri = transaction.getHostUri() + transaction.getDirectory().substring(1) + prefix + name;
			testFile(uri, prefix + name, transaction);
		}
	}
	
	private void testFile(String uri, String name, AbstractHttpTransaction referer)
	{
		try
		{
			HttpGetTransaction testTransaction = new HttpGetTransaction(scan, uri, referer.getId(), TransactionSource.ENUMERATION);

			if (requesterQueue.processRequestNow(testTransaction, "Module 7", false, true, false, true))
			{
				if (HttpUtils.fileExists(testTransaction.getLogicalResponseCode()))
				{
					scan.getCategorizerQueue().addTransaction(testTransaction);
					String longDescription = "A file that appears to be a backup of " + HtmlUtils.makeLink(referer.getBaseUri()) + 
										" has been located at " + HtmlUtils.makeLink(uri);
					String impact = "Backup files may be served up as text files by the web server, which could " +
											"reveal their source code. This could help an attacker to identify vulnerabilities " +
											"that would be difficult to find during blind attacks. ";
					String recomendations = "Remove all backup files from production web servers.";
					ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.LOW, uri, "Backup file located",
												"A file that appears to be a backup of " + name + " has been located.", 
												longDescription, impact, recomendations, "");
					scan.getReportCollector().addEvent(event);
				}
			}
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("URI syntax problem in Module0007 (" + uri + "): " + e.toString());
		}
		
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
