package com.grendelscan.tests.testModules;


import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.Header;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testTypes.ByHttpResponseCodeTest;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.MimeUtils;

public class Module0008 extends TestModule implements ByHttpResponseCodeTest
{
	private List<String> discoveredBodies;
	private List<String> discoveredHeaders;
	private String lastUrl; 
	private final int maxUrls = 5;
	public String[] getResponseCodes()
    {
	    return new String[] {"200", "301", "302", "303", "307"};
    }

	private Pattern privateIPRegEx;

	@Override
	public String getDescription()
	{
		return "Looks for private IP addresses (RFC 1918) leaked in web server responses.";
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.INFORMATION_LEAKAGE;
	}

	@Override
	public int getModuleNumber()
	{
		return 8;
	}

	@Override
	public String getName()
	{
		return "Private address leakage";
	}

	@Override
	public void initialize()
	{
		privateIPRegEx = Pattern.compile("\\b((?:10(:?\\.\\d{1,3}){3})|(?:192\\.168(?:\\.\\d{1,3}){2})|(?:172\\.[1-3]\\d(?:\\.\\d{1,3}){2}))\\b");
		discoveredBodies = new ArrayList<String>(1);
		discoveredHeaders = new ArrayList<String>(1);
	}

	public void testByHttpResponseCode(int transactionID)
	{
		if (discoveredHeaders.size() < maxUrls || discoveredBodies.size() < maxUrls)
		{
	    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
			Matcher matcher;
			if (discoveredHeaders.size() < maxUrls)
			{
				for (Header header: transaction.getAllResponseHeaders())
				{
					if (!scan.isRunning())
					{
						return;
					}
					matcher = privateIPRegEx.matcher(header.getValue());
					if (matcher.find() && isPrivateAddress(matcher.group(1)))
					{
						lastUrl = transaction.getAbsoluteUriString();
						transaction.writeToDisk();
						discoveredHeaders.add(HtmlUtils.makeLink(transaction.getSavedUrl(), transaction.getAbsoluteUriString()) + ": " 
								+ matcher.group(1) + " was found in the header named \"" + header.getName() + "\"");
						break;
					}
				}
			}
			
			if (MimeUtils.isWebMimeType(transaction.getResponseMimeType()) && discoveredBodies.size() < maxUrls)
			{
	            String body = transaction.getResponseBodyAsString();
	            matcher = privateIPRegEx.matcher(body);
	            while (matcher.find())
	            {
	            	if (!scan.isRunning())
	            	{
	            		return;
	            	}
		            if (isPrivateAddress(matcher.group(1)))
		            {
						lastUrl = transaction.getAbsoluteUriString();
						transaction.writeToDisk();
			            discoveredBodies.add(HtmlUtils.makeLink(transaction.getSavedUrl(), transaction.getAbsoluteUriString()) + ": "
			            		+ matcher.group(1) + " was found in the response body");
			            break;
		            }
	            }
            }
		}
	}

	private boolean isPrivateAddress(String address)
	{
		String sOctets[] = address.split("\\.");
		if (sOctets.length != 4)
		{
			return false;
		}

		int bOctets[] = new int[4];
		for (int index = 0; index < 4; index++)
		{
			bOctets[index] = Integer.valueOf(sOctets[index]);
			if ((bOctets[index] > 255) || (bOctets[index] < 0))
			{
				return false;
			}
		}

		switch (bOctets[0])
		{
			case 10:
				return true;
			case 192:
				if (bOctets[1] == 168)
				{
					return true;
				}
			case 172:
				if ((bOctets[1] >= 16) && (bOctets[1] <= 31))
				{
					return true;
				}
			default:
				return false;
		}
	}

	private String genLocation()
	{
		String location = "";
		if (discoveredHeaders.size() > 0)
		{
			location = "response headers";
		}

		if (discoveredBodies.size() > 0)
		{
			if (!location.equals(""))
			{
				location += " and in ";
			}
			location += "response bodies";
		}
		return location;
	}
	
	private String genRecomendations()
	{
		String recomendation = "";
		if (discoveredHeaders.size() > 0)
		{
			recomendation = "Consult the web or application server's documentation to disable this behavior.";
		}

		if (discoveredBodies.size() > 0)
		{
			if (!recomendation.equals(""))
			{
				recomendation += " ";
			}
			recomendation += "Modify the application's behavior to surpress the inclusion of private IP addresses.";
		}
		return recomendation;
	}

	private void report()
	{
		String title = "Private IP address leakage";
		String location = genLocation();
		String shortDescription = "Possible leakage of an internal IP address has been identified in " + location + ".";
		String longDescription =
		        "Private IP addreses defined by RFC 1918 are not routable on the Internet, but are " +
				"frequently used on Internet-exposed devices and then changed by Network Address Translation (NAT).<br><br>";

		if (discoveredHeaders.size() + discoveredBodies.size() > 1)
		{
			lastUrl += ", and others"; 
		}

		if (discoveredHeaders.size() + discoveredBodies.size() >= maxUrls)
		{
			longDescription += "Note that only the first " + maxUrls + " findings were documented. "; 
		}

		longDescription += "Private IP addresses were found in the following location(s):<br>\n";
		
		for (String s: discoveredHeaders)
		{
			longDescription += s + "<br><br>";
		}
		
		for (String s: discoveredBodies)
		{
			longDescription += s + "<br><br>";
		}
		
			
		
		String impact =  "Knowning the private IP address of a server can help an attacker to craft better attacks against internal assets.";
		String recomendations = genRecomendations();
		ReportableEvent event =
		        new ReportableEvent(null, "Module00008", EventSeverity.INFO, lastUrl,
		                title, shortDescription, longDescription, impact, recomendations,
		                HtmlUtils.makeLink("http://www.ietf.org/rfc/rfc1918.txt"));
		scan.getReportCollector().addEvent(event);
	}


	@Override
    public void scanIsComplete()
    {
		if (discoveredHeaders.size() > 0 || discoveredBodies.size() > 0)
		{
			report();
		}
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }

}
