package com.grendelscan.tests.testModules;

import java.util.HashSet;
import java.util.Set;

import org.apache.http.cookie.Cookie;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.SessionIDTesting.SessionID;
import com.grendelscan.tests.libraries.SessionIDTesting.SessionIDLocation;
import com.grendelscan.tests.testTypes.*;
import com.grendelscan.tests.testTypes.InitialAuthenticationTest;
import com.grendelscan.utils.CollectionUtils;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.HttpUtils;

public class Module0009 extends TestModule implements BySetCookieTest, InitialAuthenticationTest
{
	private int maxCookieRequests;
	
	private int minAcceptableBitLength;
	
	@Override
	public String getDescription()
	{
		return "Checks cookies for weak session keys based on net entropy, "
		        + "repeated values and overall key length.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SESSION_MANAGMENT;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 9;
	}
	
	@Override
	public String getName()
	{
		return "Cookie strength ";
	}
	
	public String[] getResponseHeaders()
	{
		String headers[] = { "Set-Cookie" };
		return headers;
	}
	
	@Override
	public void initialize()
	{
		maxCookieRequests = SessionID.getMaxSessionIDRequests();
		minAcceptableBitLength = SessionID.getMinAcceptableBitLength();
	}
	
	public void testBySetCookie(int transactionID, Cookie cookie)
    {
		AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		testTransaction(transaction, cookie);
    }

	
	public void testInitialAuthentication(int transactionID, SessionIDLocation sessionIDLocation)
	{
		if (sessionIDLocation == SessionIDLocation.COOKIE_SESSION_ID)
		{
			AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
			for (Cookie cookie: HttpUtils.getSetCookies(transaction))
			{
				testTransaction(transaction, cookie);
			}
		}
	}
	
	private String getTransactionLinks(AbstractHttpTransaction[] testTransactions)
	{
		String links = "The following transactions were used for the test:<br>";
		for (AbstractHttpTransaction transaction: testTransactions)
		{
			transaction.writeToDisk();
			links += HtmlUtils.makeLink(transaction.getSavedUrl(), String.valueOf(transaction.getId())) + "<br>\n";
		}
		
		return links;
	}
	
	private void checkCookieEntropy(AbstractHttpTransaction originalTransaction, String cookieName,
	        String[] testCookiesValues, AbstractHttpTransaction[] testTransactions)
	{
		
		int bitChanges;
		if ((bitChanges = SessionID.getSessionIDBitEntropy(testCookiesValues)) >= 0)
		{
			
			String longDescription =
			        "The cookie named " + cookieName + " appears to be used to track session state. "
			                + "Approximately " + String.valueOf(bitChanges)
			                + " bits of random data were observed. A minimum of " + minAcceptableBitLength
			                + " is generally recommended.<br><br>";
			longDescription += getTransactionLinks(testTransactions);
			
			String recomendations =
			        "Assure that session ID contain at least " + minAcceptableBitLength + " bits of random data.";
			String references = HtmlUtils.makeLink("http://www.owasp.org/index.php/Session_Management");
			report(originalTransaction, testCookiesValues, "Similar session IDs detected", longDescription,
			        recomendations, references, EventSeverity.MEDIUM);
		}
	}
	
	private void findDuplicateCookies(AbstractHttpTransaction originalTransaction, String cookieName,
	        String[] testCookiesValues, AbstractHttpTransaction[] testTransactions)
	{
		
		if (SessionID.findDuplicateSessionIDs(testCookiesValues))
		{
			String longDescription =
			        "The cookie named " + cookieName + " appears to be used to track session state. "
			                + "However, some duplicate cookie values were detected. If " + cookieName + " is used"
			                + "to track session state, all values supplied by the web server should be unique. "
			                + "Detection of duplicate values implies that the generation algorithm outputs "
			                + "predictably named cookieJar. <br><br>";
			longDescription += getTransactionLinks(testTransactions);
			String recomendations =
			        "Assure that cookieJar contain at least " + minAcceptableBitLength + " bits of random data.";
			String references = HtmlUtils.makeLink("http://www.owasp.org/index.php/Session_Management");
			report(originalTransaction, testCookiesValues, "Duplicate session IDs detected", longDescription,
			        recomendations, references, EventSeverity.MEDIUM);
		}
	}
	
	private String[] getCookieValues(String cookieName, AbstractHttpTransaction[] testTransactions)
	{
		String[] testCookiesValues = new String[maxCookieRequests + 1];
		int index = 0;
		for (Cookie cookie: SessionID.getSetCookiesByName(testTransactions, cookieName))
		{
			if (cookie == null)
			{
				Debug.errDebug("Cookie not found where one expected.", new Throwable());
				continue;
			}
			testCookiesValues[index++] = cookie.getValue();
		}
		
		return testCookiesValues;
	}
	
	


	
	private void report(AbstractHttpTransaction originalTransaction, String[] testCookiesValues, String title,
	        String longDescription, String recomendations, String references, EventSeverity severity)
	{
		longDescription += "<br><br>The following cookieJar were received during testing:<br>";
		String impact =
		        "If the session ID is session ID is generated in a predictable manner, an attacker could hijack legitimate sessions "
		                + "by guessing the session IDs of authenticated users.";
		for (String value: testCookiesValues)
		{
			longDescription += "<tab>" + value + "<br>";
		}
		ReportableEvent event =
		        new ReportableEvent(null, getName(), severity, originalTransaction.getUriString(), title,
		                "A problem was discovered with cookie values.", longDescription, impact, recomendations,
		                references);
		scan.getReportCollector().addEvent(event);
	}
	
	private void testCookies(AbstractHttpTransaction originalTransaction, String cookieName,
	        AbstractHttpTransaction[] testTransactions)
	{
		String[] testCookiesValues = getCookieValues(cookieName, testTransactions);
		findDuplicateCookies(originalTransaction, cookieName, testCookiesValues, testTransactions);
		checkCookieEntropy(originalTransaction, cookieName, testCookiesValues, testTransactions);
	}
	
	private void testTransaction(AbstractHttpTransaction transaction, Cookie cookie)
	{
		boolean initialized = false;
		AbstractHttpTransaction[] testTransactions = null;
		
		Set<String> oldCookies = new HashSet<String>(transaction.getUsedCookieNames());
		oldCookies.addAll(transaction.getQueryParameterNames());
		
			
		if (!scan.isRunning())
		{
			return;
		}
		/*
		 * If this cookie was supplied by the initial transaction in a
		 * header, or as a URL parameter, or if it's already been tested in
		 * another transaction, don't test it.
		 */
		synchronized(SessionID.getTestedSessionIDs())
		{
			if (CollectionUtils.containsStringIgnoreCase(oldCookies, cookie.getName())
			        || SessionID.getTestedSessionIDs().contains(cookie.getName()))
			{
				return;
			}
			
			SessionID.getTestedSessionIDs().add(cookie.getName());
		}
		
		// Request only once a testable cookie has been found
		if (!initialized)
		{
			testTransactions = SessionID.getSampleCookieTransactions(transaction, cookie, "Module " + this.getModuleNumber());
			initialized = true;
		}
		
		Debug.debug(getName() + " is testing the cookie \"" + cookie.getName() + "\"");
		
		if (SessionID.isKnownSessionID(cookie.getName()) && scan.isRunning())
		{
			testCookies(transaction, cookie.getName(), testTransactions);
		}
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }

}
