package com.grendelscan.tests.testModules;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.XSS;
import com.grendelscan.tests.libraries.TokenTesting.HtmlContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenContextType;
import com.grendelscan.tests.libraries.TokenTesting.TokenContextTypeUtils;
import com.grendelscan.tests.libraries.TokenTesting.TokenTesting;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.SelectableOption;
import com.grendelscan.tests.testModules.settings.SingleSelectOptionGroup;
import com.grendelscan.tests.testTypes.ByRepeatableOutputContextTest;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HtmlUtils;

public class Module0010 extends TestModule implements ByRepeatableOutputContextTest
{
	private List<String> findings;
	
	@Override
    public int[] getPrerequisites()
    {
	    return new int[] {20};
    }

	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	private HashMap<String, SelectableOption> agressionOptions;
	private List<ConfigurationOption> configOptions;
	private boolean low;
	private boolean medium;
	private boolean high;

	public Module0010()
    {
		SelectableOption agressionOption;
		SingleSelectOptionGroup agressionOptionsGroup = 
			new SingleSelectOptionGroup("XSS testing aggression", "Select the aggression levels for testing XSS.");
		agressionOptions = new HashMap<String, SelectableOption>();

		agressionOption = new SelectableOption("Low", false, "");
		agressionOptions.put("low", agressionOption);
		agressionOptionsGroup.addOption(agressionOption);

		agressionOption = new SelectableOption("Medium", true, "");
		agressionOptions.put("medium", agressionOption);
		agressionOptionsGroup.addOption(agressionOption);

		agressionOption = new SelectableOption("High", false, "");
		agressionOptions.put("high", agressionOption);
		agressionOptionsGroup.addOption(agressionOption);

		configOptions = new ArrayList<ConfigurationOption>();
		configOptions.add(agressionOptionsGroup);

    }
	
	@Override
	public String getDescription()
	{
		return "Tests for XSS from HTTP query parameters. Both POST and GET " +
				"transactions are tested, but only parameters known to be " +
				"used in output are tested. The base attack strings are " +
				"defined in conf/xss.conf, but the actual attack is based " +
				"on the HTML context that the output is observed in. The " +
				"response is run through an HTML parser and JavaScript " +
				"engine to identify successful attacks. Note that using " +
				"the \"high\" aggression setting can be particularly noisy " +
				"and take a while to run due to all of the parsing. " +
				"Some particularly exotic attacks (http://ha.ckers.org/xss.html) " +
				"may not be discovered. This should be improved in future releases.";
	}

	public TokenContextType[] getDesiredRepeatableContexts()
	{
		return TokenContextTypeUtils.getHtmlContexts();
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.XSS;
	}

	@Override
	public int getModuleNumber()
	{
		return 10;
	}

	@Override
	public String getName()
	{
		return "XSS - Query";
	}

	@Override
	public void initialize()
	{
		low = agressionOptions.get("low").isSelected();
		medium = agressionOptions.get("medium").isSelected();
		high = agressionOptions.get("high").isSelected();
		findings = new ArrayList<String>(1);
	}

	public void testByRepeatableOutputContext(TokenContext context)
	{
		if (!(context instanceof HtmlContext))
		{
			Debug.errDebug("Problem with the type of context at Module0010.testByRepeatableOutputContext: " + context.getContextType());
			return;
		}
		HtmlContext htmlContext = (HtmlContext) context;
		AbstractHttpTransaction originalInputTransaction = TokenTesting.getTokenTest(htmlContext.getToken());

		if (!XSS.checkForKnownXSS(originalInputTransaction, htmlContext.getOriginalInputParameterName()))
		{
			Character badChars[] = XSS.getBlockedCharacters(htmlContext, "Module " + this.getModuleNumber());
			boolean found = false;
			if (low || medium)
			{
				found = runTests(XSS.craftSimpleXSSAttacks(htmlContext.getContextType()), htmlContext, originalInputTransaction, badChars);
			}
			if (medium && !found)
			{
				found = runTests(XSS.craftMediumXSSAttacks(htmlContext.getContextType()), htmlContext, originalInputTransaction, badChars);
			}
			if (high)
			{
				found = runTests(XSS.craftAdvancedXSSAttacks(htmlContext.getContextType()), htmlContext, originalInputTransaction, badChars);
			}
		}
	}
	
	private boolean runTests(List<String> attackStrings, HtmlContext htmlContext, AbstractHttpTransaction originalInputTransaction, Character badChars[])
	{
		
		for (String rawAttackString: attackStrings)
		{
			String newToken = TokenTesting.generateToken();
			for (String processedAttackString: XSS.updateAttackStringTokens(rawAttackString, newToken, htmlContext, badChars))
			{
				if (!scan.isRunning())
				{
					return false;
				}
				AbstractHttpTransaction transactions[] =
				        TokenTesting.duplicateTokenTest(htmlContext, processedAttackString, "Module " + getModuleNumber(), TransactionSource.TEST);
				AbstractHttpTransaction testOutputTransaction = transactions[1];

				
				/*
				 * This is because of a race condition where the same token may appear multiple times
				 * in the same document, resulting in multiple contexts. This should be the case because
				 * each one needs to be detected independently. However, only one needs to be reported.
				 * For now, the synchronized block is the ugly solution. 
				 */
				Set<String> knownXSS = XSS.getKnownXSS();
				if (!scan.isRunning())
				{
					return false;
				}

				synchronized(knownXSS)
				{
					if (XSS.checkSuccessfulXSSExecution(testOutputTransaction) &&
							!XSS.checkForKnownXSS(originalInputTransaction, htmlContext.getOriginalInputParameterName()))
					{
						XSS.addXSS(originalInputTransaction, htmlContext.getOriginalInputParameterName());
						recordFinding(testOutputTransaction, htmlContext, processedAttackString, newToken);
						return true;
					}
				}
			}
		}

		return false;
	}

	
	private void recordFinding(AbstractHttpTransaction transaction, TokenContext context, 
	        String attackString, String token)
	{
		transaction.writeToDisk();
		String finding = 
			"URL: " + transaction.getBaseUri() + "<br>\n" +
			"Parameter: " + context.getOriginalInputParameterName() + "<br>\n" +
			"Method: " + transaction.getMethod().toUpperCase() + "<br>\n" +
			"Attack string: " + HtmlUtils.escapeHTML(attackString) + "<br>\n" +
			"Token: " + token + "<br>\n" +
			"Transaction: " + HtmlUtils.makeLink(transaction.getSavedUrl(), String.valueOf(transaction.getId())) + "<br>\n"; 
		
		Debug.discoveryMessage("XSS detected on " + context.getOriginalInputParameterName() + " at " + transaction.getBaseUri());

		if (transaction.getMethod().equalsIgnoreCase("GET"))
		{
			String url = transaction.getUriString();
			finding += "The attack can be duplicated following " + HtmlUtils.makeLink(url, "this link") + ".";
		}
		
		finding += "<br>\n";
		
		findings.add(finding);
	}

	
	
	@Override
    public void scanIsComplete()
    {
		report();
    }

	private void report()
	{
		if (findings.size() > 0)
		{
			String longDescription =
			        "The query parameters listed below appear vulnerable to cross site scripting (XSS) attacks. When the " +
			        "attack string was supplied as the parameter value, it appears to have been placed in the results in a " +
			        "way that allows arbitrary JavaScript to be executed. A unique token was used for tracking purposes " +
			        "during testing. <br>\n<br>\n" +
			        "Note that some test attacks use a fictional JavaScript function (" + XSS.getJavascriptMethod() + ") for testing. "
				                + "This is because some anti-XSS filters will block common JavaScript functions (e.g. \"alert\") by name. This is "
				                + "not a sufficient security control; the fictional function allows the test to proceed more rapidly than testing "
				                + "multiple real functions. Other test attacks use a fictional hostname (" + XSS.getFakeHostname() + ") or a " +
				                " fictional IP address (" + XSS.getFakeIPAddress() + ") for testing. To perform "
				                + "an actual XSS attack, replace it with the name of a host that you control.<br>\n<br>\n";
			
			for (String finding: findings)
			{
				longDescription += finding;
			}
			ReportableEvent event =
		        new ReportableEvent(null, getName(), EventSeverity.MEDIUM, "See description",
		                "Cross-Site Scripting (XSS)", "Possible cross-site scripting (XSS) discovered.",
		                longDescription, XSS.getXSSImpact(), XSS.getXSSRecomendations(), XSS.getXSSReferences());
			scan.getReportCollector().addEvent(event);
		}
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }

}
