package com.grendelscan.tests.testModules;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.regex.*;
import java.util.List;
import java.util.Set;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.HttpGetTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.XSS;
import com.grendelscan.tests.libraries.TokenTesting.HtmlContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenTesting;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.SelectableOption;
import com.grendelscan.tests.testModules.settings.SingleSelectOptionGroup;
import com.grendelscan.tests.testTypes.ByHostTest;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.StringUtils;

public class Module0011 extends TestModule implements ByHostTest
{
	
	List<String> extensions;
	
	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	private boolean low;
	private boolean medium;
	private boolean high;
	
	private HashMap<String, SelectableOption> agressionOptions;
	private List<ConfigurationOption> configOptions;
	
	public Module0011()
    {
		SelectableOption agressionOption;
		SingleSelectOptionGroup agressionOptionsGroup = 
			new SingleSelectOptionGroup("XSS testing aggression", "Select the aggression levels for testing XSS.");
		agressionOptions = new HashMap<String, SelectableOption>();

		agressionOption = new SelectableOption("Low", false, "");
		agressionOptions.put("low", agressionOption);
		agressionOptionsGroup.addOption(agressionOption);

		agressionOption = new SelectableOption("Medium", true, "");
		agressionOptions.put("medium", agressionOption);
		agressionOptionsGroup.addOption(agressionOption);

		agressionOption = new SelectableOption("High", false, "");
		agressionOptions.put("high", agressionOption);
		agressionOptionsGroup.addOption(agressionOption);

		configOptions = new ArrayList<ConfigurationOption>();
		configOptions.add(agressionOptionsGroup);

    }

	@Override
	public String getDescription()
	{
		return "Tests server/framework file not found messages for XSS "
		        + "vulnerabilities. Some 404 messages may parse the "
		        + "filename as HTML. Since the platform may have "
		        + "different error messages for different file types, each "
		        + "file extension listed under \"common_framework_extensions\" "
		        + "in conf/file_enumeration is tested separately.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.XSS;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 11;
	}
	
	@Override
	public String getName()
	{
		return "Error XSS";
	}
	
	@SuppressWarnings("unchecked")
    @Override
	public void initialize()
	{
		low = agressionOptions.get("low").isSelected();
		medium = agressionOptions.get("medium").isSelected();
		high = agressionOptions.get("high").isSelected();
		extensions = ConfigurationManager.getList("file_enumeration.common_framework_extensions");
	}
	
	public void testByServer(int transactionID)
	{
		AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		ArrayList<String> uniqueServerExtensions = new ArrayList<String>();
		uniqueServerExtensions.addAll(extensions);
		identifyUniqueErrorPages(transaction, uniqueServerExtensions);
		HashSet<String> responseBodies = new HashSet<String>();
		for (String extension: uniqueServerExtensions)
		{
			if (!scan.isRunning())
			{
				return;
			}
			testExtension(transaction, extension, responseBodies);
		}
	}
	
	private void identifyUniqueErrorPages(AbstractHttpTransaction transaction, List<String> serverExtensions)
	{
		String uri, adjustedResponseBody;
		
		String filename = StringUtils.generateRandomString(StringUtils.FORMAT_UPPER_CASE_ALPHA, 8);
		HashSet<String> responseBodies = new HashSet<String>();
		List<String> tempExtensions = new ArrayList<String>(serverExtensions);
		for (String extension: tempExtensions)
		{
			if (!scan.isRunning())
			{
				return;
			}
			uri = transaction.getHostUri() + filename + "." + extension;
			try
			{
				HttpGetTransaction testRequest = new HttpGetTransaction(scan, uri, 0, TransactionSource.TEST);
				testRequest.execute("Module " + this.getModuleNumber());
				boolean unique = false;
				if (testRequest.isSuccessfullExecution())
				{
					adjustedResponseBody = testRequest.getResponseBodyAsString();
					Pattern filenamePattern = Pattern.compile(filename + "\\." + extension, Pattern.CASE_INSENSITIVE);
					adjustedResponseBody = filenamePattern.matcher(adjustedResponseBody).replaceAll("");
					if (!isRepeatedErrorPage(adjustedResponseBody, responseBodies, 90))
					{
						// Only categorize it if the response is unique
						scan.getCategorizerQueue().addTransaction(testRequest);
						responseBodies.add(adjustedResponseBody);
						unique = true;
					}
				}
				if (!unique)
				{
					serverExtensions.remove(extension);
				}
			}
			catch (URISyntaxException e)
			{
				Debug.errDebug("URI syntax problem in Module0011 (" + uri + "): " + e.toString());
			}
		}
	}
	
	private boolean isRepeatedErrorPage(String response, HashSet<String> responseBodies, int threshold)
	{
		boolean same = false;
		for (String responseB: responseBodies)
		{
			if (StringUtils.scoreStringDifference(response, responseB, 100) > threshold)
			{
				same = true;
				break;
			}
		}
		return same;
	}
	
	private void report(AbstractHttpTransaction transaction, String attackString, String token)
	{
		String url = transaction.getUriString();
		transaction.writeToDisk();
		String longDescription =
		        "The file not found message on "
                + transaction.getHost()
                + " appears to be vulnerable to cross site scripting (XSS) attacks. When the attack string \""
                + HtmlUtils.escapeHTML(attackString)
                + "\" was supplied as the file name, it appears to have been placed in the results in a way "
                + "that allows arbitrary JavaScript to be executed.<br>"
                + "The token \""
                + token
                + "\" was used for tracking purposes during testing. Depending on the attack string, it may need to be "
                + "modified to perform an actual XSS attack. This attack can be duplicated by visiting the "
                + "following URL: " + HtmlUtils.makeLink(url) 
				+ "The transaction used for testing can be viewed " + HtmlUtils.makeLink(transaction.getSavedUrl(), "here") + ".";

		if (attackString.contains(XSS.getJavascriptMethod()))
		{
			longDescription +=
			        "Note that \""
			                + XSS.getJavascriptMethod()
			                + "\" is a fictional JavaScript function used for testing. "
			                + "This is because some anti-XSS filters will block common JavaScript functions (e.g. \"alert\") by name. This is "
			                + "not a sufficient security control; the fictional function allows the test to proceed more rapidly than testing "
			                + "multiple real functions. ";
		}
		if (attackString.contains(XSS.getFakeHostname()))
		{
			longDescription +=
			        "The hostname \"" + XSS.getFakeHostname()
			                + "\" is a fictional hostname used for testing. To perform "
			                + "an actual XSS attack, replace it with the name of a host that you control. ";
		}
		
		if (attackString.contains(XSS.getFakeIPAddress()))
		{
			longDescription +=
			        "The IP address \"" + XSS.getFakeIPAddress()
			                + "\" is a fictional IP address used for testing. To perform "
			                + "an actual XSS attack, replace it with the address of a host that you control. ";
		}
		
		ReportableEvent event =
		        new ReportableEvent(null, getName(), EventSeverity.MEDIUM, transaction.getUriString(),
		                "Cross Site Scripting (XSS)", "Possible cross site scripting (XSS) discovered.",
		                longDescription, XSS.getXSSImpact(), XSS.getXSSRecomendations(), XSS.getXSSReferences());
		scan.getReportCollector().addEvent(event);
	}
	
	private void testExtension(AbstractHttpTransaction transaction, String extension, Set<String> responseBodies)
	{
		String uri, token;
		Set<TokenContext> contexts;
		
		token = TokenTesting.generateToken();
		uri = transaction.getHostUri() + token + "." + extension;
		try
		{
			HttpGetTransaction initialTokenTestTransaction =
			        new HttpGetTransaction(scan, uri, 0, TransactionSource.TEST);
			initialTokenTestTransaction.execute("Module " + this.getModuleNumber());
			TokenTesting.recordTokenTest(token, initialTokenTestTransaction);
			contexts = TokenTesting.findTokenContexts(initialTokenTestTransaction, token);
			
			for (TokenContext rawContext: contexts)
			{
				if (!(rawContext instanceof HtmlContext))
				{
					continue;
				}
				HtmlContext context = (HtmlContext) rawContext;
				if (!scan.isRunning())
				{
					return;
				}
				Character badChars[] = XSS.getBlockedCharacters(context, "Module " + this.getModuleNumber());
				boolean found = false;
				if (high)
				{
					found = testAttackStrings(XSS.craftAdvancedXSSAttacks(context.getContextType()), context, token, uri, extension, transaction, responseBodies, badChars);
				}
				if (low || medium)
				{
					found = testAttackStrings(XSS.craftSimpleXSSAttacks(context.getContextType()), context, token, uri, extension, transaction, responseBodies, badChars);
				}
				if (medium && !found)
				{
					found = testAttackStrings(XSS.craftMediumXSSAttacks(context.getContextType()), context, token, uri, extension, transaction, responseBodies, badChars);
				}
				
				if (found)
				{
					break;
				}
			}
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("Problem with URI syntax in Module0011.testExtension (" + uri + "): " + e.toString());
		}
	}

	private boolean testAttackStrings(List<String> attackStrings, TokenContext context, String token, String uri,  
									String extension, AbstractHttpTransaction transaction, Set<String> responseBodies,
									Character badChars[])
	{
		for (String rawAttackString: attackStrings)
		{
			token = TokenTesting.generateToken();
			if (context instanceof HtmlContext)
			{
				HtmlContext htmlContext = (HtmlContext) context;
				for (String processedAttackString: XSS.updateAttackStringTokens(rawAttackString, token,
				        htmlContext, badChars))
				{
					if (!scan.isRunning())
					{
						return false;
					}
					uri = transaction.getHostUri() + processedAttackString + "." + extension;
					try
                    {
	                    HttpGetTransaction testTransaction =
	                            new HttpGetTransaction(scan, uri, 0, TransactionSource.TEST);
	                    testTransaction.execute("Module " + this.getModuleNumber());
	                    if (XSS.checkSuccessfulXSSExecution(testTransaction))
	                    {
	                    	report(transaction, processedAttackString + "." + extension, token);
	                    	responseBodies.add(testTransaction.getResponseBodyAsString());
	                    	return true;
	                    }
                    }
					catch (URISyntaxException e)
					{
						Debug.errDebug("Problem with URI syntax in Module0011.testAttackStrings (" + uri + "): " + e.toString());
					}				
				}
			}
		}
		return false;
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
	
}
