package com.grendelscan.tests.testModules;


import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

import org.apache.http.cookie.Cookie;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.SessionIDTesting.SessionID;
import com.grendelscan.tests.libraries.SessionIDTesting.SessionIDLocation;
import com.grendelscan.tests.testTypes.*;
import com.grendelscan.tests.testTypes.InitialAuthenticationTest;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.HttpUtils;
import com.grendelscan.utils.StringUtils;

public class Module0013 extends TestModule implements BySetCookieTest, InitialAuthenticationTest
{
	private Set<String> vulnerableSessionIDs;
	
    public void testInitialAuthentication(int transactionID, SessionIDLocation sessionIDLocation)
    {
		if (sessionIDLocation == SessionIDLocation.COOKIE_SESSION_ID)
		{
	    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
			testTransaction(transaction);
		}
    }

	
	@Override
    public int[] getSoftPrerequisites()
    {
		int soft[] = {9};
	    return soft;
    }

	@Override
	public String getDescription()
	{
		return "Checks for session fixation with cookies. You are encouraged to enable " +
				"the \"Cookie session ID strength checker\" module. It will help to more " +
				"accurately identify which cookies may be used for session tracking. ";
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SESSION_MANAGMENT;
	}

	@Override
	public int getModuleNumber()
	{
		return 13;
	}

	@Override
	public String getName()
	{
		return "Session fixation (cookies)";
	}

	@Override
	public void initialize()
	{
		vulnerableSessionIDs = Collections.synchronizedSet(new HashSet<String>());
	}

	
	private void testCookie(AbstractHttpTransaction transaction, Cookie cookie)
	{
		AbstractHttpTransaction testTransaction = transaction.clone(TransactionSource.TEST);
		testTransaction.removeCookie(cookie.getName());

		for (String pattern: SessionID.getFixationTestPatterns())
		{
			if (!scan.isRunning())
			{
				return;
			}
			if (testArbitraryCookie(testTransaction, pattern, cookie) || 
				testValidCookie(testTransaction, pattern, cookie))
			{
				break;
			}
		}
	}
	
	private boolean testValidCookie(AbstractHttpTransaction transaction, String testPattern, Cookie cookie)
	{
		boolean fixed = false;

		AbstractHttpTransaction request = transaction.clone(TransactionSource.TEST);
		request.setURI(updateUri(testPattern, request, cookie.getName(), cookie.getValue()));
		request.execute("Module " + this.getModuleNumber());
		if (containsSetCookie(request, cookie.getName(), cookie.getValue()))
		{
			transaction.writeToDisk();
			String longDescription =
		        "The cookie named " + cookie.getName() + " appears to be used to track session state. If so, it also appears to be " +
        		"vulnerable to session fixation. When the URL listed below was supplied, the response set the cookie to " +
        		"the same value provided in the URL. It does not appear to be possible to set the cookie to arbitrary " +
        		"values. This means that the attacker must first identify a valid session ID before fixating the victim " +
        		"on it. <br> " + request.getUriString() 
		        		+ "<br><br>The transaction used for testing can be viewed " + HtmlUtils.makeLink(transaction.getSavedUrl(), "here") + ".";

			report(transaction, cookie.getName(), longDescription);
			fixed = true;
		}

		return fixed;
	}
	
	private String updateUri(String testPattern, AbstractHttpTransaction request, String name, String value)
	{
		return testPattern.replace("%%baseurl%%", request.getBaseUri()).replace("%%idname%%", name)
								.replace("%%idvalue%%", value)
								.replace("%%query%%", request.getUriQueryString());
	}

	private boolean containsSetCookie(AbstractHttpTransaction transaction, String cookieName, String cookieValue)
	{
		boolean contains = false;
		
		for (Cookie cookie: HttpUtils.getSetCookies(transaction))
		{
			if (cookie.getName().equalsIgnoreCase(cookieName) && cookie.getValue().equalsIgnoreCase(cookieValue))
			{
				contains = true;
				break;
			}
		}

		return contains;
	}
	
	/**
	 * This checks to see if any random string can be supplied as a cookie value and result
	 * in successfull fixation.
	 * @param transaction
	 * @param testPattern
	 * @param cookieName
	 * @return
	 */
	private boolean testArbitraryCookie(AbstractHttpTransaction transaction, String testPattern, Cookie cookie)
	{
		boolean arbitrary = false;
		String newCookieValue = StringUtils.generateRandomString(StringUtils.FORMAT_LOWER_CASE_ALPHA, 6);
		AbstractHttpTransaction testTransaction = transaction.clone(TransactionSource.TEST);
		testTransaction.setURI(updateUri(testPattern, testTransaction, cookie.getName(), newCookieValue));
		testTransaction.execute("Module " + this.getModuleNumber());
		if (containsSetCookie(testTransaction, cookie.getName(), newCookieValue))
		{
			arbitrary = true;
		}
		else
		{
			// If it didn't work with letters, try numbers 
			newCookieValue = StringUtils.generateRandomString(StringUtils.FORMAT_DECIMAL, 6);
			testTransaction = transaction.clone(TransactionSource.TEST);
			testTransaction.setURI(updateUri(testPattern, testTransaction, cookie.getName(), newCookieValue));
			testTransaction.execute("Module " + this.getModuleNumber());
			if (containsSetCookie(testTransaction, cookie.getName(), newCookieValue))
			{
				arbitrary = true;
			}
		}

		if (arbitrary)
		{
			testTransaction.writeToDisk();
			String longDescription =
		        "The cookie named " + cookie.getName() + " appears to be used to track session state. If so, it also appears to be " +
		        		"vulnerable to session fixation. When the URL listed below was supplied, the response set the cookie to " +
		        		"the same value provided in the URL. It appears to be possible to set the cookie to arbitrary " +
		        		"values. This means that the attacker does not need to identify a valid session ID before fixating the victim " +
		        		"on it. <br> " + testTransaction.getUriString() 
		        		+ "<br><br>The transaction used for testing can be viewed " + HtmlUtils.makeLink(testTransaction.getSavedUrl(), "here") + ".";


			report(testTransaction, cookie.getName(), longDescription);
		}
		
		return arbitrary;
	}
	
	private synchronized void testTransaction(AbstractHttpTransaction transaction)
	{
		// if the page is already authenticated, or if it's a login transaction, then it's not fixation
		if (!transaction.isAuthenticated() && !transaction.isLoginTransaction())
		{
			Set<Cookie> cookies = HttpUtils.getSetCookies(transaction);
			for (Cookie cookie: cookies)
			{
				if (!scan.isRunning())
				{
					return;
				}
				if (! SessionID.isKnownSessionID(cookie.getName()) ||
						vulnerableSessionIDs.contains(cookie.getName().toUpperCase()))
				{
					continue;
				}
				
				testCookie(transaction, cookie);
			}
		}
	}


	private void report(AbstractHttpTransaction transaction, String cookieName, String longDescription)
	{
		vulnerableSessionIDs.add(cookieName.toUpperCase());
		
		String title = "Possible session fixation identified";
		String impact = "One form of session fixation is when an arbitrary session ID can be set from an HTTP query parameter. " +
				"If an attacker can get the user to execute the query (perhaps by clicking on a link), the " +
				"session can be easily hijacked since the session ID is already known. One way this could be " +
				"performed is by sending a URL containing the session ID to the user. ";
		String recomendations = "Correcting session fixation flaws will usually require reviewing the application " +
				"source code and removing the ability to set the session ID from a query. ";
		String references = HtmlUtils.makeLink("http://www.webappsec.org/projects/threat/classes/session_fixation.shtml");
		ReportableEvent event =
		        new ReportableEvent(null, getName(), EventSeverity.MEDIUM, transaction.getUriString(), title,
		                "A problem was discovered with cookie values.", longDescription, impact, recomendations, references);
		scan.getReportCollector().addEvent(event);
	}


	@Override
    public boolean isExperimental()
    {
	    return false;
    }


	public void testBySetCookie(int transactionID, Cookie cookie)
    {
		AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		if (! transaction.isLoginTransaction())
		{
			if (! SessionID.isKnownSessionID(cookie.getName()) ||
					vulnerableSessionIDs.contains(cookie.getName().toUpperCase()))
			{
				return;
			}
			
			testCookie(transaction, cookie);
		}
    }
	
//	public void testByResponseHeader(int transactionID, String responseHeaderName)
//	{
//    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
//		/*
//		 *  We don't want to tests logins because it could screw up the good sessions for spidering. 
//		 *  The actual login page can be tested by sending a transaction not marked as a login
//		 *  before the spidering begins.
//		 *  
//		 *  We also don't want redirected requests. We will test the request for the redirect location.
//		 */
//    	String summaryUri = transaction.getBaseUri() + "?" + transaction.getQuerySummary(false);
//		if (! transaction.isLoginTransaction() && transaction.getRedirectRequestTransactionID() == 0 &&
//				!testedUris.contains(summaryUri))
//		{
//			testedUris.add(summaryUri);
//			testTransaction(transaction);
//		}
//	}

}
