package com.grendelscan.tests.testModules;

import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.SessionIDTesting.SessionID;
import com.grendelscan.tests.testTypes.ByHttpQueryParameterTest;
import com.grendelscan.utils.CollectionUtils;
import com.grendelscan.utils.HtmlUtils;

public class Module0014 extends TestModule implements ByHttpQueryParameterTest
{
	
	private Set<String> badUrls;
	private Set<String> discoveredURLSessionIDs;
	private int maxSessionIDRequests;
	private int minAcceptableBitLength;
	
	@Override
	public String getDescription()
	{
		return "Searches responses for session IDs placed in URLs (which is bad). "
		        + "If one is found, it is tested for key strength based on net entropy, repeated values "
		        + "and overall key length.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SESSION_MANAGMENT;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 14;
	}
	
	@Override
	public String getName()
	{
		return "URL Session IDs";
	}
	
	@Override
	public void initialize()
	{
		badUrls = Collections.synchronizedSet(new HashSet<String>());
		discoveredURLSessionIDs = Collections.synchronizedSet(new HashSet<String>());
		maxSessionIDRequests = SessionID.getMaxSessionIDRequests();
		minAcceptableBitLength = SessionID.getMinAcceptableBitLength();
		
	}
	
	public void testByQueryParameter(int transactionID, String parameterName)
	{
		AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		if (SessionID.stringContainsURLSessionID(parameterName) && (transaction.getRefererID() > 0))
		{
			testTransaction(transaction.getReferer());
		}
	}
	
	private String getTransactionLinks(AbstractHttpTransaction[] testTransactions)
	{
		String links = "The following transactions were used for the test:<br>";
		for (AbstractHttpTransaction transaction: testTransactions)
		{
			transaction.writeToDisk();
			links += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" + HtmlUtils.makeLink( transaction.getSavedUrl(), String.valueOf(transaction.getId())) + "<br>\n";
		}
		
		return links;
	}

	
	private void checkSessionIDEntropy(AbstractHttpTransaction transactions[], String sessionIDName,
	        List<String> testSessionIDValues)
	{
		int bitChanges;
		if ((bitChanges = SessionID.getSessionIDBitEntropy(testSessionIDValues)) >= 0)
		{
			
			String longDescription =
			        "The parameter named " + sessionIDName + " appears to be used to track session state. "
			                + "Approximately " + String.valueOf(bitChanges)
			                + " bits of random data were observed. A minimum of " + minAcceptableBitLength
			                + " is generally recomended. <br><br>" + 
			                getTransactionLinks(transactions);
			
			String recomendations =
			        "Assure that session IDs contain at least " + minAcceptableBitLength + " bits of random data.";
			reportWeakSessionID(transactions[0], testSessionIDValues, "Similar session IDs detected",
			        longDescription, recomendations, EventSeverity.MEDIUM);
		}
	}
	
	private void findDuplicateCookies(AbstractHttpTransaction transactions[], String sessionIDName,
	        List<String> testCookiesValues)
	{
		if (SessionID.findDuplicateSessionIDs(testCookiesValues))
		{
			String longDescription =
			        "The parameter named " + sessionIDName + " appears to be used to track session state. "
			                + "However, some duplicate values were detected. If " + sessionIDName + " is used"
			                + "to track session state, all values supplied by the web server should be unique. "
			                + "Detection of duplicate values implies that the generation algorithm outputs "
			                + "predictably named session IDs. <br><br>" + 
			                getTransactionLinks(transactions);
			
			String recomendations =
			        "Assure that sessionIDs contain at least " + minAcceptableBitLength + " bits of random data.";
			reportWeakSessionID(transactions[0], testCookiesValues, "Duplicate session IDs detected",
			        longDescription, recomendations, EventSeverity.MEDIUM);
		}
	}
	
	private String getReferences()
	{
		return HtmlUtils.makeLink("http://www.owasp.org/index.php/Session_Management");
	}
	private AbstractHttpTransaction[] getMoreTransactions(AbstractHttpTransaction transaction)
	{
		AbstractHttpTransaction[] transactions = new AbstractHttpTransaction[maxSessionIDRequests];
		transactions[0] = transaction;
		for (int index = 1; index <= maxSessionIDRequests; index++)
		{
			AbstractHttpTransaction testTransaction = transaction.clone(TransactionSource.TEST);
			testTransaction.setSource(TransactionSource.TEST);
			testTransaction.execute("Module " + getModuleNumber(), false, false);
			transactions[index] = testTransaction;
			if (index == 1)
			{
				try
				{
					/*
					 * Pauses to let the second counter increment on the web
					 * server. This is because a poorly written app may generate
					 * identical session ids in a short time period. We'll pause
					 * for the first request, but subsequent requests will be
					 * executed quickly to try and catch a duplicate value.
					 */
					Thread.sleep(1500);
				}
				catch (InterruptedException e)
				{
				}
			}
		}
		return transactions;
	}
	
	private Map<String, List<String>> getSessionIDs(AbstractHttpTransaction transactions[])
	{
		
		Map<String, List<String>> sessionIDValues = new HashMap<String, List<String>>();
		for (AbstractHttpTransaction transaction: transactions)
		{
			SessionID.findUrlSessionIDsAndValues(transaction.getResponseBodyAsString(), sessionIDValues);
		}
		
		return sessionIDValues;
	}
	
	private void reportUrlSessionID(AbstractHttpTransaction originalTransaction, String sessionIDName)
	{
		String title = "Session ID stored in HTTP query parameter";
		String longDescription =
		        "The parameter named \""
		                + sessionIDName
		                + "\" appears to be used to track session state. "
		                + "However, if the session ID is stored in an HTTP query, it can be obtained from proxy server access logs, "
		                + "browser history records, and other locations that may not be properly secured for session ID storage. <br>The "
		                + "parameter was set in the body of " + originalTransaction.getUriString();
		String impact = "Session hijacking using secondary vulnerability. ";
		String recomendations = "Use cookieJar to track session IDs instead of query parameters";
		String shortDescription = "The session ID may be stored stored in an HTTP query parameter.";
		ReportableEvent event =
		        new ReportableEvent(null, getName(), EventSeverity.MEDIUM, originalTransaction.getUriString(), title,
		                shortDescription, longDescription, impact, recomendations, getReferences());
		scan.getReportCollector().addEvent(event);
	}
	
	private void reportWeakSessionID(AbstractHttpTransaction originalTransaction, List<String> testCookiesValues,
	        String title, String longDescription, String recomendations, EventSeverity severity)
	{
		longDescription += "<br><br>The following sessionIDs were received during testing:<br>";
		String impact =
		        "If the session ID is generated in a predictable manner, an attacker could hijack legitimate sessions "
		                + "by guessing the session IDs of authenticated users.";
		for (String value: testCookiesValues)
		{
			longDescription += "<tab>" + value + "<br>";
		}
		ReportableEvent event =
		        new ReportableEvent(null, getName(), severity, originalTransaction.getUriString(), title,
		                "A problem was discovered with session ID values.", longDescription, impact, recomendations,
		                getReferences());
		scan.getReportCollector().addEvent(event);
	}
	
	/**
	 * The idea is that we don't need to test the sessionID for strength if it
	 * wasn't newly assigned
	 * 
	 * @param transaction
	 * @param sessionID
	 * @return
	 */
	private boolean requestContainsID(AbstractHttpTransaction transaction, String sessionID)
	{
		
		return SessionID.transactionRequestContainsCookie(transaction, sessionID)
		        || CollectionUtils.containsStringIgnoreCase(transaction.getQueryParameterNames(), sessionID);
	}
	
	private void testSessionIDs(AbstractHttpTransaction transactions[])
	{
		Map<String, List<String>> sessionIDValues = getSessionIDs(transactions);
		for (String sessionIDName: sessionIDValues.keySet())
		{
			if (SessionID.getTestedSessionIDs().contains(sessionIDName)
			        || requestContainsID(transactions[0], sessionIDName) || scan.isQueryParameterIgnored(sessionIDName))
			{
				continue;
			}
			SessionID.getTestedSessionIDs().add(sessionIDName);
			findDuplicateCookies(transactions, sessionIDName, sessionIDValues.get(sessionIDName));
			checkSessionIDEntropy(transactions, sessionIDName, sessionIDValues.get(sessionIDName));
		}
	}
	
	private void testTransaction(AbstractHttpTransaction transaction)
	{
		badUrls.add(transaction.getUriString());
		AbstractHttpTransaction transactions[] = getMoreTransactions(transaction);
		testSessionIDs(transactions);
		
		for (String sessionIDName: SessionID.findUrlSessionIDs(transaction.getResponseBodyAsString()))
		{
			if (!scan.isRunning())
			{
				return;
			}
			if (discoveredURLSessionIDs.contains(sessionIDName))
			{
				continue;
			}
			reportUrlSessionID(transaction, sessionIDName);
		}
		
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
