package com.grendelscan.tests.testModules;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.SQLInjection;
import com.grendelscan.tests.testTypes.ByHttpQueryParameterTest;
import com.grendelscan.utils.*;
import java.util.*;
public class Module0015 extends TestModule implements ByHttpQueryParameterTest
{
	private List<String> findings;
  
	@Override
	public String getDescription()
	{
		return "Appends a single quote onto a parameter and looks for a SQL " +
				"error message in the response. The SQL error patterns are " +
				"defined in conf/sql_injection.conf. Currently, errors from " +
				"IBM DB2, Microsoft Access, Microsoft SQL Server, MySQL, " +
				"Oracle, PostgreSQL, and generic ODBC & OLE/DB errors are " +
				"detected.";
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SQL_INJECTION;
	}

	@Override
	public int getModuleNumber()
	{
		return 15;
	}

	@Override
	public String getName()
	{
		return "Error-based SQL Injection";
	}

	@Override
	public void initialize()
	{
		findings = new ArrayList<String>(1);
	}

	
	public void testByQueryParameter(int transactionID, String parameterName)
	{
		if (!scan.isRunning())
		{
			return;
		}
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	if (!MimeUtils.isWebMimeType(transaction.getResponseMimeType()))
    	{
    		return;
    	}
		AbstractHttpTransaction testTransaction = transaction.clone(TransactionSource.TEST);
		testTransaction.replaceQueryParameter(parameterName, transaction.getQueryParameterValue(parameterName) + "'");
		testTransaction.execute("Module " + this.getModuleNumber(), false);
		if (MimeUtils.isWebMimeType(testTransaction.getResponseMimeType()))
		{
			String bodyText = testTransaction.getResponseTextNodes();
			bodyText = bodyText.replaceAll("\\s++", " ");
			String platform = SQLInjection.findSQLErrorMessages(bodyText);
			if (platform != null)
			{
				recordFinding(testTransaction, parameterName, platform, bodyText);
			}
		}
	}

	
	private void recordFinding(AbstractHttpTransaction transaction, String parameterName, String platform, String bodyText)
	{
		transaction.writeToDisk();
		String finding = 
				"URL: " + transaction.getBaseUri() + "<br>\n" + 
				"Parameter name: " + parameterName + "<br>\n" +
				"Platform: " + platform + "<br>\n" +
				"Transaction: " + HtmlUtils.makeLink(transaction.getSavedUrl(), String.valueOf(transaction.getId())) + "<br>\n" +
				"<br>\n";
		Debug.discoveryMessage("Possible SQL injection discovered on " + parameterName + " on " + transaction.getBaseUri());
		findings.add(finding);
	}
	
	@Override
    public void scanIsComplete()
    {
		report();
    }

	private void report()
	{
		if (findings.size() > 0)
		{
			String briefDescription = "A SQL error message was detected";
			String longDescription = "When a single quote (') was appended to the parameters listed below, a SQL " +
					"error message was returned. This could indicate a SQL injection vulnerability.<br>\n<br>\n";
			
			for (String finding: findings)
			{
				longDescription += finding;
			}
			
			ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.HIGH, "See description", 
					"Possible SQL Injection", briefDescription, longDescription, SQLInjection.getSQLInjectionImpact(), 
					SQLInjection.getSQLInjectionRecomendations(), SQLInjection.getSQLInjectionReferences()); 
			scan.getReportCollector().addEvent(event);
		}
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }


}
