package com.grendelscan.tests.testModules;

import java.util.HashMap;
import java.util.Map;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.nikto.CurrentSoftwareVersion;
import com.grendelscan.tests.libraries.nikto.Nikto;
import com.grendelscan.tests.testTypes.FinalAnalysisTest;
import com.grendelscan.utils.HtmlUtils;

/**
 * Based on the logic in nikto_outdated.plugin
 * @author David Byrne
 *
 */
public class Module0017 extends TestModule implements FinalAnalysisTest
{
	private class OutdatedSoftware
	{
		String host;
		String header;
		String softwareString;
		CurrentSoftwareVersion version;
		AbstractHttpTransaction transaction;
		public OutdatedSoftware(String host, String header, String softwareString, CurrentSoftwareVersion version, AbstractHttpTransaction transaction)
        {
			this.transaction = transaction;
	        this.host = host;
	        this.header = header;
	        this.softwareString = softwareString;
	        this.version = version;
        }
	}
	
	private Map<String, OutdatedSoftware> softwareFindings;
	@Override
	public String getDescription()
	{
		return "Checks the software versions reported by the web server against " +
				"the current release defined in Nikto (the db_outdated database). " +
				"Some of the logic in retrieving and processing the version " +
				"number is based on logic in nikto_outdated.plugin. " +
				"\n" + 
				Nikto.getNiktoCredits();
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.NIKTO;
	}

	@Override
	public int getModuleNumber()
	{
		return 17;
	}

	@Override
	public String getName()
	{
		return "Nikto version check";
	}

	@Override
	public void initialize()
	{

	}

	public void scanIsComplete()
	{
		softwareFindings = new HashMap<String, OutdatedSoftware>();
		for (String hostUri: Nikto.getServerHeaders().keySet())
		{
			for (String headerValue: Nikto.getServerHeaders().get(hostUri))
			{
				if (!scan.isRunning())
				{
					return;
				}
				analyzeHeader(hostUri, headerValue);
			}
		}
		String description = "";
		for (OutdatedSoftware outdatedSoftware: softwareFindings.values())
		{
			if (!scan.isRunning())
			{
				return;
			}
			outdatedSoftware.transaction.writeToDisk();
			description += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Software</b>: " + outdatedSoftware.version.getFriendlySoftwareName() + "<br>\n" +
				"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Server</b>: " + outdatedSoftware.transaction.getHostUri() + "<br>\n" +
				"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Nikto message</b>: " + outdatedSoftware.version.getWarningMessage(outdatedSoftware.version.extractVersionComponent(outdatedSoftware.softwareString)) + "<br>\n" +
				"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;<b>Transaction</b>: " + HtmlUtils.makeLink(outdatedSoftware.transaction.getSavedUrl(),String.valueOf(outdatedSoftware.transaction.getId())) + "<br>\n<br>\n";
		}
		
		if (description.length() > 0)
		{
			report(description);
		}

	}
	
	
	private void report(String description)
	{
		String shortDescription = "Outdated server software was detected";
		String longDescription = 
			"Using the Nikto database, outdated version of some software was detected. The details can be found below.<br>\n<br>\n " +
			description + "<br>\n" + 
			Nikto.getNiktoCredits();
		String impact = "Outdated software often has security flaws.";
		String recomendations = "Consult the software vendor or project website for the latest version.";
		String references = "";
 			
		ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.LOW, "See description", "Outdated software detected", 
				shortDescription, longDescription, impact, recomendations, references);
		scan.getReportCollector().addEvent(event);
	}
	
	
	private void analyzeHeader(String hostUri, String headerValue)
	{
		for (String software: splitComponents(headerValue))
		{
			if (!scan.isRunning())
			{
				return;
			}
			if (!softwareFindings.containsKey(software)) 
			{
				checkSoftwareVersion(hostUri, headerValue, software);
			}
		}
	}
	
	private void checkSoftwareVersion(String hostUri, String headerValue, String softwareString)
	{
		for (CurrentSoftwareVersion version: Nikto.getCurrentSoftwareVersions())
		{
			if (!scan.isRunning())
			{
				return;
			}
			if (version.doesSoftwareNameMatch(softwareString))
			{
				if (version.isOutdated(softwareString))
				{
			    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(Nikto.getServerHeaderLocations().get(hostUri).get(headerValue));
					OutdatedSoftware outdatedSoftware = new OutdatedSoftware(hostUri, headerValue, softwareString, version, transaction);
					softwareFindings.put(outdatedSoftware.softwareString, outdatedSoftware);
				}
				break;
			}
		}
	}
	
	
	private String[] splitComponents(String header)
	{
		String components[] = {};
		
		if (header.toLowerCase().contains("apache"))
		{
			components = header.split(" ");
		}
		else if (header.toLowerCase().contains("weblogic"))
		{
			String temp[] = header.split(" ");
			if (temp.length > 1)
			{
				components = new String[1];
				components[0] = temp[0] + "/" + temp[1];
			}
		}
		else if (header.toLowerCase().contains("sitescope"))
		{
			components = new String[1];
			String temp[] = header.split(" ");
			components[0] = temp[0];
		}
		else
		{
			String match;
			// if there are no space
			if (header.indexOf(" ") == -1)
			{
				match = header;
			}
			// if there are spaces and a forward slash
			else if (header.indexOf("/") >= 0)
			{
				match = header.replaceAll("\\s+", "");
			}
			// use the last non (alphanumeric, period and parens) as a delimiter
			else
			{
				String delim = header.replaceAll("[a-zA-Z0-9\\.\\(\\)]+", "");
				// get the last character in the string
				delim = delim.substring(delim.length() - 1);
				
				// replace the delimiters with spaces and take off the last section
				match = header.replace(delim, " ").replaceFirst(" [^ ]*$", "");
			}
			match = match.replaceFirst("\\s+$", "");
			components = new String[1];
			components[0] = match;
		}
		return components;
	}

	@Override
    public int[] getPrerequisites()
    {
		int p[] = {16};
		return p;
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }

	public void runAnalysis()
    {
	    // TODO Auto-generated method stub
	    
    }

}
