package com.grendelscan.tests.testModules;

import java.util.Map;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.SQLInjection;
import com.grendelscan.tests.testTypes.ByHttpQueryParameterTest;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.MimeUtils;
import com.grendelscan.utils.StringUtils;
import com.grendelscan.utils.*;


public class Module0021 extends TestModule implements ByHttpQueryParameterTest
{
	@Override
	public String getDescription()
	{
		return "Injects SQL tautologies (such as \"or 1=1 \") and anti-tautologies " +
				"to test for SQL injection. If new pages are accessed through the " +
				"attack, they are processed for new testing and or spidering. The " +
				"test strings are defined in conf/sql_injection.conf.";
	}

	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SQL_INJECTION;
	}

	@Override
	public int getModuleNumber()
	{
		return 21;
	}

	@Override
	public String getName()
	{
		return "SQL Tautologies";
	}

	@Override
	public void initialize()
	{
	}


    public void testByQueryParameter(int transactionID, String parameterName)
    {
    	boolean binary = false;
		if (!scan.isRunning())
		{
			return;
		}
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	if (!MimeUtils.isWebMimeType(transaction.getResponseMimeType()))
    	{
    		binary = true;
    	}
    	
		boolean hit = false;
		String oldValue = transaction.getQueryParameterValue(parameterName);
		if (oldValue.matches("\\d+"))
		{
			hit = testTautologies(transaction, parameterName, SQLInjection.getNumericTautologies(), binary);
		}
		if (!scan.isRunning())
		{
			return;
		}
		if (! hit)
		{
			hit = testTautologies(transaction, parameterName, SQLInjection.getStringTautologies(), binary);
		}
	    
    }
	
	private boolean testTautologies(AbstractHttpTransaction transaction, String parameterName, Map<String, String> tautologies, boolean binary)
	{
		boolean hit = false;
		for (String tautology: tautologies.keySet())
		{
			if (!scan.isRunning())
			{
				break;
			}
			if (testTautologyPair(transaction, parameterName, tautology, tautologies.get(tautology), binary))
			{
				hit = true;
				break;
			}
		}
		return hit;
	}
	
	private boolean testTautologyPair(AbstractHttpTransaction transaction, String parameterName, String tautology, String antiTautology, boolean binary)
	{
		boolean hit = false;
		String oldValue = transaction.getQueryParameterValue(parameterName);
		AbstractHttpTransaction firstTautologyTransaction = transaction.clone(TransactionSource.TEST);
		firstTautologyTransaction.replaceQueryParameter(parameterName, oldValue + tautology);
		firstTautologyTransaction.execute("Module 21", false);
		
		if (! scan.isRunning())
		{
			return hit;
		}
		
		// Run an identical test to see if they differ (some page elements may be random). Use the difference score as 
		// the basis for the taut / anti-taut score
		AbstractHttpTransaction secondTautologyTransaction = transaction.clone(TransactionSource.TEST);
		secondTautologyTransaction.replaceQueryParameter(parameterName, oldValue + tautology);
		secondTautologyTransaction.execute("Module 21", false);
		
		if (! scan.isRunning())
		{
			return hit;
		}

//		float tautThreshold = StringUtils.scoreStringDifferenceIgnoreCase(firstTautologyTransaction.getResponseTextNodes(), secondTautologyTransaction.getResponseTextNodes(), 100) * SQLInjection.getTautologyThreshold();
		double tautThreshold = HttpTransactionUtils.scoreResponseMatch(firstTautologyTransaction, secondTautologyTransaction, 100) * SQLInjection.getTautologyThreshold();

		AbstractHttpTransaction antiTautologyTransaction = transaction.clone(TransactionSource.TEST);
		antiTautologyTransaction.replaceQueryParameter(parameterName, oldValue + antiTautology);
		antiTautologyTransaction.execute("Module 21", false);
		
		if (! scan.isRunning())
		{
			return hit;
		}

		int score;
		if (binary)
		{
			if (firstTautologyTransaction.getResponseBody().equals(antiTautologyTransaction.getResponseBody()))
			{
				score = 100;
			}
			else
			{
				score = 0;
			}
		}
		else
		{
//			score = StringUtils.scoreStringDifferenceIgnoreCase(firstTautologyTransaction.getResponseTextNodes(), antiTautologyTransaction.getResponseTextNodes(), 100);
			score = HttpTransactionUtils.scoreResponseMatch(firstTautologyTransaction, antiTautologyTransaction, 100);
		}
		if ( score < tautThreshold)
		{
			hit = true;
			// If the attack was successful, have it tested. It may lead to other pages.
			scan.getCategorizerQueue().addTransaction(secondTautologyTransaction);
			report(transaction, parameterName, tautology, firstTautologyTransaction, antiTautology, antiTautologyTransaction);
		}
		
		return hit;
	}
	
	private void report(AbstractHttpTransaction transaction, String parameterName, 
			String tautology, AbstractHttpTransaction tautologyTransaction, 
			String antiTautology, AbstractHttpTransaction antiTautologyTransaction)
	{
		tautologyTransaction.writeToDisk();
		antiTautologyTransaction.writeToDisk();
		String briefDescription = "SQL injection found with tautology";
		String longDescription = "A SQL tautology (something that is always true, such as 1=1) was injected into the \"" + parameterName +
			"\" parameter, and appears to have been successful. The tautology was (without the external quotes) \"" + 
			HtmlUtils.makeLink(tautologyTransaction.getSavedUrl(), tautology) + "\" and the anti-tautology was \"" +
			HtmlUtils.makeLink(antiTautologyTransaction.getSavedUrl(), antiTautology) + "\".";
		
		ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.HIGH, transaction.getUriString(), "SQL Injection Detected",
				briefDescription, longDescription, SQLInjection.getSQLInjectionImpact(), SQLInjection.getSQLInjectionRecomendations(),
				SQLInjection.getSQLInjectionReferences());
		scan.getReportCollector().addEvent(event);
	}

	@Override
    public boolean isExperimental()
    {
	    return true;
    }

	@Override
    public String getExperimentalText()
    {
	    return 
	    "Although this module can find SQL injection vulnerabilities, the false-positive " +
	    "rate can be high. More testing is required to tune the heuristics that identify " +
	    "a vulnerable response."; 
    }

}
