package com.grendelscan.tests.testModules;

import org.apache.http.Header;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.TokenTesting.HttpHeaderContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenContext;
import com.grendelscan.tests.libraries.TokenTesting.TokenContextType;
import com.grendelscan.tests.libraries.TokenTesting.TokenContextTypeUtils;
import com.grendelscan.tests.libraries.TokenTesting.TokenTesting;
import com.grendelscan.tests.testTypes.ByRepeatableOutputContextTest;
import com.grendelscan.utils.HtmlUtils;

public class Module0022 extends TestModule implements ByRepeatableOutputContextTest
{
	@Override
    public int[] getPrerequisites()
    {
	    return new int[] {20};
    }
	
	@Override
	public String getDescription()
	{
		return "Tests for CRLF injection into HTTP headers from HTTP query "
		        + "parameters. If the destination is a \"Location\" header "
		        + "(which is most common), redirection to an arbitrary site " + "will also be tested.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.MISCELLANEOUS_ATTACKS;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 22;
	}
	
	@Override
	public String getName()
	{
		return "CRLF Injection";
	}
	
	@Override
	public void initialize()
	{
		
	}
	
	public TokenContextType[] getDesiredRepeatableContexts()
	{
		return TokenContextTypeUtils.getHttpHeaderContexts();
	}
	
	public void testByRepeatableOutputContext(TokenContext context)
	{
		if (!scan.isRunning())
		{
			return;
		}
		HttpHeaderContext headerContext = (HttpHeaderContext) context;
		if (!testCRLFInjection(headerContext) &&
				headerContext.getContextHeader().getName().equalsIgnoreCase("location"))
		{
			testLocation(headerContext);
		}
	}
	
	private boolean testCRLFInjection(HttpHeaderContext headerContext)
	{
		if (!scan.isRunning())
		{
			return false;
		}
		String headerName = TokenTesting.generateToken();
		String headerValue = TokenTesting.generateToken();
		
		String attackString = "blah\r\n" + headerName + ":" + headerValue;
		AbstractHttpTransaction transactions[] =
		        TokenTesting.duplicateTokenTest(headerContext, attackString, "Module " + getModuleNumber(), TransactionSource.TEST);
		if (!scan.isRunning())
		{
			return false;
		}
		
		int match = checkSuccess(transactions, headerName, headerValue);
		if (match > 0)
		{
			crlfReport(transactions[0], headerContext, attackString, headerName, headerValue, transactions[0]);
		}

		return match == 2 ? true : false;
	}
	
	private void crlfReport(AbstractHttpTransaction transaction, HttpHeaderContext context, 
	        String attackString, String headerName, String headerValue, AbstractHttpTransaction attackTransaction)
	{
		attackTransaction.writeToDisk();
		String longDescription =
		        "The query parameter named " + context.getOriginalInputParameterName() + " on "
                + transaction.getUriString() + " appears to be vulnerable to carriage " +
                " return / line feed (CRLF) injection attacks. An attacker can use this " +
                "to craft arbitrary response headers, and, in the right circumstance, " +
                "craft arbitrary response bodies.<br>" +
                "<br>" +
                "The attack was performed using an HTTP " + transaction.getMethod().toUpperCase()
                + " command. When the attack string \"" + HtmlUtils.escapeHTML(attackString)
                + "\" was supplied as the parameter value, it was placed in the \"" + 
                context.getContextHeader().getName() + "\" header. The CRLF allowed a new " +
                "header named \"" + headerName + "\" to be inserted with the value " +
                "\"" + headerValue + "\".<br><br>The transaction used for testing can be viewed " +
                HtmlUtils.makeLink(attackTransaction.getSavedUrl(), "here") + ".";
		if (transaction.getMethod().equalsIgnoreCase("GET"))
		{
			String url = transaction.getUriString();
			longDescription +=
			        "<br><br>This attack can be duplicated by visiting the following URL: " + HtmlUtils.makeLink(url);
		}
		
		String impact = "";

		String recomendation = "Insert the user input into the header without unescaping the " +
								"values (e.g. %0d -> \\x0d)";
		if (context.getContextHeader().getName().equalsIgnoreCase("location"))
		{
			recomendation += "";
		}
			
		ReportableEvent event =
		        new ReportableEvent(null, getName(), EventSeverity.MEDIUM, transaction.getUriString(),
		                "CRLF Injection", "CRLF injection found.",
		                longDescription, impact, recomendation, "");
		scan.getReportCollector().addEvent(event);
	}
	
	
	private void testLocation(HttpHeaderContext headerContext)
	{
		String location = "http://www." + TokenTesting.generateToken() + ".com";
		AbstractHttpTransaction transactions[] = TokenTesting.duplicateTokenTest(headerContext, location, "Module " + getModuleNumber(), TransactionSource.TEST);
		
		if (checkSuccess(transactions, "Location", location) == 2)
		{
			locationReport(transactions[0], headerContext, location);
		}
	}

	private void locationReport(AbstractHttpTransaction transaction, HttpHeaderContext context, 
	        String attackString)
	{
		String longDescription =
		        "The query parameter named " + context.getOriginalInputParameterName() + " on "
                + transaction.getUriString() + " allows for arbitrary location headers to be " +
                "defined in the result.\n" +
                "\n" +
                "The attack was performed using an HTTP " + transaction.getMethod().toUpperCase()
                + " command. When the attack string \"" + attackString
                + "\" was supplied as the parameter value, it was placed in the \"location\" " +
                "header.\n";
		if (transaction.getMethod().equalsIgnoreCase("GET"))
		{
			String url = transaction.getUriString();
			longDescription +=
			        "This attack can be duplicated by visiting the following URL: " + HtmlUtils.makeLink(url);
		}
		
		String impact = "location blah blah blah.";

		String recomendation = "filter blah blah blah";
			
		ReportableEvent event =
		        new ReportableEvent(null, getName(), EventSeverity.LOW, transaction.getUriString(),
		                "Location Header Override", "location header...",
		                longDescription, impact, recomendation, "");
		scan.getReportCollector().addEvent(event);
	}
	

	
	/**
	 * 
	 * @param transactions
	 * @return 0 for no match, 1 for a found header name, 2 for a found header
	 *         name and value
	 */
	private int checkSuccess(AbstractHttpTransaction transactions[], String headerName, String headerValue)
	{
		Header[] headers = transactions[0].getAllResponseHeaders();
		int match = 0;
		for (Header header: headers)
		{
			if (header.getName().equalsIgnoreCase(headerName))
			{
				if (header.getValue().toUpperCase().startsWith(headerValue.toUpperCase()))
				{
					match = 2;
					break;
				}
				match = 1;
			}
		}
		
		return match;
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
	
}
