package com.grendelscan.tests.testModules;

import java.net.URISyntaxException;
import java.util.*;
import java.util.regex.Pattern;
import java.util.regex.PatternSyntaxException;

import org.w3c.dom.Node;
import org.w3c.dom.html2.HTMLElement;

import com.grendelscan.html.HtmlNodeUtilities;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.factories.UriFactory;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.TextOption;
import com.grendelscan.tests.testTypes.ByHtmlElementTest;
import com.grendelscan.utils.*;

public class Module0025 extends TestModule implements ByHtmlElementTest
{
	private List<ConfigurationOption> configOptions;
	private Set<String> testedForms;
	private TextOption whitelistConfigOption;
	private Pattern whitelistPatterns[];
	private List<String> findings;
	
	public Module0025()
	{
		whitelistConfigOption =
		        new TextOption("Action regex whitelist", ".*",
		                "Each line is a regular expression that will be compared against the form's action.", true);
		configOptions = new ArrayList<ConfigurationOption>(1);
		configOptions.add(whitelistConfigOption);
	}
	
	@Override
	public List<ConfigurationOption> getConfigurationOptions()
	{
		return configOptions;
	}
	
	@Override
	public String getDescription()
	{
		return "Tests for cross-site request forgery (CSRF) vulnerabilities in queries "
		        + "based on HTML forms. Some queries forms (e.g. a search function) don't "
		        + "need to be resistant to CSRF. The regular expression white-list is "
		        + "tested against the absolute URL version of the form's action. Each "
		        + "line contains a new regex. Any invalid regex formats will be "
		        + "skipped, so be careful. Note that the default is to test all forms.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.MISCELLANEOUS_ATTACKS;
	}
	
	public String[] getHtmlElements()
	{
		return new String[] { "FORM" };
	}
	
	@Override
	public int getModuleNumber()
	{
		return 25;
	}
	
	@Override
	public String getName()
	{
		return "CSRF";
	}
	
	@Override
	public void initialize()
	{
		whitelistPatterns = RegexUtils.stringToPatterns(whitelistConfigOption.getValue());
		testedForms = new HashSet<String>();
		findings = new ArrayList<String>();
	}
	
	public void testByHtmlElement(int transactionID, HTMLElement element, String elementType)
	{
		String action = getAbsoluteAction(element);
		String formProfile = getFormProfile(element, false);
		
		if (!shouldBeTested(action))
		{
			return;
		}
		
		boolean oldForm;
		synchronized(testedForms)
		{
			/*
			 * This logic is separate so that the duration of the lock 
			 * on testedForms is minimized.
			 */
			oldForm = testedForms.contains(formProfile);
			if (!oldForm)
			{
				testedForms.add(formProfile);
			}
		}
		
		if (!oldForm && scan.isRunning())
		{
			AbstractHttpTransaction originalTransaction = scan.getTransactionRecord().getTransaction(transactionID);
			AbstractHttpTransaction testTransaction = originalTransaction.clone(TransactionSource.TEST);
			testTransaction.execute("Module " + getModuleNumber());
			for (HTMLElement candidate: HtmlNodeUtilities.getChildElements(testTransaction.getResponseDOM(), "form"))
			{
				// If the form is identical
				if (getFormProfile(element, true).equals(getFormProfile(candidate, true)))
				{
					record(originalTransaction, action, getParamNames(element));
					break;
				}
			}
		}
	}
	
	private String getAbsoluteAction(HTMLElement element)
	{
		String action = element.getAttribute("action");
		String uri = "";
		if ((action == null) || action.equals(""))
		{
			action = element.getBaseURI();
		}
		try
		{
			uri = UriFactory.makeAbsoluteUri(action, element.getBaseURI()).toASCIIString();
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("Invalid action format: " + e.toString(), e);
		}
		return uri;
	}
	
	private class inputComparator implements Comparator<HTMLElement>
	{
		public int compare(HTMLElement element1, HTMLElement element2)
        {
			String n1 = element1.getAttribute("name");
			String n2 = element2.getAttribute("name");

			if (n1 == null)
			{
				n1 = "";
			}
			if (n2 == null)
			{
				n2 = "";
			}

	        return n1.compareTo(n2);
        }
	}
	
	private String getFormProfile(HTMLElement element, boolean useValues)
	{
		String profile = getAbsoluteAction(element) + "?";
		if (element.getTagName().equalsIgnoreCase("form"))
		{
			List<HTMLElement> inputs = HtmlNodeUtilities.getChildElements(element, "input");
			inputs.addAll(HtmlNodeUtilities.getChildElements(element, "textarea"));
			Collections.sort(inputs, new inputComparator());
			for (HTMLElement input: inputs)
			{
				String name = input.getAttribute("name");
				if ((name != null) && !name.equals(""))
				{
					profile += name;
					if (useValues)
					{
						String value = "";
						if (profile.equalsIgnoreCase("textarea"))
						{
							Node child = input.getFirstChild();
							if (child != null)
							{
								value = child.getNodeValue();
							}
						}
						else
						{
							value = input.getAttribute("value");
						}
						
						if (value == null)
						{
							value = "";
						}
						profile += "=" + value;
					}
					profile += "&";
				}
			}		
		}
		return profile.toUpperCase();
	}
	
	private String[] getParamNames(HTMLElement element)
	{
		List<String> params = new ArrayList<String>(1);
		
		List<HTMLElement> inputs = HtmlNodeUtilities.getChildElements(element, "input");
		inputs.addAll(HtmlNodeUtilities.getChildElements(element, "textarea"));
		for (HTMLElement input: inputs)
		{
			String name = input.getAttribute("name");
			if ((name != null) && !name.equals(""))
			{
				params.add(name);
			}
		}
		return params.toArray(new String[0]);
	}
	
	private void record(AbstractHttpTransaction originalTransaction, String action, String[] params)
	{
		originalTransaction.writeToDisk();
		String query = "";
		for (int i = 0; i < params.length; i++)
		{
			if (params.length > 1)
			{
				if (i == params.length - 1)
				{
					query += ", and ";
				}
				else if (i > 0)
				{
					query += ", ";
				}
			}
			query += "\"" + params[i] + "\"";
		}
		
		String description = 
					         "<b>Action</b>: " + action + "<br>" 
					        + "<b>Parameter names</b>: " + query + "<br>"
					        + "<b>Original transaction</b>: " + HtmlUtils.makeLink(originalTransaction.getSavedUrl(), String.valueOf(originalTransaction.getId())) + "<br>"
					        + "<br>" ;
		findings.add(description);
	}
	
	private void report()
	{
		if (findings.size() > 0)
		{
			String title = "Potential CSRF detected";
			String shortDescription = "A cross-site request forgery (CSRF) vulnerability may have been identified.";
			String longDescription =
			        "One or more cross-site request forgery (CSRF) vulnerabilities may have been identified. "
			                + "CSRF allows an attacker to force a user to execute arbitrary commands "
			                + "against the vulnerable website. This is possible when the structure of the "
			                + "command is predictable. If the command can be requested as a GET, then a "
			                + "simple IMG tag on an attack website can force the browser to send the "
			                + "command. A POST request can be sent using some simple JavaScript. The "
			                + "browser will send any cookies or authentication credentials associated "
			                + "with the targeted attack, because it has no way of knowing that the "
			                + "request was not intentionally executed by the user.<br>" + "<br>"
			                + "A list of queries that appear to be vulnerable to CSRF is below. " 
			                + "A specific form may be found on multiple pages, but was only tested once.<br>"
			                + "<br>";
			
			for (String finding: findings)
			{
				longDescription += finding;
			}
					
			String impact = "The impact varies depending on the nature of the command.";
			String recomendations =
			        "If this query is used to execute a sensitive function "
			                + "(e.g. creating a new user), then CSRF is a significant vulnerability. It "
			                + "can be prevented by using a random token as a query parameter. Before "
			                + "executing the command, the application should confirm that the token sent by "
			                + "the browser matches the original token sent by the application. Since an "
			                + "attacker couldn't know the random token (without a secondary vulnerability "
			                + "like XSS), a CSRF attack is not possible.";
			String references = HtmlUtils.makeLink("http://www.owasp.org/index.php/Cross-Site_Request_Forgery");
			ReportableEvent event =
			        new ReportableEvent(null, "Module " + getModuleNumber(), EventSeverity.HIGH, "See description", title, shortDescription, longDescription, impact, recomendations,
			                references);
			scan.getReportCollector().addEvent(event);
		}
	}
	
	private boolean shouldBeTested(String action)
	{
		boolean test = false;
		
		for (Pattern white: whitelistPatterns)
		{
			if (white.matcher(action).find())
			{
				test = true;
				break;
			}
		}
		return test;
	}

	@Override
    public boolean isExperimental()
    {
	    return true;
    }

	@Override
    public String getExperimentalText()
    {
	    return 
	    "The CSRF module is particularly prone to false positive findings.";
    }

	@Override
    public void scanIsComplete()
    {
		report();
    }
	
	
}
