package com.grendelscan.tests.testModules;

import java.util.*;
import java.util.regex.Pattern;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.*;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.TextOption;
import com.grendelscan.tests.testTypes.ByFileTest;
import com.grendelscan.utils.HttpTransactionUtils;
import com.grendelscan.utils.MimeUtils;
import com.grendelscan.utils.RegexUtils;
import com.grendelscan.utils.StringUtils;

public class Module0026 extends TestModule implements ByFileTest
{
	private Map<String, Integer> vulnerableUrls;
	private List<ConfigurationOption> configOptions;
	private TextOption whitelistConfigOption;
	private Pattern whitelistPatterns[];
	
	public Module0026()
	{
		whitelistConfigOption =
	        new TextOption("URL regex whitelist", ".*",
	                "Each line is a regular expression that will be compared against the base URL.", true);
		configOptions = new ArrayList<ConfigurationOption>(1);
		configOptions.add(whitelistConfigOption);
	}
	
	@Override
	public List<ConfigurationOption> getConfigurationOptions()
	{
		return configOptions;
	}

	
	@Override
	public String getDescription()
	{
		return "Tests to see if pages originally requested over HTTPS are accessible over plaintext HTTP. The " +
				"regular expression whitelist will determine which pages are tested. The regex is compared " +
				"to the base URL, but ignores any query. Each "
		        + "line contains a new regex. Any invalid regex formats will be "
		        + "skipped, so be careful. Note that the default is to test all pages.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.MISCELLANEOUS_ATTACKS;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 26;
	}
	
	@Override
	public String getName()
	{
		return "Insecure transmission";
	}
	
	@Override
	public void initialize()
	{
		whitelistPatterns = RegexUtils.stringToPatterns(whitelistConfigOption.getValue());
		vulnerableUrls = new HashMap<String, Integer>();
	}
	
	public void testByFile(int transactionID)
	{
		AbstractHttpTransaction originalTransaction = scan.getTransactionRecord().getTransaction(transactionID);
		// We're not interested in HTTP, redirects, or non-html responses
		if (!originalTransaction.isSecure() || 
			!shouldBeTested(originalTransaction.getBaseUri()) || 
			originalTransaction.getLogicalResponseCode() != 200 || 
			!MimeUtils.isHtmlMimeType(originalTransaction.getResponseMimeType()))
		{
			return;
		}
		
		AbstractHttpTransaction testTransaction = originalTransaction.clone(TransactionSource.TEST);
		testTransaction.setURI("http" + testTransaction.getAbsoluteUriString().substring(5));
		testTransaction.execute("Module " + getModuleNumber());

		if (testTransaction.getLogicalResponseCode() == 200 
				&& HttpTransactionUtils.scoreResponseMatch(originalTransaction, testTransaction, 100) < 95)
		{
			vulnerableUrls.put(originalTransaction.getBaseUri(), testTransaction.getId());
		}
		
	}
	
	private boolean shouldBeTested(String url)
	{
		boolean test = false;
		
		for (Pattern white: whitelistPatterns)
		{
			if (white.matcher(url).find())
			{
				test = true;
				break;
			}
		}
		return test;
	}

	
	private void report()
	{
		String page = vulnerableUrls.size() > 1 ? "Pages" : "A page";
		String verb = vulnerableUrls.size() > 1 ? "are" : "is";
		String title = "Insecure transmission detected";
		String shortDescription = page + " normally transmitted over HTTPS " + verb + " accessible via plaintext HTTP.";
		String longDescription =
		        page + " normally transmitted over HTTPS " + verb + " accessible via plaintext HTTP.<br>" + 
		        "<br>" +
		        "The vulnerable URL(s) are listed below. The links are to the insecure transaction.<br><ul>";
		for(String url: vulnerableUrls.keySet())
		{
	    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(vulnerableUrls.get(url));
	    	transaction.writeToDisk();
			longDescription += "<li><a href=\"" + transaction.getSavedUrl() + "\">" + url + "</a></li><br>";
		}
		longDescription += "</ul>";
		String impact = 
			"If sensitive data (authentication credentials, session IDs, etc) is involved, " +
			"it can be viewed or modified by an attacker that is " +
			"able to view network traffic, or able to launch a man-in-the-middle (MITM) attack.";
		String recomendations =
		        "Determine if the transaction in question contains sensitive information. If " +
		        "it does, configure the web or application server to only allow the transaction " +
		        "over HTTPS.";
		String references = "";
		ReportableEvent event =
		        new ReportableEvent(null, "Module " + getModuleNumber(), EventSeverity.LOW, 
		        		vulnerableUrls.get(0) + (vulnerableUrls.size() > 1 ? " and others" : ""), 
		        		title, shortDescription, longDescription, impact, recomendations,
		                references);
		scan.getReportCollector().addEvent(event);
		
	}

	@Override
    public void scanIsComplete()
    {
	    if (vulnerableUrls.size() > 0)
	    {
	    	report();
	    }
    }

	@Override
    public boolean isExperimental()
    {
	    return false;
    }

	
}
