package com.grendelscan.tests.testModules;

import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.factories.HttpTransactionFactory;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testTypes.ByDirectoryTest;
import com.grendelscan.utils.Debug;
import com.grendelscan.utils.*;

public class Module0027 extends TestModule implements ByDirectoryTest
{
	private static Pattern titlePattern = Pattern.compile("<title(.+?)</title>", Pattern.CASE_INSENSITIVE);
	private String[] basicPatterns;
	private List<String> listableDirs;
	
	private String[] titlePatterns;
	
	@Override
	public String getDescription()
	{
		return "Checks for directory listing in all directories discovered by Grendel-Scan.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.INFORMATION_LEAKAGE;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 27;
	}
	
	@Override
	public String getName()
	{
		return "Directory listings";
	}
	
	@Override
	public void initialize()
	{
		listableDirs = new ArrayList<String>();
		basicPatterns = ConfigurationManager.getStringArray("directory_indexing_patterns.basic");
		titlePatterns = ConfigurationManager.getStringArray("directory_indexing_patterns.title");
	}
	
	@Override
	public void scanIsComplete()
	{
		if (listableDirs.size() > 0)
		{
			report();
		}
	}
	
	public void testByDirectory(int transactionID, String directory)
	{
		AbstractHttpTransaction originalTransaction = scan.getTransactionRecord().getTransaction(transactionID);
		String uri = originalTransaction.getHostUriWithoutTrailingSlash() + directory;
		try
		{
			AbstractHttpTransaction testTransaction;
			if (uri.equals(originalTransaction.getAbsoluteUriString()))
			{
				testTransaction = originalTransaction;
			}
			else
			{
				testTransaction = HttpTransactionFactory.createTransaction(scan, "GET", uri, 0, TransactionSource.TEST);
				testTransaction.execute("Module " + getModuleNumber());
			}
			if (isDirectoryListing(testTransaction))
			{
				listableDirs.add(uri);
			}
		}
		catch (URISyntaxException e)
		{
			Debug.errDebug("Problem with URI format (" + HtmlUtils.makeLink(uri) + "): " + e.toString(), e);
		}
	}
	
	private boolean isDirectoryListing(AbstractHttpTransaction testTransaction)
	{
		boolean directoryListing = false;
		if (MimeUtils.isWebMimeType(testTransaction.getResponseMimeType()))
		{
			String escapedDirectory = Pattern.quote(testTransaction.getDirectory());
			String content = testTransaction.getResponseBodyAsString();
			for (String rawPattern: basicPatterns)
			{
				Pattern pattern =
				        Pattern.compile(rawPattern.replace("%%dir%%", escapedDirectory), Pattern.CASE_INSENSITIVE);
				if (pattern.matcher(content).find())
				{
					directoryListing = true;
					break;
				}
			}
			
			if (!directoryListing)
			{
				Matcher m = titlePattern.matcher(content);
				if (m.find())
				{
					String titleText = m.group(1);
					
					for (String rawPattern: titlePatterns)
					{
						Pattern pattern =
						        Pattern.compile(rawPattern.replace("%%dir%%", escapedDirectory), Pattern.CASE_INSENSITIVE);
						if (pattern.matcher(titleText).find())
						{
							directoryListing = true;
							break;
						}
					}
				}
			}
		}
		return directoryListing;
	}
	
	private void report()
	{
		String directory = listableDirs.size() > 1 ? "Directories" : "A directory";
		String verb = listableDirs.size() > 1 ? " were" : " was";
		String title = "Directory content listing detected";
		String shortDescription = directory + verb + " found supporting content listing.";
		String longDescription =
		        directory + verb + " found supporting content listing."
		                + "The vulnerable directories(s) are listed below:<br>";
		for (String url: listableDirs)
		{
			longDescription += "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;" + HtmlUtils.makeLink(url) + "<br>\n";
		}
		String impact =
		        "Directory listings can be used to explore web site content that would "
		                + "otherwise be unknown to an attacker.";
		String recomendations = "Disable directory content listing on all web servers.";
		String references = "";
		ReportableEvent event =
		        new ReportableEvent(null, "Module " + getModuleNumber(), EventSeverity.INFO, listableDirs.get(0)
		                + (listableDirs.size() > 1 ? " and others" : ""), title, shortDescription, longDescription,
		                impact, recomendations, references);
		scan.getReportCollector().addEvent(event);
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
	
}
