package com.grendelscan.tests.testModules;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testTypes.ByHttpQueryParameterTest;
import com.grendelscan.utils.*;

public class Module0028 extends TestModule implements ByHttpQueryParameterTest
{
	private String antiPattern;
	private String patterns[];
	private int antiPatternThreshold;
	private int patternThreshold;
	

	@Override
	public String getDescription()
	{
		return 
			"Tests for directory traversal vulnerabilities in query parameters. This is done " +
			"by inserting relative path strings. For example, if the default parameter value " +
			"is 'asdf', then '.asdf' is request. If this causes a different result page, " +
			"then './asdf', '.\\asdf', etc are requested. If they result in the same response " +
			"as the original request, then a directory traversal attack may be possible.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.MISCELLANEOUS_ATTACKS;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 28;
	}
	
	@Override
	public String getName()
	{
		return "Directory traversal";
	}
	
	@Override
	public void initialize()
	{
		antiPattern = ConfigurationManager.getString("directory_traversal.antipattern");
		patterns = ConfigurationManager.getStringArray("directory_traversal.patterns");
		antiPatternThreshold = ConfigurationManager.getInt("directory_traversal.antipattern_threshold");
		patternThreshold = ConfigurationManager.getInt("directory_traversal.pattern_threshold");
		
	}
	
	public void testByQueryParameter(int transactionID, String parameterName)
	{
    	AbstractHttpTransaction originalTransaction = scan.getTransactionRecord().getTransaction(transactionID);
    	String oldValue = originalTransaction.getQueryParameterValue(parameterName);
    	/**
    	 * We don't check response types later because we assume that if
    	 * the original response is good, the rest probably will be too.
    	 * Nothing bad will happen if they aren't, it's just wasted time.
    	 */
    	if (oldValue.equals("") || !MimeUtils.isWebMimeType(originalTransaction.getResponseMimeType()))
    	{
    		return;
    	}
    	
    	AbstractHttpTransaction antitestTransaction = originalTransaction.clone(TransactionSource.TEST);
    	// If it's not a redirected transaction
    	boolean redirect = false;
    	if (originalTransaction.getOriginalResponseTransactionID() > 0)
    	{
    		redirect = true;
    	}
    	String antiValue = antiPattern.replace("%%value%%", oldValue);
    	antitestTransaction.replaceQueryParameter(parameterName, antiValue);
    	antitestTransaction.setFollowRedirects(redirect);
    	antitestTransaction.execute("Module " + getModuleNumber());
		if (HttpTransactionUtils.scoreResponseMatch(originalTransaction, antitestTransaction, 100) < antiPatternThreshold)
    	{
    		for (String pattern: patterns)
    		{
    			AbstractHttpTransaction testTransaction = originalTransaction.clone(TransactionSource.TEST);
    			testTransaction.setFollowRedirects(redirect);
    			String testValue = pattern.replace("%%value%%", oldValue);
    			testTransaction.replaceQueryParameter(parameterName, testValue);
    			testTransaction.execute("Module " + getModuleNumber());
    			if (HttpTransactionUtils.scoreResponseMatch(originalTransaction, testTransaction, 100) < patternThreshold &&
    					HttpTransactionUtils.scoreResponseMatch(antitestTransaction, testTransaction, 100) < antiPatternThreshold)
    	    	{
    	    		report(originalTransaction, testTransaction, parameterName, oldValue, antiValue, testValue);
    	    		break;
    	    	}
    			
    		}
    	}
	}
	
	private void report(AbstractHttpTransaction originalTransaction, AbstractHttpTransaction testTransaction,
			String parameterName, String originalValue, String antiValue, String testValue)
	{
		originalTransaction.writeToDisk();
		testTransaction.writeToDisk();
		String title = "Directory traversal vulnerability";
		String shortDescription = "A possible directory traversal vulnerability was detected.";
		String longDescription =
				"A possible directory traversal vulnerability was detected in the \"" + parameterName +
				"\" parameter of " + originalTransaction.getBaseUri() + ". The original value " +
				"of the parameter was \"" + originalValue + "\". When the value of \"" + antiValue +
				"\" was used instead, it appears that a different page was returned, perhaps an " +
				"error message. When a value of \"" + testValue + "\" was used, it appears that " +
				"the response matched the original request. This implies that the parameter is " +
				"vulnerable to directory traversal attacks. However, this test may be prone to " +
				"false positives, so further investigation is recommended. The original transaction " +
				"can be viewed <a href=\"" + originalTransaction.getSavedUrl() + "\">here</a> and " +
				"the transaction used for testing can be viewed " +
				"<a href=\"" + testTransaction.getSavedUrl() + "\">here</a>.";
		String impact = 
				"Depending on the function of the page, directory traversal attacks might be " +
				"used to read or execute arbitrary files already on the server.";
		String recomendations =
		        "Confirm that this is a directory traversal vulnerability. If it is, at the very " +
		        "least, modify the application to sanitize input, blocking any non-alphanumeric " +
		        "characters. Ideally, a user should never be able to directly reference a file " +
		        "by name in a query parameter.";
		String references = "";
		ReportableEvent event =
		        new ReportableEvent(null, "Module " + getModuleNumber(), EventSeverity.INFO, 
		        		originalTransaction.getBaseUri(), title, shortDescription, longDescription, 
		        		impact, recomendations, references);
		scan.getReportCollector().addEvent(event);
	}

	@Override
    public boolean isExperimental()
    {
	    return true;
    }

	@Override
    public String getExperimentalText()
    {
	    
	    return 
	    "While this module can discover directory traversal vulnerabilities, more testing " +
	    "is required to reduce false positives and false negatives.";
    }
}
