package com.grendelscan.tests.testModules;

import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testTypes.ByHttpResponseCodeTest;

import java.io.*;
import java.util.*;
import java.lang.*;

import com.grendelscan.tests.testModules.settings.*;
import com.grendelscan.utils.*;
public class Module0030 extends TestModule implements ByHttpResponseCodeTest
{
	private FileNameOption outputDirectoryOption;
	private SelectableOption recordQueriesOption;
	private IntegerOption maxFileSize;
	private SelectableOption recordAllHTMLFilesOption;
	private List<ConfigurationOption> configOptions;
	private String baseDir;

	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	public Module0030()
	{
		configOptions = new ArrayList<ConfigurationOption>(3);
		outputDirectoryOption = new FileNameOption("Output directory","site-mirror", "The directory for the website mirror.", true);
		configOptions.add(outputDirectoryOption);

		recordQueriesOption = new SelectableOption("Record URL queries", true, "If selected, all unique URL query strings will be saved to separate files.");
		configOptions.add(recordQueriesOption);

		recordAllHTMLFilesOption = new SelectableOption("Record all files", true, "If not selected, only the URLs to web files (HTML, XHTML, XML, JavaScript, CSS, and text) will be recorded. This will omit images, audio, video, PDFs, etc");
		configOptions.add(recordAllHTMLFilesOption);
		
		maxFileSize = new IntegerOption("Max file size", 1000000, "The maximum amount of bytes to write to disk");
		configOptions.add(maxFileSize);
	}
	
	@Override
	public String getDescription()
	{
		return "Saves pages to disk, essentially mirroring the targeted website. Requests used only for test purposes are not saved. Right now, only the first request for a file is saved.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.ARCHITECTURE;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 30;
	}
	
	@Override
	public String getName()
	{
		return "Website mirror";
	}
	
	@Override
	public void initialize()
	{
		if (outputDirectoryOption.getValue().contains(File.separator))
		{
			baseDir = outputDirectoryOption.getValue() + File.separator;
		}
		else
		{
			baseDir = scan.getScanSettings().getOutputDirectory() + File.separator + outputDirectoryOption.getValue() + File.separator;
		}

	}
	
	public String[] getResponseCodes()
	{
	    return new String[] {"200"};
	}
	
	public void testByHttpResponseCode(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	if (recordAllHTMLFilesOption.isSelected() || MimeUtils.isWebMimeType(transaction.getResponseMimeType()))
    	{
    		saveToDisk(transaction);
    	}
	}
	
	private void saveToDisk(AbstractHttpTransaction transaction)
	{
		String fileName = getFileName(transaction);
		if (!FileUtils.fileExists(fileName))
		{
			String content = transaction.getResponseBodyAsString();
			int endIndex = content.length() > maxFileSize.getValue() ? maxFileSize.getValue() : content.length();
			FileUtils.writeToFile(fileName, content.substring(0, endIndex));
		}
	}
	
	private String getFileName(AbstractHttpTransaction transaction)
	{
		String path = baseDir;
		
		for (String dir: transaction.getDirectory().split("/"))
		{
			if (dir.length() > 0)
			{
				path += dir + File.separator;
			}
		}
		FileUtils.createDirectories(path);
		String originalFilename = transaction.getFilename();
		if (originalFilename.equals(""))
		{
			originalFilename = "_ROOT_";
		}
		String filename = originalFilename;
		if (recordQueriesOption.isSelected() && transaction.getUriQueryString().length() > 0)
		{
			String query = transaction.getUriQueryString();
			query = query.replace("?", "_QUESTION_");
			query = query.replace("/", "_FSLASH_");
			query = query.replace("\\", "_BSLASH_");
			query = query.replace("<", "_LT_");
			query = query.replace(">", "_GT_");
			query = query.replace(":", "_COL_");
			query = query.replace("*", "_STAR_");
			query = query.replace("|", "_PIPE_");
			query = query.replace("\"", "_DQUOTE_");
			query = query.replace("^", "_CARET_");
			query = query.replace("|", "_PIPE_");
			query = query.replace("|", "_PIPE_");
			query = query.replace("|", "_PIPE_");
			filename += "_QUESTION_" + query;
		}
		// fix the length before the extension is added
		int endIndex = filename.length() > 250 ? 250 : filename.length();
		filename = filename.substring(0, endIndex);

		String extension = MimeUtils.getFileExtension(transaction.getResponseMimeType());
		if (!filename.toLowerCase().endsWith(extension))
		{
			filename += extension;
		}
		return path + filename;
	}

	@Override
    public void scanIsComplete()
    {
		report();
    }

	private void report()
	{
		String title = "Website mirror";
		String shortDesc = "A mirror of discovered websites was created.";
		String longDesc = "A mirror of discovered websites was created and saved to " + HtmlUtils.makeLink(outputDirectoryOption.getValue()) +
			".";
		ReportableEvent event = new ReportableEvent(null, "Module " + getModuleNumber(), EventSeverity.INFO, "", 
				title, shortDesc, longDesc, "", "", "");
		scan.getReportCollector().addEvent(event);
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
}
