package com.grendelscan.tests.testModules;
import java.util.regex.*;
import java.net.URISyntaxException;
import com.grendelscan.utils.*;
import com.grendelscan.requester.http.transactions.*;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.tests.testTypes.ByHostTest;
import com.grendelscan.requester.http.factories.*;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;

import java.util.*;
public class Module0031 extends TestModule implements ByHostTest
{
	private Map<String, Set<String>> disallowed; 
	@Override
	public String getDescription()
	{
		return "Looks for a robots.txt file. If one is found, all paths are requested, and " +
				"disallowed paths (or similar directives) are documented in the scan report.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.INFORMATION_LEAKAGE;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 31;
	}
	
	@Override
	public String getName()
	{
		return "Robots.txt";
	}
	
	@Override
	public void initialize()
	{
		disallowed = new HashMap<String, Set<String>>(1);
	}
	
	public void testByServer(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	if (! scan.isRunning())
    	{
    		return;
    	}
    	String robotsUri = transaction.getHostUri() + "robots.txt";
    	try
        {
	        HttpGetTransaction robotGet = new HttpGetTransaction(scan, robotsUri, 0, TransactionSource.TEST);
	        robotGet.execute("Module " + getModuleNumber());
	        if (HttpUtils.fileExists(robotGet.getLogicalResponseCode()))
	        {
	        	String robotsTxt = robotGet.getResponseBodyAsString();
	        	String lines[] = robotsTxt.split("[\r\n]+");
	        	boolean init = false;
	        	Set<String> uris = new HashSet<String>();
	        	Pattern p = Pattern.compile("(disallow|allow):\\s+(\\S+)", Pattern.CASE_INSENSITIVE);
	        	for (String line: lines)
	        	{
	        		Matcher m = p.matcher(line);
	        		if (m.find())
	        		{
		        		String directive = m.group(1).toLowerCase();
		        		String path = m.group(2);
		        		if (!directive.equals("user-agent"))
		        		{
			        		String uri = transaction.getHostUriWithoutTrailingSlash() + path;
		        			try
		        			{
		        				HttpGetTransaction disallowGet = new HttpGetTransaction(scan, uri, 0, TransactionSource.TEST);
			        			scan.getRequesterQueue().addRequest(disallowGet, "Module " + getModuleNumber());
		        			}
		        			catch (URISyntaxException e) 
							{
		        				Debug.errDebug("Bad URI in robots.txt (" + uri + "): " + e.toString(), e);
							}
		        			if (isBlockDirective(directive))
		        			{
			        			if (!init)
			        			{
			        				init = true;
			        				disallowed.put(transaction.getHostUri(), uris);
			        			}
		        				uris.add(path);
		        			}
		        		}
	        		}
	        	}
	        }
        }
        catch (URISyntaxException e)
        {
	        Debug.errDebug("Problem with robots.txt URI ("+ robotsUri + "): " + e.toString(), e);
        }
    	
	}
	
	@Override
    public void scanIsComplete()
    {
		if (disallowed.size() > 0)
		{
			String title = "Blocked paths in robots.txt";
			String briefDescription = "Blocked paths were discovered in robots.txt";
			String longDescription = "Blocked paths were discovered in robots.txt, which is primarily used for controlling search engine access. The " +
					"paths are listed below:<br>";
			String firstUri = "";
			for (String host: disallowed.keySet())
			{
				if (firstUri.equals(""))
				{
					firstUri = host + "robots.txt";
				}
				longDescription += "<br>" + HtmlUtils.makeLink(host + "robots.txt") + ":<br>";
				for (String path: disallowed.get(host))
				{
					longDescription += "&nbsp;&nbsp;&nbsp;&nbsp;" + path + "<br>";
				}
			}
			String impact = "Robots.txt should not be used to hide sensitive areas of a website. Not only is it ineffective, it provides information " +
					"to potential attackers.";
			String recomendations = "Confirm that robots.txt is only being used to optimize search engines, and is not being used as a security " +
					"control.";
			String references = HtmlUtils.makeLink("http://www.robotstxt.org/") + "<br>" + HtmlUtils.makeLink("http://en.wikipedia.org/wiki/Robots_exclusion_standard");
			
			ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.INFO, firstUri + (disallowed.size() > 1 ? " and others" : ""), 
					title, briefDescription, longDescription, impact, recomendations, references);
			scan.getReportCollector().addEvent(event);
		}
    }

	private boolean isBlockDirective(String directive)
	{
		boolean block = false;
		if (directive.equals("noindex")
				|| directive.equals("nofollow")
				|| directive.equals("noarchive")
				|| directive.equals("opreview")
				|| directive.equals("nosnippet")
				|| directive.equals("disallow")
			)
		{
			block = true;
		}
		
		return block;
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
	
}
