package com.grendelscan.tests.testModules;

import java.util.*;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.UnvalidatedHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.libraries.TokenTesting.TokenTesting;
import com.grendelscan.tests.testTypes.ByHostTest;

public class Module0033 extends TestModule implements ByHostTest
{
	
	@Override
	public String getDescription()
	{
		return "Attempts to identify web servers that will proxy requests to other hosts. " +
				"This is most accurate if the proxy has access to Internet sites.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.WEB_SERVER_CONFIGURATION;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 33;
	}
	
	@Override
	public String getName()
	{
		return "Proxy detection";
	}
	
	@Override
	public void initialize()
	{
	}
	
	public void testByServer(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		if (testConnect(transaction) || testInternet(transaction) || testIntranet(transaction))
		{
		}
	}
	
	private boolean testConnect(AbstractHttpTransaction transaction)
	{
		boolean proxy = false;
		
		UnvalidatedHttpTransaction testTransaction = new UnvalidatedHttpTransaction(scan, 0, "CONNECT", "www.grendel-scan.com:443", transaction.getHost(), transaction.getPort(), false, TransactionSource.TEST, null);
		testTransaction.addRequestHeader("Host", "www.grendel-scan.com");
		testTransaction.execute("Module " + getModuleNumber(), false, true);
		String detectionMethod = "";
		if (testTransaction.getResponseStatusCode() == 200)
		{
			proxy = true;
			detectionMethod = "A CONNECT command was issued for \"www.grendel-scan.com:443\". The response code was 200, indicating " +
					"a successful connection. Note that this test is not very sophisticated right now, and should be manually confirmed.";
		}
		else if (testTransaction.getResponseStatusCode() == 504) 
		{
			proxy = true;
			detectionMethod = "A CONNECT command was issued for \"www.grendel-scan.com:443\". The response code was 504, indicating " +
			"a gateway timeout. This implies that the proxy server tried to connect to the host, but was unsuccessful.";
		}
		else if (testTransaction.getResponseStatusCode() == 407)
		{
			proxy = true;
			detectionMethod = "A CONNECT command was issued for \"www.grendel-scan.com:443\". The response code was 407, which is a " +
			"request for proxy authentication. This implies that the server accepts proxy requests, but only with authentication " +
			"credentials.";
		}
		if (proxy)
		{
			report(transaction.getHost() + ":" + transaction.getPort(), detectionMethod, testTransaction);
		}
		return proxy;
	}
	
	private boolean testInternet(AbstractHttpTransaction transaction)
	{
		boolean proxy = false;
		
		UnvalidatedHttpTransaction testTransaction = new UnvalidatedHttpTransaction(scan, 0, "GET", "http://www.grendel-scan.com/proxy.txt", transaction.getHost(), transaction.getPort(), false, TransactionSource.TEST, null);
		testTransaction.addRequestHeader("Host", "www.grendel-scan.com");
		testTransaction.execute("Module " + getModuleNumber(), false, true);
			
		String detectionMethod = "";
		if (testTransaction.getResponseBodyAsString().contains("grendel-scan proxy test"))
		{
			proxy = true;
			detectionMethod = "A request for http://www.grendel-scan.com/proxy.txt was sent to the server. The response contained the text " +
					"\"grendel-scan proxy test\", demonstrating a successful request.";
		}
		else if (testTransaction.getResponseStatusCode() == 504) 
		{
			proxy = true;
			detectionMethod = "A request for http://www.grendel-scan.com/proxy.txt was sent to the server. The response code was 504, indicating " +
			"a gateway timeout. This implies that the proxy server tried to connect to the host, but was unsuccessful.";
		}
		else if (testTransaction.getResponseStatusCode() == 407)
		{
			proxy = true;
			detectionMethod = "A request for http://www.grendel-scan.com/proxy.txt was sent to the server. The response code was 407, which is a " +
			"request for proxy authentication. This implies that the server accepts proxy requests, but only with authentication " +
			"credentials.";
		}
		if (proxy)
		{
			report(transaction.getHost() + ":" + transaction.getPort(), detectionMethod, testTransaction);
		}
		return proxy;
	}
	
	private boolean testIntranet(AbstractHttpTransaction transaction)
	{
		boolean proxy = false;
		
		UnvalidatedHttpTransaction testTransaction = new UnvalidatedHttpTransaction(scan, 0, "GET", "http://randomhost.tld/", transaction.getHost(), transaction.getPort(), false, TransactionSource.TEST, null);
		testTransaction.addRequestHeader("Host", "randomhost.tld");
		testTransaction.execute("Module " + getModuleNumber(), false, true);
		String detectionMethod = "";
		if (testTransaction.getResponseStatusCode() == 504) 
		{
			proxy = true;
			detectionMethod = "A request for http://randomhost.tld/ was sent to the server. The response code was 504, indicating " +
			"a gateway timeout. This implies that the proxy server tried to connect to the fictional host, but was unsuccessful.";
		}
		else if (testTransaction.getResponseStatusCode() == 407)
		{
			proxy = true;
			detectionMethod = "A request for http://randomhost.tld/ was sent to the server. The response code was 407, which is a " +
			"request for proxy authentication. This implies that the server accepts proxy requests, but only with authentication " +
			"credentials.";
		}
		if (proxy)
		{
			report(transaction.getHost() + ":" + transaction.getPort(), detectionMethod, testTransaction);
		}
			
		return proxy;
	}
	

	private void report(String hostAndPort, String detectionMethod, AbstractHttpTransaction transaction)
	{
		transaction.writeToDisk();
		String title = "Proxy server detected";
		String briefDescription = "A web server is also acting as a proxy server.";
		String longDescription = "The web server at " + hostAndPort + " is also acting as a proxy server. " + detectionMethod +
				"The transaction used for testing can be viewed " +
						"<a href=\"" + transaction.getSavedUrl() + "\">here</a>.";
		
		String impact = "An attacker might be able to access web servers through the proxy server that would otherwise be blocked " +
				"by a firewall. If the CONNECT method is supported, any TCP protocol can be tunneled.";
		String recomendations = "Disable proxying capabilities.";
		String references = "";
		
		ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.LOW, hostAndPort, 
				title, briefDescription, longDescription, impact, recomendations, references);
		scan.getReportCollector().addEvent(event);
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }

}
