package com.grendelscan.tests.testModules;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.w3c.dom.Node;
import org.w3c.dom.html2.HTMLScriptElement;

import com.grendelscan.html.HtmlNodeUtilities;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.EventSeverity;
import com.grendelscan.scan.reporting.ReportableEvent;
import com.grendelscan.tests.testModules.settings.ConfigurationOption;
import com.grendelscan.tests.testModules.settings.FileNameOption;
import com.grendelscan.tests.testModules.settings.MultiSelectOptionGroup;
import com.grendelscan.tests.testModules.settings.SelectableOption;
import com.grendelscan.tests.testTypes.ByMimeTypeTest;
import com.grendelscan.utils.FileUtils;
import com.grendelscan.utils.HtmlUtils;
import com.grendelscan.utils.MimeUtils;

public class Module0034 extends TestModule implements ByMimeTypeTest
{
	
	private List<String> allComments;
	private List<ConfigurationOption> configOptions;
	private MultiSelectOptionGroup contentTypeGroup;
	private Map<String, List<String>> htmlComments;
	private SelectableOption htmlTypeOption;
	private Pattern javascriptBlockCommentPattern = Pattern.compile("/\\*(.+?)\\*/", Pattern.MULTILINE);
	private Map<String, List<String>> javascriptComments;
	private Pattern javascriptLineCommentPattern = Pattern.compile("//(.+)");
	
	private SelectableOption javascriptTypeOption;
	
	private FileNameOption outputFileOption;
	
	public Module0034()
	{
		configOptions = new ArrayList<ConfigurationOption>();
		
		contentTypeGroup = new MultiSelectOptionGroup("Content types", "Content types to search for comments.");
		configOptions.add(contentTypeGroup);
		
		htmlTypeOption = new SelectableOption("HTML", true, "Search HTML content for comments.");
		contentTypeGroup.addOption(htmlTypeOption);
		
		javascriptTypeOption = new SelectableOption("JavaScript", true, "Search JavaScript content for comments.");
		contentTypeGroup.addOption(javascriptTypeOption);
		
		outputFileOption =
		        new FileNameOption("Output file name", "comments.txt", "The file to save the comments in.", false);
		configOptions.add(outputFileOption);
	}
	
	@Override
	public List<ConfigurationOption> getConfigurationOptions()
	{
		return configOptions;
	}
	
	@Override
	public String getDescription()
	{
		return "Saves all observed comments into the specified file. Only unique comments are saved; if a comment "
		        + "is seen more than once, only the first instance will be reported. For the sake of performance, JavaScript "
		        + "comment searches aren't very intelligent. For example, something like this will match as a "
		        + "comment:\n" + "    a = \"123/*fake comment*/321\";";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.INFORMATION_LEAKAGE;
	}
	
	public String[] getMimeTypes()
	{
		Set<String> types = new HashSet<String>();
		if (htmlTypeOption.isSelected())
		{
			for (String type: MimeUtils.getHtmlMimeTypes())
			{
				types.add(type);
			}
		}
		
		if (javascriptTypeOption.isSelected())
		{
			for (String type: MimeUtils.getJavaScriptMimeTypes())
			{
				types.add(type);
			}
		}
		
		return types.toArray(new String[0]);
	}
	
	@Override
	public int getModuleNumber()
	{
		return 34;
	}
	
	@Override
	public String getName()
	{
		return "Comment lister";
	}
	
	@Override
	public void initialize()
	{
		if (htmlTypeOption.isSelected())
		{
			htmlComments = new HashMap<String, List<String>>(50);
		}
		
		if (javascriptTypeOption.isSelected())
		{
			javascriptComments = new HashMap<String, List<String>>(50);
		}
		
		allComments = new ArrayList<String>(200);
	}
	
	@Override
	public void scanIsComplete()
	{
		String reportString = "";
		
		if (!(htmlTypeOption.isSelected() || javascriptTypeOption.isSelected()))
		{
			return;
		}
		
		if (htmlTypeOption.isSelected())
		{
			reportString +=
			        "===========================================\n" + "HTML Comments:\n"
			                + "===========================================\n";
			reportString += compileComments(htmlComments);
		}
		
		if (javascriptTypeOption.isSelected())
		{
			reportString +=
			        "===========================================\n" + "JavaScript Comments:\n"
			                + "===========================================\n";
			reportString += compileComments(javascriptComments);
		}
		
		String filename;
		if (outputFileOption.getValue().contains(File.separator))
		{
			filename = outputFileOption.getValue();
		}
		else
		{
			filename = scan.getScanSettings().getOutputDirectory() + File.separator + outputFileOption.getValue();
		}
		if (!FileUtils.writeToFile(filename, reportString))
		{
			reportString = "Writing to the original file name failed\n\n" + reportString;
			FileUtils.writeToFile(scan.getScanSettings().getOutputDirectory() + File.separator + "comments.txt",
			        reportString);
		}
		
		genReport();
	}
	
	public void testByMimeType(int transactionID, String mimeType)
	{
		AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		String a =  transaction.toString();
		if (MimeUtils.isHtmlMimeType(mimeType))
		{
			htmlComments.put(transaction.getAbsoluteUriString(), getHtmlComments(transaction.getResponseDOM()));
			javascriptComments.put(transaction.getAbsoluteUriString(), getHtmlScriptComments(transaction
			        .getResponseDOM()));
		}
		
		if (MimeUtils.isJavaScriptMimeType(mimeType))
		{
			javascriptComments.put(transaction.getAbsoluteUriString(), getJavaScriptComments(transaction
			        .getResponseBodyAsString()));
		}
	}
	
	private String compileComments(Map<String, List<String>> map)
	{
		String reportString = "";
		String urlsArray[] = htmlComments.keySet().toArray(new String[0]);
		Arrays.sort(urlsArray);
		for (String url: urlsArray)
		{
			if (map.get(url).size() > 0)
			{
				reportString += url + "\n";
				for (String comment: map.get(url))
				{
					reportString += "\t" + comment + "\n";
				}
				reportString += "\n";
			}
		}
		return reportString;
	}
	
	private void genReport()
	{
		if (allComments.size() > 0)
		{
			String types = "";
			if (htmlTypeOption.isSelected())
			{
				types = "HTML";
			}
			if (htmlTypeOption.isSelected())
			{
				if (!types.equals(""))
				{
					types += " and ";
				}
				types += "JavaScript";
			}
			String title = "Comment list";
			String shortDesc = "A list of " + types + " comments were generated";
			String longDesc =
			        "A list of " + types + " comments were generated and saved to " + HtmlUtils.makeLink(outputFileOption.getValue(), outputFileOption.getValue()) + 
			        ". In total, " + allComments.size() + " comments were recorded.";
			ReportableEvent event =
			        new ReportableEvent(null, "Module " + getModuleNumber(), EventSeverity.INFO, "", title, shortDesc,
			                longDesc, "", "", "");
			scan.getReportCollector().addEvent(event);
		}
	}
	
	private List<String> getHtmlComments(Node node)
	{
		List<String> comments = new ArrayList<String>();
		for (String comment: HtmlNodeUtilities.getAllComments(node))
		{
			recordComment(comment, comments);
		}
		return comments;
	}
	
	private List<String> getHtmlScriptComments(Node node)
	{
		List<String> comments = new ArrayList<String>();
		for (Node child: HtmlNodeUtilities.getChildElements(node, "SCRIPT"))
		{
			HTMLScriptElement script = (HTMLScriptElement) child;
			for (String comment: getJavaScriptComments(script.getText()))
			{
				recordComment(comment, comments);
			}
		}
		
		return comments;
	}
	
	private List<String> getJavaScriptComments(String javaScript)
	{
		List<String> comments = new ArrayList<String>();
		
		{
			Matcher blockMatcher = javascriptBlockCommentPattern.matcher(javaScript);
			while (blockMatcher.find())
			{
				recordComment(blockMatcher.group(1), comments);
			}
		}
		
		{
			Matcher lineMatcher = javascriptLineCommentPattern.matcher(javaScript);
			while (lineMatcher.find())
			{
				recordComment(lineMatcher.group(1), comments);
			}
		}
		
		return comments;
	}
	
	private synchronized void recordComment(String comment, List<String> comments)
	{
		if (!allComments.contains(comment) && !comments.contains(comment))
		{
			allComments.add(comment);
			comments.add(comment);
		}
	}

	@Override
    public boolean isExperimental()
    {
	    return false;
    }
	
}
