package com.grendelscan.tests.testModules;
import com.grendelscan.scan.reporting.*;
import java.util.*;
import com.grendelscan.utils.*;
import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.*;
import com.grendelscan.tests.testTypes.ByHttpQueryParameterTest;
import com.grendelscan.tests.testModules.settings.*;
import com.grendelscan.tests.libraries.platformErrorMessages.PlatformErrorMessages;
public class Module0035 extends TestModule implements ByHttpQueryParameterTest
{
	private List<ConfigurationOption> configOptions;
	private TextOption plainTextCharsOption;
	private TextOption hexCharsOption;
	private TextOption stringsOption;
	private SelectableOption ignoreTransactionsWithErrorsOption;
	private Set<String> patterns;
	private List<String> findings;
	
	
	public Module0035()
	{
		configOptions = new ArrayList<ConfigurationOption>(1);
		ignoreTransactionsWithErrorsOption = new SelectableOption("Ignore original responses with platform error messages", true, 
				"If enabled, transactions that have platform error messages in the original responses will not be fuzzed.");
		configOptions.add(ignoreTransactionsWithErrorsOption);
		
		plainTextCharsOption = new TextOption("Plain text fuzzing characters", 
				ConfigurationManager.getString("fuzzer.default_plain_text_list"), 
				"Plain text characters that are fuzzed one at a time.", true);
		configOptions.add(plainTextCharsOption);
		
		hexCharsOption = new TextOption("Hex encoded fuzzing characters", 
				ConfigurationManager.getString("fuzzer.default_hex_list"), 
				"Hex encoded characters, separated by spaces.", true);
		configOptions.add(hexCharsOption);
		
		stringsOption = new TextOption("Fuzzing strings", 
				ConfigurationManager.getString("fuzzer.default_string_list"), 
				"Fuzz strings, one on each line.", true);
		configOptions.add(stringsOption);
		
		configOptions.add(PlatformErrorMessages.getRegexErrorPatternsOption());
	}
	
	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }

	@Override
	public String getDescription()
	{
		return "The fuzzing strings defined below are appended onto every query parameter and the " +
				"output is searched for standard platform error messages. To ignore transactions that " +
				"have platform error messages in the original response, enable the first option. Note " +
				"that this option will only be used if the \"Platform error messages\" module under " +
				"\"Information leakage\" is also enabled.\n" +
				"\n" +
				"The first text box contains characters that are individually tested. The second box " +
				"contains the hex values of non-printable characters to test, separated by whitespace. " +
				"The third box contains a fuzzing string on each line. The long line contains 1024 'a' " +
				"characters. The fourth box contains an optional list of regular expressions that " +
				"define platform error messages. The expression is matched against any part of the " +
				"response, so \"error\" will work (you don't " +
				"need \".*error.*\"). The patterns are case-insensitive and will be tested against both the raw response body, and the " +
				"combined text elements of the HTML. This option is useful if you know that there are unusual " +
				"messages in the targeted application. This setting is shared with the \"Platform error " +
				"messages\" module.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.MISCELLANEOUS_ATTACKS;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 35;
	}
	
	@Override
	public String getName()
	{
		return "Generic fuzzing";
	}
	
	@Override
	public void initialize()
	{
		patterns = new HashSet<String>();
		findings = new ArrayList<String>();
		
		for (String pattern: stringsOption.getValue().split("[\r\n]+"))
		{
			patterns.add(pattern);
		}
		
		final String value = plainTextCharsOption.getValue();
		for (int index = 0; index < value.length(); index++)
        {
			patterns.add(String.valueOf(value.charAt(index)));
        }

		for (String pattern: hexCharsOption.getValue().split("\\s+"))
        {
			try
			{
				patterns.add(String.valueOf(Character.toChars(Integer.parseInt(pattern, 16))[0]));
			}
			catch (Exception e) 
			{
				Debug.errDebug("Bad hex pattern (" + pattern + "): " + e.toString(), e);
			}
        }
	}

	
	@Override
    public int[] getSoftPrerequisites()
    {
	    return new int[] {36};
    }

	public void testByQueryParameter(int transactionID, String parameterName)
	{
		if (ignoreTransactionsWithErrorsOption.isSelected() && PlatformErrorMessages.getMessageHits().containsKey(transactionID))
		{
			return;
		}
		
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	String originalValue = transaction.getQueryParameterValue(parameterName);
    	
    	for (String fuzzString: patterns)
    	{
    		if (!scan.isRunning())
    		{
    			break;
    		}
    		AbstractHttpTransaction testTransaction = transaction.clone(TransactionSource.TEST);
    		testTransaction.replaceQueryParameter(parameterName, originalValue + fuzzString);
    		long one = new Date().getTime();
    		testTransaction.execute("Module " + getModuleNumber());
    		long two = new Date().getTime();
    		String matchPattern = PlatformErrorMessages.isErrorMatch(testTransaction, false);
    		long three = new Date().getTime();
//    		System.err.println("Module35   --   execute: " + String.valueOf(two-one) + "    parse: " + String.valueOf(three-two) 
//    				+ "\n\n" + testTransaction.toString() + "\n\n");
    		if (matchPattern.equals("") && testTransaction.getLogicalResponseCode() >= 500)
    		{
    			matchPattern = "500 response code";
    		}
    		
    		if (!matchPattern.equals(""))
    		{
    			recordFinding(testTransaction, parameterName, originalValue, fuzzString, matchPattern);
    		}
    	}
	}

	private void recordFinding(AbstractHttpTransaction testTransaction, String parameterName, String originalValue, String fuzzString, String matchPattern)
	{
		testTransaction.writeToDisk();
		String goodFuzzString = "";
		if (fuzzString.matches(".*[\\x00-\\x1f\\x7f-\\xff].*") || fuzzString.equals(" "))
		{
			for (int i = 0; i < fuzzString.length(); i++)
            {
				goodFuzzString += String.format("0x%02X ", Integer.valueOf(fuzzString.charAt(i)));
            }
		}
		else
		{
			goodFuzzString = fuzzString;
		}

		String finding = "<tr>" +
				"<td class=\"fuzzheader\">Test transaction:</td>" +
				"<td>" + HtmlUtils.makeLink(testTransaction.getSavedUrl(), testTransaction.getHostUri()) + "</td>" +
				"</tr><tr>" +
				"<td class=\"fuzzheader\">Parameter name:</td>" +
				"<td>" + parameterName + "</td>" +
				"</tr><tr>" +
				"<td class=\"fuzzheader\">Fuzz string:</td>" +
				"<td>" + goodFuzzString + "</td>" +
				"</tr><tr>" +
				"<td class=\"fuzzheader\">Original value:</td>" +
				"<td>" + originalValue + "</td>" +
				"</tr><tr>" +
				"<td class=\"fuzzheader\">Match pattern:</td>" +
				"<td>" + HtmlUtils.escapeHTML(matchPattern) + "</td>" +
				"</tr><tr><td>&nbsp</td></tr>\n";
		findings.add(finding);
	}

	@Override
    public void scanIsComplete()
    {
		if (findings.size() > 0)
		{
			String title = "Successful fuzzing tests";
			String shortDesc = "Some generic fuzzing tests appear to be successful";
			String longDesc = "Some generic fuzzing tests appear to have been successful at generating an " +
					"infrastructure error. The results are listed in the table below:<br><br>" +
					"<style>.fuzzheader{font-weight: bold;vertical-align: top;text-align: left;}</style>\n<table>";
			for(String finding: findings)
			{
				longDesc += finding;
			}
			longDesc += "</table>\n";
			
			String impact = "The impact will vary depending on the nature of the error.";
			String recomendations = "Each result should be individually investigated. The error message " +
					"may provide a hint as to how the user-supplied data is being used.";
			ReportableEvent event = new ReportableEvent(null, this.getName(), EventSeverity.INFO, 
					"Multiple", title, shortDesc, longDesc, impact, recomendations, "");
			scan.getReportCollector().addEvent(event);
		}
    }

	@Override
    public boolean isExperimental()
    {
	    return true;
    }

	@Override
    public String getExperimentalText()
    {
	    return 
	    	"The generic fuzzing module has great potential for discovering improper " +
	    	"input validation. However, some performance problems have been observed " +
	    	"with it. Specifically, test transactions generated by the fuzzer may " +
	    	"become increasingly slow to process as the scan progresses. This may be " +
	    	"alieviated by reducing the number of strings and characters used to " +
	    	"generate fuzzing requests. It may also be advisable to run a scan where " +
	    	"the fuzzing test module is the only test module enabled, other than the " +
	    	"spider.";
    }


}
