package com.grendelscan.tests.testModules;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.scan.reporting.*;
import com.grendelscan.tests.testTypes.*;
import com.grendelscan.utils.*;
import com.grendelscan.tests.testModules.settings.*;

import java.util.*;

public class Module0037 extends TestModule implements ByHttpMethodTest
{
	private Map<String, int[]> unenforcedUris;
	private Set<String> testedUris;
	private List<ConfigurationOption> configOptions;
	private SelectableOption postTransactionOption;
	private SelectableOption getTransactionOption;
	
	public Module0037()
	{
		configOptions = new ArrayList<ConfigurationOption>();
		postTransactionOption = new SelectableOption("POST", false, "Check to test POST transactions. This can be dangerous."); 
		configOptions.add(postTransactionOption);
		getTransactionOption = new SelectableOption("GET", true, "Check to test POST transactions."); 
		configOptions.add(getTransactionOption);
	}
	
	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
	    return configOptions;
    }
	
	public void testByHttpMethod(int transactionID)
	{
    	AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
    	
    	if (!HttpUtils.fileExists(transaction.getLogicalResponseCode()))
    	{
    		return;
    	}
    	
    	String transactionFingerprint = transaction.getUriString() + transaction.getPayloadQueryString();
    	if(!testedUris.contains(transactionFingerprint))
    	{
    		testedUris.add(transactionFingerprint);
    		if (transaction.isAuthenticated())
    		{
    			testAuthenticationEnforcement(transaction, transactionFingerprint);
    		}
    		else
    		{
    			unenforcedUris.remove(transactionFingerprint);
    		}
    	}
	}
	
	private void testAuthenticationEnforcement(AbstractHttpTransaction transaction, String transactionFingerprint)
	{
		AbstractHttpTransaction testTransaction = transaction.clone(TransactionSource.TEST);
		testTransaction.removeAllAuthentication();
		testTransaction.execute("Module " + getModuleNumber());
		if (HttpTransactionUtils.scoreResponseMatch(transaction, testTransaction, 100) < 95)
		{
			unenforcedUris.put(transactionFingerprint, new int[] {transaction.getId(), testTransaction.getId()});
		}
	}


	@Override
    public void scanIsComplete()
    {
		if (unenforcedUris.size() > 0)
		{
			report();
		}
    }

	@Override
	public String getDescription()
	{
		return "This module will report pages / queries that appear " +
				"to be vulnerable to authentication bypass. Note that " +
				"testing POSTs can be dangerous because the " +
				"transaction has to be repeated to test the result.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SESSION_MANAGMENT;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 37;
	}
	
	@Override
	public String getName()
	{
		return "Authentication enforcement";
	}
	
	@Override
	public void initialize()
	{
		unenforcedUris = new HashMap<String, int[]>(50);
		testedUris = new HashSet<String>(50);
	}

	public String[] getHttpMethods()
    {
		Set<String> methods = new HashSet<String>();
		if (postTransactionOption.isSelected())
		{
			methods.add("POST");
		}
		if (getTransactionOption.isSelected())
		{
			methods.add("GET");
		}
	    return methods.toArray(new String[0]);
    }
	
	private void report()
	{
		String title = "Authentication bypass detected";
		String shortDesc = "An authentication bypass vulnerability may have been detected";
		String longDesc = "An authentication bypass vulnerability may have been detected. Some authenticated " +
				"requests were successfully executed without providing a session ID. The table below contains " +
				"the relevant transactions:\n" +
				"<table>" +
				"<tr style=\"font-weight: bold\"><td>Original Transaction</td><td>Unauthenticated transaction</td><tr>\n";
		
		String firstUrl = "";
		for (int[] transactions: unenforcedUris.values())
		{
			AbstractHttpTransaction originalTransaction = scan.getTransactionRecord().getTransaction(transactions[0]);
			AbstractHttpTransaction testTransaction = scan.getTransactionRecord().getTransaction(transactions[1]);
			if (firstUrl.equals(""))
			{
				firstUrl = originalTransaction.getBaseUri();
			}
			originalTransaction.writeToDisk();
			testTransaction.writeToDisk();
			longDesc += "<tr><td>" + HtmlUtils.makeLink(originalTransaction.getSavedUrl(), String.valueOf(originalTransaction.getId())) + "<td>";
			longDesc += "<td>" + HtmlUtils.makeLink(testTransaction.getSavedUrl(), String.valueOf(testTransaction.getId())) + "<td></tr>\n";
		}
		longDesc += "</table>\n";
		
		if (unenforcedUris.size() > 1)
		{
			firstUrl += " and others";
		}
		String recomendations = "These results could be false positives if the content is intended to be accessible " +
				"by unauthenticated users. A manual review should be performed to determine the accuracy.";
		String impact = "If the data presented to authenticated users is sensitive, it could be disclosed to an attacker.";
		String references = HtmlUtils.makeLink("http://www.owasp.org/index.php/Top_10_2007-A7");
		ReportableEvent event = new ReportableEvent(null, getName(), EventSeverity.HIGH, firstUrl, title, shortDesc, longDesc, 
	        		impact, recomendations, references);
		scan.getReportCollector().addEvent(event);
	}

	@Override
    public boolean isExperimental()
    {
	    return true;
    }

	@Override
    public String getExperimentalText()
    {
	    return "This module requires more testing to assure its accuracy.";
    }
	
	
}
