package com.grendelscan.tests.testModules;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.*;
import java.util.regex.*;

import org.apache.http.HttpException;
import org.apache.http.client.HttpClient;

import com.grendelscan.requester.TransactionSource;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;
import com.grendelscan.requester.http.transactions.NonScanHttpTransaction;
import com.grendelscan.tests.testModules.settings.*;
import com.grendelscan.tests.testTypes.ByHostTest;
import com.grendelscan.utils.Debug;
import com.grendelscan.requester.http.factories.*;

public class Module0038 extends TestModule implements ByHostTest
{
	private List<ConfigurationOption> configOptions;
	private SelectableOption liveSearchOption;
	private SelectableOption googleSearchOption;
	private SelectableOption yahooSearchOption;
	private IntegerOption maxResultsOption;
	private IntegerOption delayOption;
	private int maxResults;
	private int minPause;
	
	@Override
    public List<ConfigurationOption> getConfigurationOptions()
    {
		return configOptions;
    }

	public Module0038()
	{
		MultiSelectOptionGroup engines = new MultiSelectOptionGroup("Search engines", "Select which search engines to use.");
		liveSearchOption = new SelectableOption("Live.com", true, "Search live.com");
		googleSearchOption = new SelectableOption("Google.com", false, "Search google.com");
		yahooSearchOption = new SelectableOption("Yahoo.com", false, "Search yahoo.com");
		engines.addOption(liveSearchOption);
		engines.addOption(googleSearchOption);
		engines.addOption(yahooSearchOption);
		maxResultsOption = new IntegerOption("Max results", 500, "Stop searching after this number of URLs has been discovered from the search engines");
		delayOption = new IntegerOption("Request delay (ms)", 1000, "The minimum number of milliseconds between search engine requests. Note that each request to the search engine will result in many URLs from the target website.");
		
		configOptions = new ArrayList<ConfigurationOption>(4);
		configOptions.add(engines);
		configOptions.add(maxResultsOption);
		configOptions.add(delayOption);
	}
	
	@Override
	public String getDescription()
	{
		return "Using the \"site:\" query command with the targeted websites, this module will " +
				"attempt to discover content on the selected search engines. Since search " +
				"engines will index pages that other sites link to, this can technique can " +
				"reveal content that standard spidering won't discover. Obviously this is only " +
				"useful if the sites are present on the Internet.\n" +
				"\n" +
				"Be careful in decreasing the delay between search engine requests. Submitting " +
				"too many requests can result in your IP address being temporarily blocked by " +
				"the search engine.\n" +
				"\n" +
				"Live.com is the default, because they seem to have a larger database than " +
				"the other two. Also note that Yahoo.com will only return the first 1000 " +
				"records, although they can be fetched in plain text format with a single " +
				"request.";
	}
	
	@Override
	public TestModuleGUIPath getGUIDisplayPath()
	{
		return TestModuleGUIPath.SPIDER;
	}
	
	@Override
	public int getModuleNumber()
	{
		return 38;
	}
	
	@Override
	public String getName()
	{
		return "Search engine recon";
	}
	
	@Override
	public void initialize()
	{
		maxResults = maxResultsOption.getValue();
		minPause = delayOption.getValue();
	}
	
	
	private int search(SearchEngine searchEngine, String host, HttpClient client, int currentCount)
	{
		int startLocation = 1;
		long lastRequest = 0;
		while (scan.isRunning() && currentCount++ <= maxResults)
		{
			String query = searchEngine.getQueryURI(host, startLocation);
			try
            {
	            NonScanHttpTransaction queryTransaction = new NonScanHttpTransaction(scan, "GET", query);
	            searchEngine.updateTransaction(queryTransaction);
				long pause = lastRequest - new Date().getTime() + minPause;
				if (pause > 0)
				{
					Debug.debug(this.getName() + " (" + this.getModuleNumber() + ") is pausing for " + pause + " ms");
					synchronized(this)
					{
						try
	                    {
		                    this.wait(pause);
	                    }
	                    catch (InterruptedException e)
	                    {
	                    }
					}
				}
				if (!scan.isRunning())
				{
					break;
				}
	            queryTransaction.execute(client);
	            lastRequest= new Date().getTime();
	            String body = new String(queryTransaction.getResponseBody());
	            Matcher m = searchEngine.getFindResultPattern().matcher(body);
	            while(scan.isRunning() && m.find() && currentCount <= maxResults)
	            {
	            	String uri = m.group(1);
	            	AbstractHttpTransaction spiderTransaction = HttpTransactionFactory.createTransaction(scan, "GET", uri, 0, TransactionSource.SPIDER);
	            	if (scan.getRequesterQueue().isScopeAndDepthGood(spiderTransaction))
	            	{
	            		currentCount++;
	            		scan.getRequesterQueue().addSpiderRequest(spiderTransaction, false, this.getName() + "(" + this.getModuleNumber() + ")");
	            	}
	            }
	            if (searchEngine.isMoreResults(body))
	            {
	            	startLocation += searchEngine.getQueryIncrement();
	            	continue;
	            }
            }
            catch (URISyntaxException e)
            {
	           Debug.errDebug("Problem with URL format with search engine recon: " + e.toString(), e);
            }
            catch (HttpException e)
            {
 	           Debug.errDebug("Problem with request for search engine recon: " + e.toString(), e);
            }
            catch (IOException e)
            {
  	           Debug.errDebug("Problem sending request for search engine recon: " + e.toString(), e);
            }
            break;
		}
		return currentCount;
	}
	
	
	
	@Override
	public boolean isExperimental()
	{
		return false;
	}
	
	public void testByServer(int transactionID)
	{
		AbstractHttpTransaction transaction = scan.getTransactionRecord().getTransaction(transactionID);
		String host = transaction.getHost();
		
		HttpClient client = NonScanClientFactory.getClient(scan);
		int totalRequests = 0;
		if (googleSearchOption.isSelected() && scan.isRunning())
		{
			totalRequests = search(new Google(), host, client, totalRequests);
		}

		if (liveSearchOption.isSelected() && scan.isRunning())
		{
			totalRequests = search(new Live(), host, client, totalRequests);
		}
		
		if (yahooSearchOption.isSelected() && scan.isRunning())
		{
			totalRequests = search(new Yahoo(), host, client, totalRequests);
		}
	}
	
	private class Google implements SearchEngine
	{
		private Pattern googleFindResultPattern = Pattern.compile("<div class=g><a href=\"([^\"]+)", Pattern.CASE_INSENSITIVE);
		private Pattern googleMoreResultsPattern = Pattern.compile("<span>Next</span>", Pattern.CASE_INSENSITIVE);
		public Pattern getFindResultPattern()
        {
	        return googleFindResultPattern;
        }

		public boolean isMoreResults(String body)
        {
	        return googleMoreResultsPattern.matcher(body).find();
        }

		public int getQueryIncrement()
        {
	        return 100;
        }

		public String getQueryURI(String host, int startLocation)
        {
	        return "http://www.google.com/search?filter=0&num=100&as_sitesearch=" + host + "&start=" + startLocation;
        }

		// Not needed for Google
		public void updateTransaction(NonScanHttpTransaction queryTransaction)
        {
        }
		
	}
	
	private class Live implements SearchEngine
	{
		private Pattern liveFindResultPattern = Pattern.compile("<li><h3><a href=\"([^\"]+)", Pattern.CASE_INSENSITIVE);
		private Pattern liveMoreResultsPattern = Pattern.compile("FORM=PORE\">Next</a></li></ul>", Pattern.CASE_INSENSITIVE);
		public Pattern getFindResultPattern()
        {
	        return liveFindResultPattern;
        }

		public boolean isMoreResults(String body)
        {
	        return liveMoreResultsPattern.matcher(body).find();
        }

		public int getQueryIncrement()
        {
	        return 200;
        }

		public String getQueryURI(String host, int startLocation)
        {
	        return "http://search.live.com/results.aspx?q=site%3a" + host + "&first=" + startLocation;
        }

		public void updateTransaction(NonScanHttpTransaction queryTransaction)
        {
			queryTransaction.addRequestHeader("Cookie", "SRCHHPGUSR=NRSLT=200&NRSPH=0");
        }
		
	}

	
	private class Yahoo implements SearchEngine
	{
		private Pattern yahooFindResultPattern = Pattern.compile("\r?\n[^\t]*\t([^\t]+)");
		public Pattern getFindResultPattern()
        {
	        return yahooFindResultPattern;
        }

		public int getQueryIncrement()
        {
	        return 1000;
        }

		public String getQueryURI(String host, int startLocation)
        {
	        return "https://siteexplorer.search.yahoo.com/advtsv?p=http%3A%2F%2F" + host;
        }

		public void updateTransaction(NonScanHttpTransaction queryTransaction)
        {
        }

		// Yahoo will only return 1000 results
		public boolean isMoreResults(String body)
        {
	        return false;
        }
		
	}

	private interface SearchEngine
	{
		public String getQueryURI(String host, int startLocation);
		public int getQueryIncrement();
		public Pattern getFindResultPattern();
		public boolean isMoreResults(String body);
		public void updateTransaction(NonScanHttpTransaction queryTransaction);
	}
	
}
