package com.grendelscan.tests.testModules.collections;


import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import com.grendelscan.scan.ConfigurationManager;
import com.grendelscan.scan.Scan;
import com.grendelscan.tests.testModules.TestModule;
import com.grendelscan.utils.Debug;

/**
 * @author David Byrne
 *
 */
public class MasterTestModuleCollection
{
	private Set<TestModule> enabledModules = new HashSet<TestModule>();
	private Map<Integer, TestModule> allModules = new HashMap<Integer, TestModule>();
	private Scan scan;
	
	/** 
	 * This will flush unused test modules and enable all prerequsites. Prereqs are not
	 * enabled when the module is added because it would be difficult to remove them
	 * if the module was removed.
	 */
	public void finishModules()
	{
		enablePrerequisites();
		flushUnusedModules();
	}
	
	private void enablePrerequisites()
	{
		Set<TestModule> tempEnabledModules = new HashSet<TestModule>(enabledModules);
		for (TestModule module: tempEnabledModules)
		{
			enablePrerequsites(module);
		}
	}
	
	private void enablePrerequsites(TestModule module)
	{
		for (int prereq: module.getPrerequisites())
		{
			enablePrerequsites(getTestModule(prereq));
			enableTestModule(prereq);
		}
	}
	
	private void flushUnusedModules()
	{
		Set <TestModule>tempModules = new HashSet<TestModule>();
		tempModules.addAll(allModules.values());
		for(TestModule nextModule: tempModules)
		{
			if (enabledModules.contains(nextModule) || nextModule.alwaysEnabled())
			{
				continue;
			}
			allModules.remove(nextModule.getModuleNumber());
		}
	}
	
	public void initializeModules()
	{
		for(TestModule module: allModules.values())
		{
			module.initialize();
		}
	}
	
	public TestModule getTestModule(int moduleNumber)
	{
		if (! allModules.containsKey(moduleNumber))
		{
			Debug.debug("Module " + moduleNumber + " doesn't seem to exist. This could be okay if it's a soft prerequisite. (MasterTestModuleCollection.getTestModule())");
		}
		return allModules.get(moduleNumber);
	}
	
	
	public Collection<TestModule> getAllTestModules()
	{
		return allModules.values();
	}

	
	public boolean enableTestModule(int moduleNumber)
	{
		TestModule module;
		
		// Check to see if there is such a module
		if ((module = allModules.get(moduleNumber)) == null)
		{
			return false;
		}
		return enabledModules.add(module);
	}

	public boolean isModuleEnabled(TestModule module)
	{
		return enabledModules.contains(module); 
	}
	
	public boolean disableTestModule(int moduleNumber)
	{
		TestModule module;
		if ((module = allModules.get(moduleNumber)) == null)
		{
			return false;
		}
		return enabledModules.remove(module);
	}
	
	private void instantiateTestingModules() throws FileNotFoundException
	{
		Class nextModuleClass;
		String allModuleNames[];
		ClassLoader classLoader = ClassLoader.getSystemClassLoader();
		

		allModuleNames = ConfigurationManager.getStringArray("module_list");
		
		for(String moduleName: allModuleNames)
		{
			try
			{
				nextModuleClass = classLoader.loadClass("com.grendelscan.tests.testModules." + moduleName);
				TestModule module = (TestModule) nextModuleClass.newInstance();
				module.setScan(scan);
				allModules.put(module.getModuleNumber(), module);
			}
			catch (ClassNotFoundException e)
			{
				Debug.errDebug("Some sort of problem loading testing module " + moduleName + ": " + e.toString(), e);
				continue;
			}
			catch (IllegalAccessException e)
			{
				Debug.errDebug("Some sort of problem loading testing module " + moduleName + ": " + e.toString(), e);
				continue;
			}
			catch (InstantiationException e)
			{
				Debug.errDebug("Some sort of problem loading testing module " + moduleName + ": " + e.toString(), e);
				continue;
			}

		}
	}
	
	
	public MasterTestModuleCollection(Scan scan) throws FileNotFoundException
	{
		this.scan = scan;
		instantiateTestingModules();
	}
	
}

