package com.grendelscan.utils;
import org.apache.http.Header;
import org.w3c.dom.Document;

import java.util.*;

import com.grendelscan.requester.http.transactions.*;
public class HttpTransactionUtils
{
	public static int scoreResponseMatch(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB, int maxScore)
	{
		HttpTransactionMatchCriteriaWeights weights = new HttpTransactionMatchCriteriaWeights();
		double dScore = scoreResponseMatch(transactionA, transactionB, weights) / weights.getMaxScore();
		return (int) Math.round(dScore * maxScore);
	}
	
	public static int scoreResponseMatch(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB, HttpTransactionMatchCriteriaWeights weights, int maxScore)
	{
		double dScore = scoreResponseMatch(transactionA, transactionB, weights) / weights.getMaxScore();
		return (int) Math.round(dScore * maxScore);
	}
	
	public static double scoreResponseMatch(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB, 
			HttpTransactionMatchCriteriaWeights weights)
	{
		double score = 0;
		if (!transactionA.isSuccessfullExecution() || !transactionB.isSuccessfullExecution())
		{
			return 0;
		}
		
		score += getResponseCodeScore(transactionA, transactionB) * weights.getResponseCode();
		score += getLocationHeaderScore(transactionA, transactionB) * weights.getLocationHeader();
		score += getMimeTypeScore(transactionA, transactionB) * weights.getMimeType();
		score += getSetCookieScore(transactionA, transactionB) * weights.getSetCookieName();
		score += getTextNodesScore(transactionA, transactionB) * weights.getTextNodes();
		score += getTagCountsScore(transactionA, transactionB, weights);
		return score;
	}
	
	private static double getTagCountsScore(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB, HttpTransactionMatchCriteriaWeights weights)
	{
		double score = 0;
		for (String tagName: weights.getTagCounts().keySet())	
		{
			score += getTagCountScore(tagName, transactionA, transactionB, weights);
		}
		return score;
	}
	
	private static double getTagCountScore(String tagName, AbstractHttpTransaction transactionA, 
			AbstractHttpTransaction transactionB, HttpTransactionMatchCriteriaWeights weights)
	{
		Document domA = transactionA.getResponseDOM();
		Document domB = transactionA.getResponseDOM();
		
		if (domA == null || domB == null)
		{
			if (domA != domB)
			{
				return 0;
			}
			else
			{
				return 1;
			}
		}
		int countA = domA.getElementsByTagName(tagName).getLength();
		int countB = domB.getElementsByTagName(tagName).getLength();
		
		int max = Math.max(countA, countB);
		int min = Math.min(countA, countB);
		
		
		double score;
		if (max == 0)
		{
			score = 1;
		}
		else 
		{
			score = min / max;
		}
		return score * weights.getTagCounts().get(tagName);
	}
	
	
	private static double getTextNodesScore(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB)
	{
		String textA = normalizeResponseTextContent(transactionA);
		String textB = normalizeResponseTextContent(transactionB);
		
		int perfectScore = 10000; 
		double diff = (perfectScore - StringUtils.scoreStringDifference(textA, textB, perfectScore)) / perfectScore;
		return 0 - diff;
	}

	private static String normalizeResponseTextContent(AbstractHttpTransaction transaction)
	{
		String uri = transaction.getAbsoluteUriString();
		String newResponse = transaction.getResponseTextNodes();
		newResponse = newResponse.toUpperCase();
		newResponse = newResponse.replaceAll("\\s++", " ");
		newResponse = newResponse.replace(URIStringUtils.getPath(uri).toUpperCase(), "");
		newResponse = newResponse.replace(URIStringUtils.getFilename(uri).toUpperCase(), "");
		newResponse = newResponse.replace("?" + URIStringUtils.getQuery(uri).toUpperCase(), "");
		
		return newResponse;
	}

	
	private static double getSetCookieScore(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB)
	{
		Set<String> cookieNamesA = getSetCookieNames(transactionA);
		Set<String> cookieNamesB = getSetCookieNames(transactionB);
		
		
		if (cookieNamesA.size() == 0 && cookieNamesB.size() == 0)
		{
			return 0;
		}
		double score = 0;
		double increment;
		if ((cookieNamesA.size() >= cookieNamesB.size()))
		{
			increment = 1 / cookieNamesA.size() / 2;
		}
		else 
		{
			increment = 1 / cookieNamesB.size() / 2;
		}
		for (String name: cookieNamesA)
		{
			if(!cookieNamesB.contains(name))
			{
				score -= increment;
			}
		}
		for (String name: cookieNamesB)
		{
			if(!cookieNamesA.contains(name))
			{
				score -= increment;
			}
		}
		return score;
	}
	
	private static Set<String> getSetCookieNames(AbstractHttpTransaction transaction)
	{
		Header[] cookieHeaders = transaction.getResponseHeaders("Set-Cookie");
		Set<String> nameSet = new HashSet<String>(cookieHeaders.length);
		for (Header header: cookieHeaders)
		{
			nameSet.add(header.getValue());
		}
		return nameSet;
	}
	
	private static double getMimeTypeScore(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB)
	{
		double score = -1;
		if (transactionA.getResponseMimeType().equalsIgnoreCase(transactionB.getResponseMimeType()))
		{
			score = 1;
		}
		return score;
	}

	
	private static double getResponseCodeScore(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB)
	{
		double score = -1;
		if (transactionA.getResponseStatusCode() == transactionB.getResponseStatusCode())
		{
			score = 1;
		}
		return score;
	}


	private static double getLocationHeaderScore(AbstractHttpTransaction transactionA, AbstractHttpTransaction transactionB)
	{
		Header[] locationsA = transactionA.getResponseHeaders("Location");
		Header[] locationsB = transactionB.getResponseHeaders("Location");
		
		if (locationsA.length == 0 && locationsB.length == 0)
		{
			return 0;
		}
		if ((locationsA.length != locationsB.length))
		{
			return -1;
		}
		int perfectScore = 10000; 
		// perfectScore*2 because a complete mismatch should be much more damaging than a slight mismatch
		double diff = (perfectScore - StringUtils.scoreStringDifference(locationsA[0].getName(), locationsB[0].getName(), perfectScore)) / (perfectScore * 2);
		
		return 0 - diff;
	}
}
