package com.grendelscan.utils;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashSet;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.Header;
import org.apache.http.HttpEntity;
import org.apache.http.cookie.Cookie;
import org.apache.http.cookie.MalformedCookieException;
import org.apache.http.util.ByteArrayBuffer;

import com.grendelscan.GUI.http.HttpFormatException;
import com.grendelscan.requester.http.UnvalidatedHttpEntityRequest;
import com.grendelscan.requester.http.UnvalidatedHttpRequest;
import com.grendelscan.requester.http.serializable.SerializableHttpHeader;
import com.grendelscan.requester.http.transactions.AbstractHttpTransaction;

public class HttpUtils
{
	/**
	 * 
	 * @param entity
	 * @param maxSize Set to zero for no limit
	 * @return
	 * @throws IOException
	 */
	public static byte[] entityToByteArray(final HttpEntity entity, int maxSize) throws IOException
	{
		if (maxSize <= 0)
		{
			maxSize = Integer.MAX_VALUE;
		}
		if (entity == null)
		{
			throw new IllegalArgumentException("HTTP entity may not be null");
		}
		InputStream instream = entity.getContent();
		if (instream == null)
		{
			return new byte[] {};
		}
		int i = (int) entity.getContentLength();
		if (entity.getContentLength() > maxSize)
		{
			i = maxSize;
		}
		else if (i < 0)
		{
			i = 4096;
		}
		ByteArrayBuffer buffer = new ByteArrayBuffer(i);
		try
		{
			byte[] tmp = new byte[4096];
			int l;
			while ((l = instream.read(tmp)) != -1 && buffer.length() <= maxSize)
			{
				buffer.append(tmp, 0, l);
			}
		}
		finally
		{
			instream.close();
		}
		return buffer.toByteArray();
	}
	
	/**
	 * It might seem strange to say that a 302 (or other redirect) isn't a file
	 * found. Keep in mind two things. First, many file not found messages are
	 * in redirects. Second, the redirect's destination will be tested as a file
	 * found (if it is found).
	 * 
	 * @param httpStatusCode
	 * @return
	 */
	public static boolean fileExists(int httpStatusCode)
	{
		boolean exists = false;
		if (((httpStatusCode >= 100) && (httpStatusCode < 300)) || (httpStatusCode == 304)
		        || ((httpStatusCode >= 401) && (httpStatusCode <= 403)))
		{
			exists = true;
		}
		return exists;
	}
	
	
	public static Set<Cookie> getSetCookies(AbstractHttpTransaction transaction)
	{
		Set<Cookie> cookies = new HashSet<Cookie>();
		for (Header setCookie: transaction.getResponseHeaders("Set-Cookie"))
		{
			try
			{
				for (Cookie cookie: transaction.getCookieSpec().parse(setCookie, transaction.getCookieOrigin()))
				{
					cookies.add(cookie);
				}
			}
			catch (MalformedCookieException e)
			{
				Debug.errDebug("Problem with set-cookie header in HttpUtils.getSetCookies: " + e.toString(), e);
			}
		}
		return cookies;
	}
	
	public static Cookie getSetCookie(AbstractHttpTransaction transaction, String cookieName)
	{
		for (Header setCookie: transaction.getResponseHeaders("Set-Cookie"))
		{
			try
			{
				for (Cookie cookie: transaction.getCookieSpec().parse(setCookie, transaction.getCookieOrigin()))
				{
					if (cookie.getName().equalsIgnoreCase(cookieName))
					{
						return cookie;
					}
				}
			}
			catch (MalformedCookieException e)
			{
				Debug.errDebug("Problem with set-cookie header in HttpUtils.getSetCookies: " + e.toString(), e);
			}
		}
		return null;
	}

	static private Pattern httpVersionPattern = Pattern.compile("^(\\w+)/(\\d+)\\.(\\d+)$");
	static private Pattern headerPattern = Pattern.compile("([^ \r\n\t]+): ([^\r\n]+)");
	public static UnvalidatedHttpRequest parseIntoHttpRequest(String rawRequest) throws HttpFormatException
	{
		String method = "";
		String uri = "";
		String protocolVersion = "";
		String bodyString = "";
		Pattern requestPattern = Pattern.compile("^([^ \t\r\n]+) ([^ \t\r\n]+) ([^ \t\r\n]+)\r?\n((?:[^ \r\n\t]+: [^\r\n]+\r?\n)*+)\r?\n(.*)$", Pattern.DOTALL);
		Matcher requestMatcher = requestPattern.matcher(rawRequest);
		if (requestMatcher.matches())
		{
			method = requestMatcher.group(1);
			uri = requestMatcher.group(2);
			protocolVersion = requestMatcher.group(3);
			String rawHeaders = requestMatcher.group(4);
			bodyString = requestMatcher.group(5);
			if (bodyString == null)
			{
				bodyString = "";
			}
			if (rawHeaders == null)
			{
				rawHeaders = "";
			}
	
			int major, minor;
			String protocol = "";
			Matcher versionMatcher = httpVersionPattern.matcher(protocolVersion);
			if (versionMatcher.find())
			{
				protocol = versionMatcher.group(1);
				major = Integer.valueOf(versionMatcher.group(2));
				minor = Integer.valueOf(versionMatcher.group(3));
			}
			else
			{
				throw new HttpFormatException("Invalid protocol version format. It should look something like \"HTTP/1.0\".");
			}


			String host = URIStringUtils.getHost(uri);
			Matcher headerMatcher = headerPattern.matcher(rawHeaders);
			List<SerializableHttpHeader> headers = new ArrayList<SerializableHttpHeader>();
			while(headerMatcher.find())
			{
				String name = URIStringUtils.decodeUrl(headerMatcher.group(1));
				String value = URIStringUtils.decodeUrl(headerMatcher.group(2));
				if (name.equalsIgnoreCase("Host"))
				{
					host = value;
				}
				headers.add(new SerializableHttpHeader(name, value));
			}

			int port = URIStringUtils.getPort(uri);
			boolean ssl = URIStringUtils.getScheme(uri).equalsIgnoreCase("https") ? true : false;
			byte[] body = bodyString.getBytes();
			UnvalidatedHttpRequest unvalidatedRequest;
			if (body.length > 0)
			{
				unvalidatedRequest =
			        new UnvalidatedHttpEntityRequest(method, uri, protocol, major, minor, host, port, ssl, body);
			}
			else
			{
				unvalidatedRequest =
			        new UnvalidatedHttpRequest(method, uri, protocol, major, minor, host, port, ssl);
			}
			
			unvalidatedRequest.addHeaders(headers.toArray(new Header[0]));
			
			return unvalidatedRequest;
		}
		throw new HttpFormatException("Unparsable request format.");
	}
	
}
