package com.grendelscan.utils;

import java.lang.*;
import java.io.ByteArrayOutputStream;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.codec.DecoderException;
import org.apache.commons.codec.net.URLCodec;
import org.apache.http.NameValuePair;
import org.apache.http.message.BasicNameValuePair;

import com.grendelscan.requester.http.factories.UriFactory;

public class URIStringUtils
{
	private static Pattern badNonQueryPattern = Pattern.compile("([&=])");
	private static final Pattern badUriCharPattern =
	        Pattern.compile("((?:[\\x00-\\x20\\x7f-\\xff\\{\\}\\\\\"'`^#|\\[\\]<>\\(\\)])|(?:%(?![0-9a-f]{2})))",
	        		Pattern.CASE_INSENSITIVE | Pattern.DOTALL);
	/**
	 * This is different from badUriCharPattern, in that it doesn't allow equal
	 * signs or ampersands
	 */
	private static final Pattern badUriParameterCharPattern =
	        Pattern.compile("((?:[=&\\x00-\\x20\\x7f-\\xff\\{\\}\\\\\"'`^#|\\[\\]<>\\(\\)])|(?:%(?![0-9a-f]{2})))",
	                Pattern.CASE_INSENSITIVE);
	
	private static final int INDEX_DIRECTORY = 3;
	private static final int INDEX_FILE = 4;
	private static final int INDEX_FRAGMENT = 6; // <>#%{}|\^[]`
	private static final int INDEX_HOST = 1;
	
	private static final int INDEX_PORT = 2;
	
	private static final int INDEX_QUERY = 5;
	
	private static final int INDEX_SCHEME = 0;
	
	// http :// host :443 /full/ filename query fragment
	private static Pattern uriPattern =
	        Pattern
	                .compile(
	                        "^(?:([a-z]+)://([a-z0-9\\-\\.]++)(?::(\\d++))?)?+([^\\?#]*/)?+([^\\?/#]*)(?:\\?([^#]*))?(?:#(.*))?$",
	                        Pattern.CASE_INSENSITIVE);
	
	public static String decodeUrl(String url)
	{
		byte[] bytes = url.getBytes();
		
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();
		
		for (int i = 0; i < bytes.length; i++)
		{
			int b = bytes[i];
			if (b == '+')
			{
				buffer.write(' ');
			}
			else if (b == '%')
			{
				int u = -1;
				int l = -1;
				try
				{
					u = Character.digit((char) bytes[i + 1], 16);
					l = Character.digit((char) bytes[i + 2], 16);
					if ((u == -1) || (l == -1))
					{
						throw new DecoderException("Invalid URL encoding");
					}
					buffer.write((char) ((u << 4) + l));
				}
				catch (Exception e)
				{
					if (bytes.length > i)
					{
						buffer.write(bytes[i + 1]);
					}
					
					if (bytes.length > i + 1)
					{
						buffer.write(bytes[i + 2]);
					}
				}
				i += 2;
			}
			else
			{
				buffer.write(b);
			}
		}
		
		return buffer.toString();
	}
	
	/**
	 * Can be used for escaping names or values
	 * 
	 * @param component
	 * @return
	 */
	public static String escapeQueryParameterComponent(String component)
	{
		return escapeString(component, badUriParameterCharPattern);
	}
	
	/**
	 * This forces URI encoding on all non alpha-numeric characters
	 * @param uri
	 * @return
	 */
	public static String escapeAllCharacters(String string)
	{
		string = string.replace("%", "%25");
//		while (true)
//		{
//			Matcher m = nonAlphaNumericNonPercent.matcher(string);
//			if (m.find())
//			{
//				string = string.replace(m.group(1), String.format("%%%02X", m.group(1).codePointAt(0)));
//				continue;
//			}
//			break;
//		}
		
		return escapeUri(string);
	}
	

	
	/**
	 * Escapes out problematic characters in a URL.
	 * 
	 * @param uri
	 * @return
	 */
	public static String escapeUri(String uri)
	{
		String tempUri = escapeString(uri, badUriCharPattern);
		/*
		 * All of this is to escape badNonQueryPattern characters, which are
		 * only = and & now, but I don't think those actually need to be escaped
		 */
		// int queryPosition = tempUri.indexOf('?');
		// String query = "";
		// if (queryPosition > 0)
		// {
		// query = tempUri.substring(queryPosition);
		// }
		// else
		// {
		// queryPosition = tempUri.length();
		// }
		// String nonQuery = tempUri.substring(0, queryPosition);
		// tempUri = escapeString(nonQuery, badNonQueryPattern) + query;
		// if (!tempUri.equals(uri))
		// {
		// System.out.print("changed");
		// }
		return tempUri;
	}
	
	/**
	 * Returns the URI scheme, host, port, path & file. No query or fragment is
	 * included. Forces a trailing slash if it's only a host with no path
	 */
	public static String getBaseUri(String uri)
	{
		int queryStart = uri.indexOf("?");
		if (queryStart < 0)
		{
			queryStart = uri.length();
		}
		int fragmentStart = uri.indexOf("#");
		if (fragmentStart < 0)
		{
			fragmentStart = uri.length();
		}
		
		int end;
		if (queryStart < fragmentStart)
		{
			end = queryStart;
		}
		else
		{
			end = fragmentStart;
		}
		String trailingSlash = "";
		boolean absolute = uri.toLowerCase().startsWith("http");
		
		if ((absolute && (uri.lastIndexOf("/") <= 7)) || (!absolute && (uri.lastIndexOf("/") == -1)))
		{
			trailingSlash = "/";
		}
		return uri.substring(0, end) + trailingSlash;
	}
	
	/**
	 * This only returns the directory, not the host, port, scheme, etc
	 * 
	 * @param uri
	 * @return
	 */
	public static String getDirectory(String uri)
	{
		return parseUriString(uri)[INDEX_DIRECTORY];
	}
	
	/**
	 * Returns the URI scheme, host, port, and path. No file, query or fragment
	 * is included.
	 */
	public static String getDirectoryUri(String uri)
	{
		String base = getBaseUri(uri);
		int lastSlash = base.lastIndexOf("/");
		if (lastSlash < 8)
		{
			lastSlash = uri.length() - 1;
		}
		// String s = base.substring(0, lastSlash + 1);
		return base.substring(0, lastSlash + 1);
	}
	
	public static String getFilename(String uri)
	{
		return parseUriString(uri)[INDEX_FILE];
	}
	
	public static String getFirstQueryParameter(String URI, String parameterName)
	{
		String value = null;
		for (NameValuePair param: getQueryParametersFromUri(URI))
		{
			if (param.getName().equalsIgnoreCase(parameterName))
			{
				value = param.getValue();
				break;
			}
		}
		return value;
	}
	
	public static String getFragment(String uri)
	{
		return parseUriString(uri)[INDEX_FRAGMENT];
	}
	
	public static String getHost(String uri)
	{
		return parseUriString(uri)[INDEX_HOST];
	}
	
	/**
	 * 
	 * @param uri
	 * @return Directory and filename
	 */
	public static String getPath(String uri)
	{
		String components[] = parseUriString(uri);
		return components[INDEX_DIRECTORY];
	}
	
	public static int getPort(String uri)
	{
		String components[] = parseUriString(uri);
		String portString = components[INDEX_PORT];
		if (portString.equals(""))
		{
			if (components[INDEX_SCHEME].equals(""))
			{
				portString = "0";
			}
			else if (components[INDEX_SCHEME].equalsIgnoreCase("https"))
			{
				portString = "443";
			}
			else
			{
				portString = "80";
			}
		}
		return Integer.valueOf(portString);
	}
	
	public static String getQuery(String uri)
	{
		return parseUriString(uri)[INDEX_QUERY];
	}
	
	public static List<NameValuePair> getQueryParametersFromQuery(String query)
	{
		List<NameValuePair> params = new ArrayList<NameValuePair>(1);
		
		if ((query != null) && !query.equals(""))
		{
			String rawPairs[] = query.split("&");
			for (String rawPair: rawPairs)
			{
				String name;
				String value;
				int ampPos = rawPair.indexOf('=');
				if (ampPos >= 0)
				{
					name = rawPair.substring(0, ampPos);
					value = rawPair.substring(ampPos + 1, rawPair.length());
				}
				else
				{
					name = rawPair;
					value = "";
				}
				params.add(new BasicNameValuePair(name, value));
			}
		}
		return params;
	}
	
	public static List<NameValuePair> getQueryParametersFromUri(String URI)
	{
		String query = getQuery(URI);
		return getQueryParametersFromQuery(query);
	}
	
	public static String getScheme(String uri)
	{
		return parseUriString(uri)[INDEX_SCHEME];
	}
	
	/**
	 * Will return something like http://www.grendel-scan.com:80 It doesn't
	 * check to see if this is an absolute URL, so you should check if it could
	 * be relative
	 */
	public static String getSchemeHostPort(String uri)
	{
		String components[] = parseUriString(uri);
		return components[INDEX_SCHEME] + "://" + components[INDEX_HOST] + ":" + components[INDEX_PORT];
	}
	
	/**
	 * Checks to see if a URI is usable from a web point of view. Currently, it
	 * checks for a scheme of http, https or nothing, and a host if there is a
	 * scheme. It also checks for mailto: and javascript: URLs.
	 * 
	 * @param uri
	 * @return
	 */
	public static boolean isUsableUri(String uri)
	{
		boolean usable = false;
		if ((uri != null) && !uri.equals(""))
		{
			String uriLower = uri.toLowerCase();
			if (!(uriLower.startsWith("javascript:") || uriLower.startsWith("mailto:")))
			{
				String components[] = parseUriString(uri);
				if (components[INDEX_SCHEME].equals("")
				        || ((components[INDEX_SCHEME].equalsIgnoreCase("http") || components[INDEX_SCHEME]
				                .equalsIgnoreCase("https")) && !components[INDEX_HOST].equals("")))
				{
					usable = true;
				}
			}
		}
		
		return usable;
	}
	
	/**
	 * Takes the URI and adds a port (if the scheme is known), sorts the
	 * parameter order, removes duplicate paramater names (if this breaks your
	 * stuff, follow the standard (RTFRFC)), removes extra ampersands (&) in the
	 * query, sets the scheme and host to all lower case (path & query can be
	 * case sensitive depending on OS & web server), and removes the fragment
	 * (since it isn't relilvent for requests)
	 * 
	 * @param uri
	 * @return
	 */
	public static String normalizeUri(String uri)
	{
		String components[] = parseUriString(uri);
		if (StringUtils.notEmpty(components[INDEX_SCHEME]) && !StringUtils.notEmpty(components[INDEX_PORT]))
		{
			if (components[INDEX_SCHEME].equalsIgnoreCase("HTTPS"))
			{
				components[INDEX_PORT] = "443";
			}
			else if (components[INDEX_SCHEME].equalsIgnoreCase("HTTP"))
			{
				components[INDEX_PORT] = "80";
			}
		}
		
		String newUri = "";
		if (StringUtils.notEmpty(components[INDEX_SCHEME]))
		{
			newUri += components[INDEX_SCHEME].toLowerCase() + "://";
		}
		
		if (StringUtils.notEmpty(components[INDEX_HOST]))
		{
			newUri += components[INDEX_HOST].toLowerCase();
		}
		
		if (StringUtils.notEmpty(components[INDEX_PORT]))
		{
			newUri += ":" + components[INDEX_PORT];
		}
		
		if (StringUtils.notEmpty(components[INDEX_DIRECTORY]))
		{
			newUri += components[INDEX_DIRECTORY];
		}
		
		if (StringUtils.notEmpty(components[INDEX_FILE]))
		{
			newUri += components[INDEX_FILE];
		}
		
		if (StringUtils.notEmpty(components[INDEX_QUERY]))
		{
			newUri += "?" + normalizeUriQuery(components[INDEX_QUERY]);
		}
		
		return newUri;
	}
	
	public static String normalizeUriQuery(String query)
	{
		String newQuery = query;
		String parameters[] = newQuery.split("&");
		java.util.Arrays.sort(parameters, String.CASE_INSENSITIVE_ORDER);
		newQuery = "";
		Set<String> parameterNames = new HashSet<String>(parameters.length);
		for (String parameter: parameters)
		{
			String name;
			String value;
			
			int eq = parameter.indexOf("=");
			if (eq > 0)
			{
				name = parameter.substring(0, eq);
				value = parameter.substring(eq + 1);
			}
			else
			{
				name = parameter;
				value = "";
			}
			if (parameterNames.contains(name))
			{
				continue;
			}
			newQuery += name + "=" + value + "&";
		}
		
		return newQuery.replaceAll("&{2,}", "&").replaceFirst("^&+", "").replaceFirst("&+$", "");
	}
	
	public static String removeQueryParameter(String uri, String parameterName)
	{
		String newUri = uri;
		String query = getQuery(newUri);
		if (StringUtils.notEmpty(query))
		{
			query = query.replaceAll("(?:&|^)" + Pattern.quote(parameterName) + "=[^=&]+", "").replaceFirst("^&", "");
			newUri = getBaseUri(newUri) + "?" + query;
		}
		
		return newUri;
	}
	
	public static String replaceQueryParameter(String uri, String parameterName, String parameterValue)
	{
		String newUri = uri;
		String query = getQuery(newUri);
		if (StringUtils.notEmpty(query))
		{
			query = query.replaceAll("&?+" + Pattern.quote(parameterName) + "=[^=&+]", parameterValue);
			newUri = getBaseUri(newUri) + "?" + query;
		}
		
		return newUri;
	}
	
	public static String urlEncode(NameValuePair[] pairs)
	{
		return urlEncode(Arrays.asList(pairs));
	}
	
	public static String urlEncode(Iterable<NameValuePair> pairs)
	{
		boolean first = true;
		StringBuilder buf = new StringBuilder();
		for (NameValuePair pair: pairs)
		{
			if (pair.getName() != null)
			{
				if (!first)
				{
					buf.append("&");
				}
				first = false;
				buf.append(escapeQueryParameterComponent(pair.getName()));
				buf.append("=");
				if (pair.getValue() != null)
				{
					buf.append(escapeQueryParameterComponent(pair.getValue()));
				}
			}
		}
		
		return buf.toString();
	}
	
	public static boolean validateURISyntax(String URI, boolean mustBeAbsolute)
	{
		boolean good = true;
		try
		{
			URI uri = UriFactory.makeUri(URI, false);
			if (mustBeAbsolute && !uri.isAbsolute())
			{
				good = false;
			}
		}
		catch (URISyntaxException e)
		{
			good = false;
		}
		
		return good;
	}
	
	private static String escapeString(String string, Pattern p)
	{
		String newString = string;
		if (newString != null)
		{
			Matcher m = p.matcher(newString);
			while (m.find())
			{
				String badchar = m.group(1);
				String firstHalf = newString.substring(0, m.start());
				String lastHalf = newString.substring(m.end());
				newString = firstHalf + String.format("%%%02x", badchar.codePointAt(0)).toUpperCase() + lastHalf;
				m = p.matcher(newString);
			}
		}
		return newString;
	}
	
	private static String[] parseUriString(String uri)
	{
		String components[] = { "", "", "", "", "", "", "" };
		
		Matcher m = uriPattern.matcher(uri);
		if (m.matches())
		{
			components[INDEX_SCHEME] = m.group(1);
			if (components[INDEX_SCHEME] == null)
			{
				components[INDEX_SCHEME] = "";
			}
			
			components[INDEX_HOST] = m.group(2);
			if (components[INDEX_HOST] == null)
			{
				components[INDEX_HOST] = "";
			}
			
			components[INDEX_PORT] = m.group(3);
			if (components[INDEX_PORT] == null)
			{
				components[INDEX_PORT] = "";
			}
			
			components[INDEX_DIRECTORY] = m.group(4);
			if (components[INDEX_DIRECTORY] == null)
			{
				components[INDEX_DIRECTORY] = "";
			}
			
			components[INDEX_FILE] = m.group(5);
			if (components[INDEX_FILE] == null)
			{
				components[INDEX_FILE] = "";
			}
			
			components[INDEX_QUERY] = m.group(6);
			if (components[INDEX_QUERY] == null)
			{
				components[INDEX_QUERY] = "";
			}
			
			components[INDEX_FRAGMENT] = m.group(7);
			if (components[INDEX_FRAGMENT] == null)
			{
				components[INDEX_FRAGMENT] = "";
			}
			
		}
		return components;
	}
}
