/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
/*
 * Created on Jun 6, 2005
 */
package org.cobra_grendel.util;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * @author J. H. S.
 */
public class History implements java.io.Serializable
{
	private class TimedEntry implements Comparable, java.io.Serializable
	{
		private static final long serialVersionUID = 2257845000000000200L;
		private long timestamp = System.currentTimeMillis();
		private final String value;
		
		/**
		 * @param value
		 */
		public TimedEntry(String value)
		{
			this.value = value;
		}
		
		/*
		 * (non-Javadoc)
		 * 
		 * @see java.lang.Comparable#compareTo(java.lang.Object)
		 */
		public int compareTo(Object arg0)
		{
			if (equals(arg0))
			{
				return 0;
			}
			TimedEntry other = (TimedEntry) arg0;
			long time1 = timestamp;
			long time2 = other.timestamp;
			if (time1 > time2)
			{
				// More recent goes first
				return -1;
			}
			else if (time2 > time1)
			{
				return +1;
			}
			else
			{
				int diff = System.identityHashCode(this) - System.identityHashCode(other);
				if (diff == 0)
				{
					return +1;
				}
				return diff;
			}
		}
		
		@Override
		public boolean equals(Object obj)
		{
			TimedEntry other = (TimedEntry) obj;
			return other.value.equals(value);
		}
		
		public void touch()
		{
			timestamp = System.currentTimeMillis();
		}
	}
	
	private static final long serialVersionUID = 2257845000800300100L;
	
	private int commonEntriesCapacity;
	private final Map historyMap = new HashMap();
	private transient ArrayList historySequence;
	
	private final SortedSet historySortedSet = new TreeSet();
	private final SortedSet historyTimedSet = new TreeSet();
	
	private int sequenceCapacity;
	
	private transient int sequenceIndex;
	
	/**
	 * @param sequenceCapacity
	 * @param commonEntriesCapacity
	 */
	public History(int sequenceCapacity, int commonEntriesCapacity)
	{
		super();
		historySequence = new ArrayList();
		sequenceIndex = -1;
		this.sequenceCapacity = sequenceCapacity;
		this.commonEntriesCapacity = commonEntriesCapacity;
	}
	
	public void addAsRecentOnly(String item)
	{
		TimedEntry entry = (TimedEntry) historyMap.get(item);
		if (entry != null)
		{
			historyTimedSet.remove(entry);
			entry.touch();
			historyTimedSet.add(entry);
		}
		else
		{
			entry = new TimedEntry(item);
			historyTimedSet.add(entry);
			historyMap.put(item, entry);
			historySortedSet.add(item);
			if (historyTimedSet.size() > commonEntriesCapacity)
			{
				// Most outdated goes last
				TimedEntry entryToRemove = (TimedEntry) historyTimedSet.last();
				historyMap.remove(entryToRemove.value);
				historySortedSet.remove(entryToRemove.value);
				historyTimedSet.remove(entryToRemove);
			}
		}
	}
	
	public void addItem(String item, boolean updateAsRecent)
	{
		int newIndex = sequenceIndex + 1;
		
		while (newIndex >= historySequence.size())
		{
			historySequence.add(null);
		}
		historySequence.set(newIndex, item);
		sequenceIndex = newIndex;
		
		int expectedSize = newIndex + 1;
		while (historySequence.size() > expectedSize)
		{
			historySequence.remove(expectedSize);
		}
		
		while (historySequence.size() > sequenceCapacity)
		{
			historySequence.remove(0);
			sequenceIndex--;
		}
		
		if (updateAsRecent)
		{
			addAsRecentOnly(item);
		}
	}
	
	public String back()
	{
		if (sequenceIndex > 0)
		{
			sequenceIndex--;
			return getCurrentItem();
		}
		else
		{
			return null;
		}
	}
	
	public String forward()
	{
		if (sequenceIndex + 1 < historySequence.size())
		{
			sequenceIndex++;
			return getCurrentItem();
		}
		else
		{
			return null;
		}
	}
	
	/**
	 * @return Returns the commonEntriesCapacity.
	 */
	public int getCommonEntriesCapacity()
	{
		return commonEntriesCapacity;
	}
	
	public String getCurrentItem()
	{
		if (sequenceIndex >= 0)
		{
			return (String) historySequence.get(sequenceIndex);
		}
		else
		{
			return null;
		}
	}
	
	public Collection getHeadMatchItems(String item, int maxNumItems)
	{
		Object[] array = historySortedSet.toArray();
		int idx = Arrays.binarySearch(array, item);
		int startIdx = idx >= 0 ? idx : (-idx - 1);
		int count = 0;
		Collection items = new LinkedList();
		for (int i = startIdx; (i < array.length) && (count++ < maxNumItems); i++)
		{
			String potentialItem = (String) array[i];
			if (potentialItem.startsWith(item))
			{
				items.add(potentialItem);
			}
			else
			{
				break;
			}
		}
		return items;
	}
	
	public Collection getRecentItems(int maxNumItems)
	{
		Collection items = new LinkedList();
		Iterator i = historyTimedSet.iterator();
		int count = 0;
		while (i.hasNext() && (count++ < maxNumItems))
		{
			TimedEntry entry = (TimedEntry) i.next();
			items.add(entry.value);
		}
		return items;
	}
	
	/**
	 * @return Returns the sequenceCapacity.
	 */
	public int getSequenceCapacity()
	{
		return sequenceCapacity;
	}
	
	/**
	 * @param commonEntriesCapacity
	 *            The commonEntriesCapacity to set.
	 */
	public void setCommonEntriesCapacity(int commonEntriesCapacity)
	{
		this.commonEntriesCapacity = commonEntriesCapacity;
	}
	
	/**
	 * @param sequenceCapacity
	 *            The sequenceCapacity to set.
	 */
	public void setSequenceCapacity(int sequenceCapacity)
	{
		this.sequenceCapacity = sequenceCapacity;
	}
	
	private void readObject(java.io.ObjectInputStream in) throws ClassNotFoundException, java.io.IOException
	{
		historySequence = new ArrayList();
		sequenceIndex = -1;
		in.defaultReadObject();
	}
	
}
