/*
 * GNU LESSER GENERAL PUBLIC LICENSE Copyright (C) 2006 The Lobo Project
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 * 
 * Contact info: xamjadmin@users.sourceforge.net
 */
package org.cobra_grendel.util;

import java.util.HashMap;
import java.util.Map;
import java.util.TreeSet;

/**
 * A cache with least-recently-used policy. Note that this class is not thread
 * safe by itself.
 */
public class LRUCache implements java.io.Serializable
{
	private class OrderedValue implements Comparable, java.io.Serializable
	{
		private static final long serialVersionUID = 340227625744215821L;
		private int approximateSize;
		private long timestamp;
		private Object value;
		
		private OrderedValue(Object value, int approxSize)
		{
			this.value = value;
			approximateSize = approxSize;
			touch();
		}
		
		public int compareTo(Object arg0)
		{
			if (this == arg0)
			{
				return 0;
			}
			OrderedValue other = (OrderedValue) arg0;
			long diff = timestamp - other.timestamp;
			if (diff != 0)
			{
				return (int) diff;
			}
			int hc1 = System.identityHashCode(this);
			int hc2 = System.identityHashCode(other);
			if (hc1 == hc2)
			{
				hc1 = System.identityHashCode(value);
				hc2 = System.identityHashCode(other.value);
			}
			return hc1 - hc2;
		}
		
		private final void touch()
		{
			timestamp = System.currentTimeMillis();
		}
	}
	
	private static final long serialVersionUID = 940427225784212823L;
	private final int approxMaxSize;
	
	private final Map cacheMap = new HashMap();
	private int currentSize = 0;
	
	/**
	 * Ascending timestamp order. First is least recently used.
	 */
	private final TreeSet timedSet = new TreeSet();
	
	public LRUCache(int approxMaxSize)
	{
		this.approxMaxSize = approxMaxSize;
	}
	
	public Object get(Object key)
	{
		OrderedValue ordVal = (OrderedValue) cacheMap.get(key);
		if (ordVal != null)
		{
			timedSet.remove(ordVal);
			ordVal.touch();
			timedSet.add(ordVal);
			return ordVal.value;
		}
		else
		{
			return null;
		}
	}
	
	public void put(Object key, Object value, int approxSize)
	{
		if (approxSize > approxMaxSize)
		{
			throw new IllegalArgumentException("Max size is " + approxMaxSize);
		}
		OrderedValue ordVal = (OrderedValue) cacheMap.get(key);
		if (ordVal != null)
		{
			currentSize += (approxSize - ordVal.approximateSize);
			timedSet.remove(ordVal);
			ordVal.approximateSize = approxSize;
			ordVal.value = value;
			ordVal.touch();
			timedSet.add(ordVal);
		}
		else
		{
			ordVal = new OrderedValue(value, approxSize);
			cacheMap.put(key, ordVal);
			timedSet.add(ordVal);
			currentSize += approxSize;
		}
		while (currentSize > approxMaxSize)
		{
			removeLRU();
		}
	}
	
	private void removeLRU()
	{
		OrderedValue ordVal = (OrderedValue) timedSet.first();
		if (ordVal != null)
		{
			if (timedSet.remove(ordVal))
			{
				currentSize -= ordVal.approximateSize;
			}
			else
			{
				throw new IllegalStateException("Could not remove existing tree node.");
			}
		}
		else
		{
			throw new IllegalStateException("Cannot remove LRU since the cache is empty.");
		}
	}
}
