package org.cobra_grendel.util;

import java.util.HashSet;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * A thread pool that allows cancelling all running tasks without shutting down
 * the thread pool.
 */
public class SimpleThreadPool
{
	private class ThreadRunnable implements Runnable
	{
		public void run()
		{
			Object monitor = taskMonitor;
			LinkedList tl = taskList;
			Set rs = runningSet;
			int iam = idleAliveMillis;
			SimpleThreadPoolTask task = null;
			for (;;)
			{
				try
				{
					synchronized (monitor)
					{
						if (task != null)
						{
							rs.remove(task);
						}
						numIdleThreads++;
						try
						{
							long waitBase = System.currentTimeMillis();
							INNER: while (tl.isEmpty())
							{
								long maxWait = iam - (System.currentTimeMillis() - waitBase);
								if (maxWait <= 0)
								{
									if (numThreads > minThreads)
									{
										// Should be only way to exit thread.
										numThreads--;
										return;
									}
									else
									{
										waitBase = System.currentTimeMillis();
										continue INNER;
									}
								}
								monitor.wait(maxWait);
							}
						}
						finally
						{
							numIdleThreads--;
						}
						task = (SimpleThreadPoolTask) taskList.removeFirst();
						rs.add(task);
					}
					Thread currentThread = Thread.currentThread();
					String baseName = currentThread.getName();
					try
					{
						try
						{
							currentThread.setName(baseName + ":" + task.toString());
						}
						catch (Throwable thrown)
						{
							logger.log(Level.WARNING, "run(): Unable to set task name.", thrown);
						}
						try
						{
							task.run();
						}
						catch (Throwable thrown)
						{
							logger.log(Level.SEVERE, "run(): Error in task: " + task + ".", thrown);
						}
					}
					finally
					{
						currentThread.setName(baseName);
					}
				}
				catch (Throwable thrown)
				{
					logger.log(Level.SEVERE, "run(): Error in thread pool: " + name + ".", thrown);
				}
			}
		}
	}
	
	private static final Logger logger = Logger.getLogger(SimpleThreadPool.class.getName());
	private final int idleAliveMillis;
	private final int maxThreads;
	private final int minThreads;
	private final String name;
	private int numIdleThreads = 0;
	private int numThreads = 0;
	private final Set runningSet = new HashSet();
	private final LinkedList taskList = new LinkedList();
	private final Object taskMonitor = new Object();
	
	private int threadNumber = 0;
	
	public SimpleThreadPool(String name, int minShrinkToThreads, int maxThreads, int idleAliveMillis)
	{
		minThreads = minShrinkToThreads;
		this.maxThreads = maxThreads;
		this.idleAliveMillis = idleAliveMillis;
		this.name = name;
	}
	
	public void cancel(SimpleThreadPoolTask task)
	{
		synchronized (taskMonitor)
		{
			taskList.remove(task);
		}
		task.cancel();
	}
	
	/**
	 * Cancels all waiting tasks and any currently running task.
	 */
	public void cancelAll()
	{
		synchronized (taskMonitor)
		{
			taskList.clear();
			Iterator i = runningSet.iterator();
			while (i.hasNext())
			{
				((SimpleThreadPoolTask) i.next()).cancel();
			}
		}
	}
	
	public void schedule(SimpleThreadPoolTask task)
	{
		if (task == null)
		{
			throw new IllegalArgumentException("null task");
		}
		Object monitor = taskMonitor;
		synchronized (monitor)
		{
			if (numIdleThreads == 0)
			{
				addThreadImpl();
			}
			taskList.add(task);
			monitor.notify();
		}
	}
	
	private void addThreadImpl()
	{
		if (numThreads < maxThreads)
		{
			Thread t = new Thread(new ThreadRunnable(), name + threadNumber++);
			t.setDaemon(true);
			t.start();
			numThreads++;
		}
	}
}
